package com.qls.ali.biz.data.controller;

import cn.hutool.json.JSONArray;
import com.qls.ali.biz.service.fz.Cache;
import com.qls.ali.biz.service.fz.HotelService;
import com.qls.ali.biz.service.mybatis.HtHotelService;
import com.qls.core.dto.PushHotelBody;
import com.qls.core.entity.DtStore;
import com.qls.core.entity.HtAliHotel;
import com.qls.core.entity.HtAliRoom;
import com.qls.core.entity.HtHotel;
import com.qls.core.utils.R;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Objects;


/**
 * @author yunchao
 */
@RestController
@RequestMapping("hotel")
@Slf4j
public class HotelController {


    @Autowired
    private HotelService hotelService;

    @Autowired
    private Cache cache;

    @Autowired
    private HtHotelService htHotelService;

    /**
     * 酒店推送
     *
     * @param pushHotelBody
     * @return
     */
    @PostMapping("push")
    public R push(@RequestBody PushHotelBody pushHotelBody) {
        Integer storeId = pushHotelBody.getStoreId();
        Integer type = pushHotelBody.getType();
        Long hotelId = pushHotelBody.getHotelId();
        DtStore dtStore = cache.getStore(storeId);
        if (Objects.isNull(dtStore)) {
            String msg = String.format("酒店推送，推送类型店铺id：%s，type：%d，酒店id：%d，未设置店铺缓存", type, storeId, hotelId);
            log.warn(msg);
            return R.error(msg);
        }

        String appKey = dtStore.getAppKey();
        String session = dtStore.getSessionKey();
        String secret = dtStore.getSecret();
        // 执行任务类型 1、全量推送 2、只推送酒店 -1、全量删除 -2、只删除酒店
        switch (type) {
            case -1:
                hotelService.delAll(hotelId, storeId, appKey, secret, session);
                break;
            case -2:
                hotelService.hotelDel(hotelId, storeId, appKey, secret, session);
                break;
            case 1:
                hotelService.hotelPushAll(hotelId, storeId, appKey, secret, session);
                break;
            case 2:
                HtHotel htHotel = htHotelService.getById(hotelId);
                if (Objects.isNull(htHotel)) {
                    String msg = String.format("酒店推送，店铺id：%s，推送类型type：%d，酒店id：%d，查询结果为 null", type, storeId, hotelId);
                    log.warn(msg);
                    return R.error(msg);
                }
                hotelService.hotelPush(htHotel, storeId, appKey, secret, session);
                break;
            default:
                String msg = String.format("酒店推送，店铺id：%s，推送类型type：%d，酒店id：%d，推送类型错误", type, storeId, hotelId);
                log.warn(msg);
                return R.error(msg);
        }
        return R.ok();
    }

    /**
     * 获取人工匹配阿里标准酒店列表
     *
     * @param map Map<String, Object>
     * @return R
     */
    @GetMapping( "getMatchHotels")
    public R getMatchHotels(@RequestParam Map<String, Object> map) {
        List<HtAliHotel> list = hotelService.getMatchHotels(map);
        return R.ok().put("list",list);
    }

    /**
     * 人工匹配酒店
     *
     * @param map Map<String, Object>
     * @return R
     */
    @GetMapping( "manualMatchHotel")
    public R manualMatchHotel(@RequestParam Map<String, Object> map) {
        Boolean flag = hotelService.manualMatchHotel(map);
        return R.ok().put("flag",flag);
    }

    /**
     * 获取人工匹配房型列表
     *
     * @param map Map<String, Object>
     * @return R
     */
    @GetMapping( "getMatchRooms")
    public R getMatchRooms(@RequestParam Map<String, Object> map) {
        JSONArray list = hotelService.getMatchRooms(map);
        return R.ok().put("list",list);
    }
    /**
     * 人工匹配房型
     *
     * @param map Map<String, Object>
     * @return R
     */
    @GetMapping( "manualMatchRoom")
    public R manualMatchRoom(@RequestParam Map<String, Object> map) {
        Boolean flag = hotelService.manualMatchRoom(map);
        return R.ok().put("flag",flag);
    }
}
