package com.qls.ali.biz.data.util;

import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;


/**
 * @author doyoung
 */
@Slf4j
public class FzSdk {

    private static String sign(String secret, JSONObject data) {

        Set<String> keySet = data.keySet();
        List<String> keyList = new ArrayList<String>(keySet);
        Collections.sort(keyList);
        StringBuilder sb = new StringBuilder();
        for (String key : keyList) {
            String strKV = key + data.get(key);
            sb.append(strKV);
        }
        return SecureUtil.md5(secret + sb + secret).toUpperCase();
    }

    private static JSONObject commonData(String appKey, String session, String method, JSONObject jsonData) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("method", method);
        jsonObject.put("app_key", appKey);
        jsonObject.put("session", session);
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String date = df.format(new Date());
        jsonObject.put("timestamp", date);
        String format = "json";
        jsonObject.put("format", format);
        String version = "2.0";
        jsonObject.put("v", version);
        String signMethod = "md5";
        jsonObject.put("sign_method", signMethod);
        JSONObject jsonObjectAll = new JSONObject();
        jsonObjectAll.putAll(jsonData);
        jsonObjectAll.putAll(jsonObject);
        return jsonObjectAll;
    }

    /**
     * 接口调用
     *
     * @param method 接口方法
     * @param data   传参数据
     * @param secret 密钥
     * @param appKey appk
     * @param session 会话
     * @return String
     */
    public static String execute(String method, JSONObject data, String secret, String appKey, String session) {
        JSONObject jsonObject = commonData(appKey, session, method, data);
        String sign = sign(secret, jsonObject);
        jsonObject.put("sign", sign);
        String url = "https://eco.taobao.com/router/rest";
        return HttpRequest.post(url).timeout(100000).form(jsonObject).timeout(100000).execute().body();
    }

    /**
     * 失败 重试接口调用(重试 网络等原因失败、飞猪内部服务调用失败、限流等重试)
     *
     * @param method 接口方法
     * @param data   传参数据
     * @param secret 密钥
     * @param appKey appk
     * @param session 会话
     * @param retryMax 重试次数
     * @return String
     */
    public static String retryExecute(String method, JSONObject data, String secret, String appKey, String session, int retryMax) {
        if(retryMax < 0) {
            retryMax = 0;
        }
        int count = 0;
        while (retryMax >= 0) {
            count ++;
            try {
                String result = execute(method, data, secret, appKey, session);
                // 飞猪内部服务调用超时
                if(result.contains("isp.top-remote-connection-timeout")) {
                    log.warn("【飞猪接口调用失败】第{}次调用失败，appk：{}，method：{}，req：{}，resp：{}",  count, appKey, method, data, result);
                    retryMax --;
                }
                // 限流
                else if (result.contains("accesscontrol.limited-by-api-access-count")) {
                    retryMax --;
                    log.warn("【飞猪接口调用失败】第{}次调用失败，appk：{}，method：{}，req：{}，resp：{}",  count, appKey, method, data, result);
                    // 休眠一秒
                    TimeUnit.SECONDS.sleep(1);
                }
                // ISP流控
                else if(result.contains("isp.call-limited")) {
                    retryMax --;
                    log.warn("【飞猪接口调用失败】第{}次调用失败，appk：{}，method：{}，req：{}，resp：{}",  count, appKey, method, data, result);
                    // 休眠一秒
                    TimeUnit.SECONDS.sleep(1);
                }
                // 并发更新等待新增结果超过重试次数
                else if(result.contains("并发更新等待新增结果超过重试次数")) {
                    retryMax --;
                    log.warn("【飞猪接口调用失败】第{}次调用失败，appk：{}，method：{}，req：{}，resp：{}",  count, appKey, method, data, result);
                    // 休眠一秒
                    TimeUnit.SECONDS.sleep(1);
                }
                else {
                    return result;
                }
            } catch (Exception e) {
                log.error("【飞猪接口调用失败】，appk：{}，method：{}，req：{}，e：{}", appKey, method, data, e.toString());
                retryMax --;
            }
        }
        return null;
    }

    /**
     * 失败 重试接口调用 默认重试三次
     *
     * @param method 接口方法
     * @param data   传参数据
     * @param secret 密钥
     * @param appKey appk
     * @param session 会话
     * @return String
     */
    public static String retryExecute(String method, JSONObject data, String secret, String appKey, String session) {
        return retryExecute(method, data, secret, appKey, session, 3);
    }

}
