package com.qls.ali.biz.order.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qls.ali.biz.data.util.FzUtil;
import com.qls.ali.biz.order.entity.OrderEntity;
import com.qls.ali.biz.order.service.biz.OrderNotice;
import com.qls.ali.biz.order.service.biz.ReceOrderService;
import com.qls.ali.biz.order.service.mybatis.DtStoreService;
import com.qls.ali.biz.order.service.mybatis.OrderService;
import com.qls.ali.biz.order.util.OrderConstant;
import com.qls.core.ali.order.deligoods.request.DeliGoods;
import com.qls.core.entity.DtStore;
import com.qls.core.utils.R;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.Objects;

/**
 * 飞猪订单接入
 *
 * @author doyoung
 */
@Slf4j
@RestController
public class OrderController {

    @Autowired
    private OrderNotice orderNotice;

    @Autowired
    private OrderService orderService;

    @Autowired
    private DtStoreService dtStoreService;

    @Autowired
    private ReceOrderService receOrderService;

    /**
     * 阿里订单通知接口
     *
     * @param request  HttpServletRequest
     * @param response HttpServletResponse
     * @throws IOException IOException
     */
    @RequestMapping("order/notice")
    public void aliOrder(HttpServletRequest request, HttpServletResponse response) throws IOException {
        BufferedReader br = new BufferedReader(new InputStreamReader(request.getInputStream(), StandardCharsets.UTF_8));
        StringBuilder sb = new StringBuilder();
        String line;
        while ((line = br.readLine()) != null) {
            sb.append(line);
        }
        String postXmlData = sb.toString();
        String result = orderNotice.router(postXmlData);
        response.setCharacterEncoding("UTF-8");
        response.getWriter().write(result);
    }

    /**
     * 下单渠道回调通知
     *
     * @param data JSONObject
     * @return R
     */
    @RequestMapping(value = "order/callback", method = RequestMethod.POST)
    public R callback(@RequestBody JSONObject data) {
        String buyOrderId = data.getString("buyOrderId");
        int code = data.getInteger("code");
        OrderEntity orderEntity = orderService.getBaseMapper().selectOne(new QueryWrapper<OrderEntity>().eq("buy_order_id", buyOrderId));
        if (Objects.isNull(orderEntity)) {
            log.warn("【美团订单回调通知】美团订单号：{}，数据库不存在此订单", buyOrderId);
            return R.error();
        }
        DtStore dtStore = dtStoreService.getById(orderEntity.getDtStoreId());
        DeliGoods deliGoods = new DeliGoods();
        deliGoods.setTid(orderEntity.getOrderCode());
        // 0预定成功 -1预定失败  1取消成功 2客服介入退款 3已入住
        switch (code) {
            // 预定成功
            case 0:
                log.info("【美团订单回调通知】预定成功！ 店铺：{}，飞猪订单号：{}，美团订单号：{}", dtStore.getName(), orderEntity.getOrderCode(), buyOrderId);
                deliGoods.setOptType(OrderConstant.AliOrderUpdateStatus.Confirm.getValue());
                orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Purchased.getValue());
                orderEntity.setOrderStatus(OrderConstant.AliLocalOrderStatus.Delivered.getValue());
                break;
            //预定失败
            case -1:
                log.info("【美团订单回调通知】预定失败！ 店铺：{}，飞猪订单号：{}，美团订单号：{}", dtStore.getName(), orderEntity.getOrderCode(), buyOrderId);
                orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Purchfailure.getValue());
//                取消预订失败通知飞猪
//                deliGoods.setOptType(OrderConstant.AliOrderUpdateStatus.NonConfirm.getValue());
//                break;
                orderService.updateById(orderEntity);
                return R.ok();
            // 取消成功
            case 1:
                orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Canceled.getValue());
                log.info("【美团订单回调通知】取消成功！ 店铺：{}，飞猪订单号：{}，美团订单号：{}", dtStore.getName(), orderEntity.getOrderCode(), buyOrderId);
                return R.ok();
            // 客服介入退款
            case 2:
                orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Chargebacked.getValue());
                log.info("【美团订单回调通知】美团客服介入退款！ 店铺：{}，飞猪订单号：{}，美团订单号：{}", dtStore.getName(), orderEntity.getOrderCode(), buyOrderId);
                return R.ok();
            // 已入住
            case 3:
                log.info("【美团订单回调通知】客人已入住！ 店铺：{}，飞猪订单号：{}，美团订单号：{}", dtStore.getName(), orderEntity.getOrderCode(), buyOrderId);
                deliGoods.setOptType(OrderConstant.AliOrderUpdateStatus.CheckIn.getValue());
                orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.CheckIn.getValue());
                orderEntity.setOrderStatus(OrderConstant.AliLocalOrderStatus.CheckIn.getValue());
                break;
            default:
                log.info("【美团订单回调通知】未知状态！ 店铺：{}，飞猪订单号：{}，美团订单号：{}", dtStore.getName(), orderEntity.getOrderCode(), buyOrderId);
                break;
        }
        String result = FzUtil.order(deliGoods, dtStore.getAppKey(), dtStore.getSessionKey(), dtStore.getSecret());
        if(result == null || result.contains("error_response")) {
            log.info("【美团订单回调通知】更新飞猪订单状态失败！ 店铺：{}，飞猪订单号：{}，美团订单号：{}", dtStore.getName(), orderEntity.getOrderCode(), buyOrderId);
        }
        orderService.updateById(orderEntity);
        return R.ok();
    }

    /**
     * 手动接口下单
     *
     * @param map Map<String, Object>
     * @return R
     */
    @RequestMapping(value = "order/buy", method = RequestMethod.GET)
    public R buy(@RequestParam Map<String, Object> map) {
        Integer id = Integer.valueOf(map.get("id").toString());
        return receOrderService.manuBook(id);
    }
}
