package com.qls.ali.biz.order.service.biz.impl;


import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.NumberUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.qls.ali.biz.order.entity.*;
import com.qls.ali.biz.order.service.biz.ReceOrderService;
import com.qls.ali.biz.order.service.mybatis.*;
import com.qls.ali.biz.order.util.GeneValidateResponse;
import com.qls.ali.biz.order.util.OrderConstant;
import com.qls.ali.biz.order.util.PriceUtil;
import com.qls.ali.biz.order.util.XmlUtils;
import com.qls.ali.biz.service.fz.HotelService;
import com.qls.core.ali.order.bookrq.request.BookRq;
import com.qls.core.ali.order.bookrq.request.OrderGuest;
import com.qls.core.ali.order.bookrq.response.BookRqResult;
import com.qls.core.ali.order.cancelrq.CancelRq;
import com.qls.core.ali.order.cancelrq.CancelRqResult;
import com.qls.core.ali.order.common.AuthenticationToken;
import com.qls.core.ali.order.common.DailyInfo;
import com.qls.core.ali.order.orderrefundrq.request.OrderRefundRq;
import com.qls.core.ali.order.paysuccessrq.request.PaySuccess;
import com.qls.core.ali.order.paysuccessrq.response.PaySuccessResult;
import com.qls.core.ali.order.querystatusrq.request.QueryStatusRqRequest;
import com.qls.core.ali.order.querystatusrq.response.QueryStatusRqResult;
import com.qls.core.ali.order.validaterq.request.ValidateRq;
import com.qls.core.constant.Constant;
import com.qls.core.dto.HtDailyRoom;
import com.qls.core.dto.PushRatesBody;
import com.qls.core.entity.DtStore;
import com.qls.core.entity.OpOurRateplanPrice;
import com.qls.core.utils.DateUtils;
import com.qls.core.utils.R;
import com.qls.mt.api.MtRemoteService;
import com.qls.rabbitmq.SendMsg;
import com.qls.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author doyoung
 */
@Service
@Slf4j
@SuppressWarnings("unchecked")
public class ReceOrderServiceImpl implements ReceOrderService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private DailyInfoService dailyInfoService;

    @Autowired
    private DailyOriInfoService dailyOriInfoService;

    @Autowired
    private DailyDisInfoService dailyDisInfoService;

    @Autowired
    private GuestService guestService;

    @Autowired
    private OpGroupService opGroupService;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private SendMsg sendMsg;

    @Autowired
    private MtRemoteService mtRemoteService;

    @Autowired
    private DtStoreService dtStoreService;

    @Autowired
    private HotelService hotelService;

    /**
     * 下单前校验
     *
     * @param xml String
     * @return String
     */
    @Override
    public String validate(String xml) {
        ValidateRq validateRq = XmlUtils.fromXml(xml, ValidateRq.class);
        if (Objects.isNull(validateRq)) {
            log.warn("【飞猪验单通知】验单失败，报文解析失败，xml：{}", xml);
            return XmlUtils.toXml(GeneValidateResponse.gene(false, null, "2"));
        }
        String[] strings = validateRq.getRatePlanCode().split("ssj");
        // 缓存价格查询
        List<HtDailyRoom> dailyRoomsCache = redisUtil.getRatesBatch(DateUtils.getBetweenDays(validateRq.getCheckIn(), validateRq.getCheckOut()), strings[0]);
        if (dailyRoomsCache.isEmpty()) {
            log.warn("【飞猪验单通知】验单失败，缓存价格日历缺失，xml：{}", xml);
            return XmlUtils.toXml(GeneValidateResponse.gene(false, null, "-1"));
        }

        // 赋值 下单计算价格用
        List<DailyInfo> dailyInfos = new ArrayList<>();
        for (HtDailyRoom htDailyRoom : dailyRoomsCache) {
            DailyInfo dailyInfo = new DailyInfo();
            dailyInfo.setDay(htDailyRoom.getDate());
            dailyInfo.setPrice(htDailyRoom.getPrice());
            dailyInfo.setSubPrice(htDailyRoom.getSubPrice());
            dailyInfos.add(dailyInfo);
        }
        validateRq.setDailyInfos(dailyInfos);
        validateRq.setRatePlanCode(strings[0]);
        // 掉用验单接口
        R r = mtRemoteService.validate(validateRq);

        // 缓存价格加幅查询
        List<OpOurRateplanPrice> ourRateplanPrices = (List<OpOurRateplanPrice>) redisUtil.hashGet(Constant.Redis.CRPID_USE_CPRICE.getValue(), strings[1]);
        if (ourRateplanPrices.isEmpty()) {
            log.warn("【飞猪验单通知】验单失败，自定义价格计划缺失阶梯加幅，xml：{}", xml);
            return XmlUtils.toXml(GeneValidateResponse.gene(false, null, "-1"));
        }
        // 对缓存价格 进行加幅 返回飞猪比价用
        List<DailyInfo> dailyInfosList = validateRq.getDailyInfos();
        for (DailyInfo dailyInfo : dailyInfosList) {
            dailyInfo.setPrice(PriceUtil.calcPrice(ourRateplanPrices, dailyInfo.getPrice()));
        }
        int code = r.getCode();
//        boolean success = code == 0;
//        boolean b = outOfSavePointV1(validateRq.getAuthenticationToken().getUsername(), success);
//        // 验单成功 正常执行业务逻辑
        if (code == 0) {
            // 接口返回的 价格日历
            List<HtDailyRoom> htDailyRooms = transform((List<Object>) r.get("data"));
            // 价格比较 佣金比较 返回值为0 价格佣金没问题 1 佣金变化 2价格变化
            int flag = this.isUpdate(htDailyRooms, dailyRoomsCache);
            // 佣金变化 或者价格变化 进行更新
            this.cacheAndPush(htDailyRooms, validateRq.getCheckIn(), validateRq.getCheckOut(), strings[0], flag);
            log.info("【飞猪验单通知】验单成功，缓存价格日历：{}，接口价格日历：{}，xml：{}", JSON.toJSON(dailyRoomsCache), JSON.toJSON(htDailyRooms), xml);
            return XmlUtils.toXml(GeneValidateResponse.gene(true, dailyInfosList, "1"));
        }
//        // 验单失败的 判断 失败率 进行执行假验单 还是真实验单
//
//        // 执行真实验单 直接返回验单失败 无库存
//        if(b) {
//            return XmlUtils.toXml(GeneValidateResponse.gene(false, dailyInfosList, "1"));
//        }
        // 执行假验单 进行验单成功
        // 清除库存
        clearInventory(dailyRoomsCache);
        // 更新缓存店铺推送
        this.cacheAndPush(dailyRoomsCache, validateRq.getCheckIn(), validateRq.getCheckOut(), strings[0], 2);
        log.warn("【飞猪验单通知】校验失败，更新库存为零，现缓存价格日历：{}，xml：{}", JSON.toJSON(dailyRoomsCache), xml);
        // 全部返回成功 失败会关房 无法下单
        return XmlUtils.toXml(GeneValidateResponse.gene(true, dailyInfosList, "1"));
    }

    /**
     * v1版本验单规则
     * 验单成功率的安全点
     * 根据店铺用户名 进行统计
     * 半小时统计一次 滑动窗口计算 滑动时间为半小时
     *
     * @param username 店铺用户名
     * @param success  实际验单是否成功
     * @return boolean
     */
    private boolean outOfSavePointV1(String username, boolean success) {
        // 验单失败的key
        String errorKey = username + "errorValidate";
        // 验单总数
        String all = username + "allValidate";
        // 过期时间/s
        long expire = getExpire();
        // 验单总数
        long allCheck = redisUtil.incr(all, 1, expire);
        // 最大错误比率 0.06f 设置为0.05f 防止出现 redis 两条命令 出现并发问题
        float maxErrorRate = 0.05f;
        // 验单失败处理逻辑
        if (!success) {
            // 错误的验单数
            long errorCheck = redisUtil.incr(errorKey, 1, expire);
            // 验单只错一次不会被屏蔽
            if (errorCheck == 1) {
                log.info("【下单前校验】验单失败，失败次数为1，验单正常执行");
                return true;
            }
            // 当验单错误大于1 时 需要计算错误比率 如果错误比率大于等于 百分之6 会被屏蔽 此时需要进行假验单
            float rate = (float) errorCheck / allCheck;
            if (rate < maxErrorRate) {
                log.info("【下单前校验】验单失败，失败次数为：{}，当前半小时验单总次数为：{}，失败比率为：{}，当前失败比率在正常阈值之内，订单正常执行", errorCheck, allCheck, rate);
                return true;
            }
            log.info("【下单前校验】验单失败，失败次数为：{}，当前半小时验单总次数为：{}，失败比率为：{}，当前失败比率超过设定阈值，订单执行假验单", errorCheck, allCheck, rate);
            return false;
        }
        log.info("【下单前校验】验单成功，当前半小时验单总次数为：{}，验单正常执行", allCheck);
        return true;
    }

    /**
     * 获取key的生效时间
     *
     * @return
     */
    private long getExpire() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        // 当前时间/s
        long now = System.currentTimeMillis() / 1000;
        // 当前整点时间/s
        long nowHour = calendar.getTimeInMillis() / 1000;
        // 半小时/s
        long halfHour = 1800;
        // 半小时之后
        long halfHourAfter = nowHour + halfHour;
        // 生效时间
        long expire;
        if (now < halfHourAfter) {
            expire = halfHourAfter - now;
        } else {
            expire = 2 * halfHour - now + nowHour;
        }
        return expire;
    }

//    /**
//     * v2版本验单规则（兜底规则）
//     *
//     * @param username  店铺用户名
//     * @param success   实际验单是否成功
//     * @return boolean
//     */
//    private boolean outOfSavePointV2(String username, boolean success) {
//        // todo
//        return false;
//    }


    /**
     * 数据转换
     *
     * @param objects 价态对象集合
     * @return List<HtDailyRoom>
     */
    private List<HtDailyRoom> transform(List<Object> objects) {
        List<HtDailyRoom> htDailyRooms = new ArrayList<>();
        for (Object object : objects) {
            String jsonObject = JSON.toJSONString(object);
            // 将json转成需要的对象
            HtDailyRoom htDailyRoom = JSONObject.parseObject(jsonObject, HtDailyRoom.class);
            htDailyRooms.add(htDailyRoom);
        }
        return htDailyRooms;
    }

    /**
     * 库存清除
     *
     * @param htDailyRooms 价态集合
     */
    private void clearInventory(List<HtDailyRoom> htDailyRooms) {
        for (HtDailyRoom htDailyRoom : htDailyRooms) {
            htDailyRoom.setSaleNum(0);
        }
    }

    /**
     * 价格和佣金的比较
     *
     * @param htDailyRooms    List<HtDailyRoom>
     * @param dailyRoomsCache List<HtDailyRoom>
     * @return int  0价格佣金一致  1只有佣金变化  2价格变化   （只有佣金变化不需要更新店铺） 只有价格发生变化需要更新店铺
     */
    private int isUpdate(List<HtDailyRoom> htDailyRooms, List<HtDailyRoom> dailyRoomsCache) {
        boolean sub = true;
        for (HtDailyRoom htDailyRoom : htDailyRooms) {
            for (HtDailyRoom dailyRoomCache : dailyRoomsCache) {
                if (htDailyRoom.getDate().equals(dailyRoomCache.getDate())) {
                    if (!htDailyRoom.getPrice().equals(dailyRoomCache.getPrice())) {
                        return 2;
                    }
                    if (!htDailyRoom.getSubPrice().equals(dailyRoomCache.getSubPrice())) {
                        sub = false;
                    }
                    break;
                }
            }
        }
        if (!sub) {
            return 1;
        }
        return 0;
    }

    /**
     * 更新缓存 更新店铺
     *
     * @param htDailyRoomList List<HtDailyRoom>
     * @param rpCode          String
     * @param flag            int
     */
    private void cacheAndPush(List<HtDailyRoom> htDailyRoomList, String checkIn, String checkOut, String rpCode, int flag) {
        if (flag == 0) {
            return;
        }
        // 更新缓存 (无论价格或佣金变化都需要更新缓存)
        redisUtil.saveOrUpdateBatchRates(htDailyRoomList);
        if (flag == 1) {
            log.warn("【飞猪验单通知】佣金发生变化，缓存佣金更新，data：{}", JSON.toJSON(htDailyRoomList));
            return;
        }
        if (flag == 2) {
            log.warn("【飞猪验单通知】价格发生变化，佣金可能发生变化，缓存更新，data：{}", JSON.toJSON(htDailyRoomList));
        }
        // 放入队列进行店铺更新
        Map<String, Object> map = (Map<String, Object>) redisUtil.hashGet(Constant.Redis.RP_BL_STORE.getValue(), rpCode);
        if (map == null) {
            log.warn("【飞猪验单通知】未查询到rp对应的店铺缓存，rpCode：{}，data：{}", rpCode, JSON.toJSON(htDailyRoomList));
            return;
        }
        List<Integer> storeIds = (List<Integer>) map.get("storeIds");
        if (Objects.isNull(storeIds)) {
            log.warn("【飞猪验单通知】未查询到rp对应的店铺缓存，rpCode：{}，缓存：{}，data：{}", rpCode, JSON.toJSON(map), JSON.toJSON(htDailyRoomList));
            return;
        }
        List<HtDailyRoom> htDailyRooms = redisUtil.getRatesBatch(DateUtils.getBetweenDays(checkIn, checkOut), rpCode);
//        Map<Long, List<HtDailyRoom>> listMap = new HashMap<>();
//        listMap.put(Long.valueOf(rpCode), htDailyRooms);
        for (Integer storeId : storeIds) {
//            hotelService.ratesPushBatch(listMap, storeId);
            PushRatesBody pushRatesBody = new PushRatesBody();
            pushRatesBody.setHtDailyRooms(htDailyRooms);
            pushRatesBody.setRpCode(Long.valueOf(rpCode));
            pushRatesBody.setStoreId(storeId);
            sendMsg.execute(Constant.RabbitMq.RATES, storeId, pushRatesBody);
        }
    }

    /**
     * 预定 （没有付款）
     *
     * @param xml String
     * @return String
     */
    @Override
    public String book(String xml, Integer dtId, Integer storeId) {
        BookRq bookRq = XmlUtils.fromXml(xml, BookRq.class);
        BookRqResult bookRqResult = new BookRqResult();
        if (Objects.isNull(bookRq)) {
            bookRqResult.setMessage("请求报文解析错误");
            bookRqResult.setResultCode("-102");
            log.warn("【飞猪预定通知】报文解析错误：{}", xml);
            return XmlUtils.toXml(bookRqResult);
        }
        String ratePlanCode = bookRq.getRatePlanCode();
        String hotelId = bookRq.getHotelId();
        String username = bookRq.getAuthenticationToken().getUsername();
        String rpCode = ratePlanCode.substring(0, ratePlanCode.indexOf("ssj"));
        bookRq.setRatePlanCode(rpCode);
        // 默认订单为人工订单
        Integer orderType = OrderConstant.OrdType.Manu.getValue();
        // 查看数据库是直连订单还是人工订单
        Integer isDirect = opGroupService.getIsDirectByHotelCodeAndUsername(username, Long.valueOf(hotelId));
        // 判断是否直连订单
        if (isDirect.equals(0)) {
            log.info("【飞猪预定通知】人工订单，店铺：{}，酒店：{}，RP：{}，xml：{}", username, hotelId, rpCode, xml);
        } else {
            orderType = OrderConstant.OrdType.Auto.getValue();
            log.info("【飞猪预定通知】直连订单，店铺：{}，酒店：{}，RP：{}，xml：{}", username, hotelId, rpCode, xml);
        }
        // 只保存订单到数据库 当付款通知过来在进行到美团进行下单
        OrderEntity orderEntity = this.sqlSaveOrder(bookRq, OrderConstant.Channel.Mt.getValue(), orderType, OrderConstant.AliLocalOrderStatus.Unpaid.getValue(), dtId, storeId);
        bookRqResult.setOrderId(orderEntity.getId().toString());
        bookRqResult.setMessage("创建订单成功");
        bookRqResult.setResultCode("0");
        return XmlUtils.toXml(bookRqResult);
    }

    /**
     * 数据库保存订单
     *
     * @param bookRq    BookRq
     * @param buySource Integer
     * @param orderType Integer
     * @param dtId      Integer
     * @param storeId   Integer
     * @return OrderEntity
     */
    private OrderEntity sqlSaveOrder(BookRq bookRq, Integer buySource, Integer orderType, Integer paidType, Integer dtId, Integer storeId) {
        OrderEntity orderEntity = new OrderEntity();
        orderEntity.setDtId(dtId);
        orderEntity.setDtStoreId(storeId);
        orderEntity.setBuySource(buySource);
        orderEntity.setOrderType(orderType);
        orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Unpurchased.getValue());
        orderEntity.setOrderStatus(paidType);
        orderEntity.setDate(new Date());
        orderEntity.setOrderCode(Long.valueOf(bookRq.getTaoBaoOrderId()));
        orderEntity.setShid(Long.valueOf(bookRq.getTaoBaoHotelId()));
        orderEntity.setHotelCode(Long.valueOf(bookRq.getHotelId()));
        orderEntity.setSrc(Long.valueOf(bookRq.getTaoBaoRoomTypeId()));
        orderEntity.setRoomCode(Long.valueOf(bookRq.getRoomTypeId()));
        orderEntity.setSrpc(Long.valueOf(bookRq.getTaoBaoRatePlanId()));
        orderEntity.setRateplanCode(Long.valueOf(bookRq.getRatePlanCode()));
        orderEntity.setPlantfrom(bookRq.getChannel());
        orderEntity.setPlanformGoodsId(Long.valueOf(bookRq.getTaoBaoGid()));
        orderEntity.setCheckInDate(DateUtils.stringToDate(bookRq.getCheckIn(), "yyyy-MM-dd"));
        orderEntity.setCheckOutDate(DateUtils.stringToDate(bookRq.getCheckOut(), "yyyy-MM-dd"));
        orderEntity.setEarliestTime(DateUtils.stringToDate(bookRq.getEarliestArriveTime(), "yyyy-MM-dd HH:mm:ss"));
        orderEntity.setLatestTime(DateUtils.stringToDate(bookRq.getLatestArriveTime(), "yyyy-MM-dd HH:mm:ss"));
        orderEntity.setRoomNum(bookRq.getRoomNum());
        orderEntity.setPerNum(Integer.valueOf(bookRq.getOccupancy()));
        orderEntity.setPriceType(Integer.valueOf(bookRq.getPriceType()));
        orderEntity.setInventoryType(Integer.valueOf(bookRq.getInventoryType()));
        orderEntity.setTotalPrice(bookRq.getTotalPrice());
        orderEntity.setOtherFees(bookRq.getOtherFee());
        orderEntity.setPayPrice(bookRq.getPaidPrice());
        orderEntity.setReducedPrice(Integer.valueOf(bookRq.getTotalSellerPromotion()));
        orderEntity.setPaymentType(Integer.valueOf(bookRq.getPaymentType()));
        orderEntity.setName(bookRq.getContactName());
        // 20220907：处理虚拟号码情况，删除'-'及后面数字
        String str = bookRq.getContactTel();
        if (str == null) {
            str = "";
        }
        String[] strings = str.split("-");
        orderEntity.setPhone(strings[0]);
        orderEntity.setEmail(bookRq.getContactEmail());
        orderEntity.setNote(bookRq.getComment());
        orderEntity.setZfbId(bookRq.getAlipayTradeNo());
        orderEntity.setPaymentType(Integer.valueOf(bookRq.getOriginPaymentType()));
        orderService.save(orderEntity);
        List<DailyInfoEntity> dailyInfoEntities = Lists.newArrayList();
        List<DailyOriInfoEntity> dailyOriInfoEntities = Lists.newArrayList();
        List<DailyDisInfoEntity> dailyDisInfoEntities = Lists.newArrayList();
        List<GuestEntity> guestEntities = Lists.newArrayList();
        if (Objects.nonNull(bookRq.getDailyInfos())) {
            for (DailyInfo dailyInfo : bookRq.getDailyInfos()) {
                DailyInfoEntity dailyroomEntity = new DailyInfoEntity();
                dailyroomEntity.setOrderCode(orderEntity.getOrderCode());
                dailyroomEntity.setDate(DateUtils.stringToDate(dailyInfo.getDay(), "yyyy-MM-dd"));
                dailyroomEntity.setPrice(Long.valueOf(dailyInfo.getPrice()));
                dailyroomEntity.setBreakfastNum(dailyInfo.getBreakFast());
                dailyInfoEntities.add(dailyroomEntity);
            }
            dailyInfoService.saveBatch(dailyInfoEntities);
        }
        if (Objects.nonNull(bookRq.getSignMemberDailyInfos())) {
            for (DailyInfo dailyInfo : bookRq.getSignMemberDailyInfos()) {
                DailyOriInfoEntity dailyOriInfoEntity = new DailyOriInfoEntity();
                dailyOriInfoEntity.setOrderCode(orderEntity.getOrderCode());
                dailyOriInfoEntity.setDate(DateUtils.stringToDate(dailyInfo.getDay(), "yyyy-MM-dd"));
                dailyOriInfoEntity.setPrice(Long.valueOf(dailyInfo.getPrice()));
                dailyOriInfoEntity.setBreakfastNum(dailyInfo.getBreakFast());
                dailyOriInfoEntities.add(dailyOriInfoEntity);
            }
            dailyOriInfoService.saveBatch(dailyOriInfoEntities);
        }
        if (Objects.nonNull(bookRq.getDisDailyInfos())) {
            for (DailyInfo dailyInfo : bookRq.getDisDailyInfos()) {
                DailyDisInfoEntity dailyDisInfoEntity = new DailyDisInfoEntity();
                dailyDisInfoEntity.setOrderCode(orderEntity.getOrderCode());
                dailyDisInfoEntity.setDate(DateUtils.stringToDate(dailyInfo.getDay(), "yyyy-MM-dd"));
                dailyDisInfoEntity.setPrice(Long.valueOf(dailyInfo.getPrice()));
                dailyDisInfoEntity.setBreakfastNum(dailyInfo.getBreakFast());
                dailyDisInfoEntities.add(dailyDisInfoEntity);
            }
            dailyDisInfoService.saveBatch(dailyDisInfoEntities);
        }
        if (Objects.nonNull(bookRq.getOrderGuests())) {
            for (OrderGuest guest : bookRq.getOrderGuests()) {
                GuestEntity guestEntity = new GuestEntity();
                guestEntity.setOrderCode(orderEntity.getOrderCode());
                guestEntity.setName(guest.getName());
                guestEntity.setRoomPos(Integer.valueOf(guest.getRoomPos()));
                guestEntity.setIdType(guest.getIdType());
                guestEntity.setIdCode(guest.getIdCode());
                guestEntities.add(guestEntity);
            }
            guestService.saveBatch(guestEntities);
        }
        return orderEntity;
    }

    /**
     * 订单已付款 数据源渠道进行付款
     *
     * @param xml      String
     * @param username String
     * @return String
     */
    @Override
    public String paySuccess(String xml, String username) {
        PaySuccess paySuccess = XmlUtils.fromXml(xml, PaySuccess.class);
        PaySuccessResult paySuccessResult = new PaySuccessResult();
        if (Objects.isNull(paySuccess)) {
            log.warn("【飞猪付款通知】店铺：{}，付款时报文解析失败，xml：{}", username, xml);
            paySuccessResult.setResultCode("-400");
            paySuccessResult.setMessage("报文解析异常");
            return XmlUtils.toXml(paySuccessResult);
        }
        // 数据库查询订单详情
        OrderEntity orderEntity = orderService.getById(paySuccess.getOrderId());
        orderEntity.setOrderStatus(OrderConstant.AliLocalOrderStatus.Paid.getValue());
        orderEntity.setZfbId(paySuccess.getAlipayTradeNo());
        // 入住日期
        String checkIn = DateUtils.format(orderEntity.getCheckInDate());
        // 离店日期
        String checkOut = DateUtils.format(orderEntity.getCheckOutDate());
        // 查询缓存 价格日历
        List<HtDailyRoom> htDailyRooms = redisUtil.getRatesBatch(DateUtils.getBetweenDays(checkIn, checkOut), orderEntity.getRateplanCode().toString());
        if (Objects.isNull(htDailyRooms)) {
            log.warn("【飞猪付款通知】店铺：{}，飞猪订单号：{}，订单付款时查询缓存价格失败", username, paySuccess.getTaoBaoOrderId());
            paySuccessResult.setResultCode("-400");
            paySuccessResult.setMessage("未查询到价格缓存");
            return XmlUtils.toXml(paySuccessResult);
        }
        // 总价
        int amount = 0;
        // 佣金总和
        int sub = 0;
        // 单间价格佣金总和计算
        for (HtDailyRoom htDailyRoom : htDailyRooms) {
            amount += htDailyRoom.getPrice();
            sub += htDailyRoom.getSubPrice();
        }
        Integer roomNum = orderEntity.getRoomNum();
        // 总价
        int payment = (amount - sub) * roomNum;
        orderEntity.setPayPrice(paySuccess.getPayment());
        orderEntity.setBuyOrderPrice(payment);
        orderService.updateById(orderEntity);
        // 计算毛利
        Double grossProfit;
        Double num = 0.9;
        Double mul = NumberUtil.mul(Convert.toDouble(orderEntity.getPayPrice()), num);
        grossProfit = NumberUtil.sub(mul, Convert.toDouble(orderEntity.getBuyOrderPrice()));
        // 判断是否直连
        Integer isDirect = opGroupService.getIsDirectByHotelCodeAndUsername(username, orderEntity.getHotelCode());
        if (isDirect.equals(0)) {
            log.info("【飞猪付款通知】人工订单，店铺：{}，飞猪订单号：{}，xml：{}", username, paySuccess.getTaoBaoOrderId(), xml);
            // 更新数据库状态 正在预定中（采购中），人工手动接口下单或者其他渠道进行下单
            orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Purchasing.getValue());
            // 人工订单判断是否为负毛利，负毛利则添加提示
            if (grossProfit < 0D){
                orderEntity.setNote("注意！人工订单，毛利小于0。");
            }
            orderService.updateById(orderEntity);
            paySuccessResult.setResultCode("0");
            paySuccessResult.setMessage("成功");
            return XmlUtils.toXml(paySuccessResult);
        }
        // 直连订单判断是否为负毛利，负毛利则取消直连下单
        if (grossProfit < 0D) {
            log.info("【飞猪付款通知】直连订单，毛利小于0，取消直连，店铺：{}，飞猪订单号：{}，xml：{}", username, paySuccess.getTaoBaoOrderId(), xml);
            // 更新数据库状态  采购失败，人工手动接口下单或者其他渠道进行下单
            orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Purchfailure.getValue());
            orderEntity.setNote("注意！直连订单，毛利小于0，取消直连，请人工处理。");
            orderService.updateById(orderEntity);
            paySuccessResult.setResultCode("0");
            paySuccessResult.setMessage("成功");
            return XmlUtils.toXml(paySuccessResult);
        }

        BookRq bookRq = this.transForm(orderEntity, htDailyRooms, username);
        R r = mtRemoteService.book(bookRq);
        int code = r.getCode();
        if (code == 0) {
            log.info("【飞猪付款通知】直连订单，支付成功，店铺：{}，飞猪订单号：{}，xml：{}", username, paySuccess.getTaoBaoOrderId(), xml);
            orderEntity.setBuyOrderId(r.get("buyOrderId").toString());
            orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Purchasing.getValue());
            orderService.updateById(orderEntity);
            paySuccessResult.setResultCode("0");
            paySuccessResult.setMessage("成功");
            return XmlUtils.toXml(paySuccessResult);
        }
        log.warn("【飞猪付款通知】直连订单，支付失败，店铺：{}，飞猪订单号：{}，xml：{}", username, paySuccess.getTaoBaoOrderId(), xml);
        orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Purchfailure.getValue());
        orderService.updateById(orderEntity);
        paySuccessResult.setResultCode("支付失败");
        paySuccessResult.setMessage((String) r.get("message"));
        return XmlUtils.toXml(paySuccessResult);
    }

    /**
     * 查询订单状态
     *
     * @param xml String
     * @return String
     */
    @Override
    public String queryStatus(String xml) {
        QueryStatusRqResult queryStatusRqResult = new QueryStatusRqResult();
        QueryStatusRqRequest queryStatusRqRequest = XmlUtils.fromXml(xml, QueryStatusRqRequest.class);
        if (Objects.isNull(queryStatusRqRequest)) {
            log.warn("【飞猪订单查询】查询订单报文解析异常，xml：{}", xml);
            queryStatusRqResult.setMessage("报文解析异常");
            queryStatusRqResult.setResultCode("-301");
            return XmlUtils.toXml(queryStatusRqResult);
        }
        String orderCode = queryStatusRqRequest.getTaoBaoOrderId();
        OrderEntity orderEntity = orderService.sqlSelectByOrderCode(Long.valueOf(orderCode));
        // 未付款的订单不在库中 查询会失败（假数据返回）
        if (orderEntity == null) {
            log.warn("【飞猪订单查询】订单不存在，xml：{}", xml);
            queryStatusRqResult.setMessage("处理成功");
            queryStatusRqResult.setResultCode("0");
            queryStatusRqResult.setStatus("-302");
            queryStatusRqResult.setOrderId(queryStatusRqRequest.getOrderId());
            return XmlUtils.toXml(queryStatusRqResult);
        }
        queryStatusRqResult.setMessage("处理成功");
        queryStatusRqResult.setResultCode("0");
        queryStatusRqResult.setTaoBaoOrderId(orderCode);
        queryStatusRqResult.setOrderId(orderEntity.getId().toString());
        OrderConstant.MtLocalOrdBuyStatus mtLocalOrdBuyStatus = OrderConstant.MtLocalOrdBuyStatus.getLocalOrdBuyStatusByValue(orderEntity.getBuyStatus());
        switch (mtLocalOrdBuyStatus) {
            case Purchased:
                if (orderEntity.getOtherFees() != null) {
                    queryStatusRqResult.setOtherFee(orderEntity.getOtherFees().toString());
                } else {
                    queryStatusRqResult.setOtherFee("0");
                }
                queryStatusRqResult.setTotalRoomPrice(orderEntity.getBuyOrderPrice().toString());
                queryStatusRqResult.setStatus(OrderConstant.AliQueryStatus.Confirm.getValue().toString());
                break;
            // 采购失败
            case Purchfailure:
                // 返回飞猪订单处理中，人工后台系统查询“采购失败”订单电话联系客人退款
                queryStatusRqResult.setStatus(OrderConstant.AliQueryStatus.Dealing.getValue().toString());
                break;
            // 已取消
            case Canceled:
                queryStatusRqResult.setStatus(OrderConstant.AliQueryStatus.Canceled.getValue().toString());
                break;
            // 已入住
            case CheckIn:
                queryStatusRqResult.setStatus(OrderConstant.AliQueryStatus.CheckIn.getValue().toString());
                break;
            // 未采购
            // 已受理
            // 采购中
            // 退款中
            // 已退款
            // 退款失败
            // 订单取消中
            // 取消失败
            // 已采购
            default:
                queryStatusRqResult.setStatus(OrderConstant.AliQueryStatus.Dealing.getValue().toString());
                break;
        }
        return XmlUtils.toXml(queryStatusRqResult);
    }

    /**
     * 申请退款通知
     *
     * @param xml String
     * @return String
     */
    @Override
    public String orderRefund(String xml) {
        OrderRefundRq orderRefundRq = XmlUtils.fromXml(xml, OrderRefundRq.class);
        CancelRqResult cancelRqResult = new CancelRqResult();
        if (Objects.isNull(orderRefundRq)) {
            log.warn("【飞猪订单退款通知】报文解析异常，xml：{}", xml);
            cancelRqResult.setMessage("报文解析异常");
            cancelRqResult.setResultCode("-200");
            return XmlUtils.toXml(cancelRqResult);
        }
        String orderCode = orderRefundRq.getTaoBaoOrderId();
        if (StringUtils.isBlank(orderCode)) {
            log.warn("【飞猪订单退款通知】订单号不存在，xml：{}", xml);
            cancelRqResult.setMessage("订单号不存在");
            cancelRqResult.setResultCode("-204");
            return XmlUtils.toXml(cancelRqResult);
        }
        OrderEntity orderEntity = orderService.sqlSelectByOrderCode(Long.valueOf(orderCode));
        //查不到数据返回成功
        if (Objects.isNull(orderEntity)) {
            log.warn("【飞猪订单退款通知】系统未查询到此订单，xml：{}", xml);
            cancelRqResult.setMessage("不存在此订单");
            cancelRqResult.setResultCode("-204");
            return XmlUtils.toXml(cancelRqResult);
        }
        log.info("【飞猪订单退款通知】xml：{}", xml);
        // 申请退款只是通知 不需要其他操作 只改数据库状态
        cancelRqResult.setOrderId(orderEntity.getId().toString());
        cancelRqResult.setResultCode("0");
        orderEntity.setOrderStatus(OrderConstant.AliLocalOrderStatus.ApplyRefund.getValue());
        orderService.updateById(orderEntity);
        return XmlUtils.toXml(cancelRqResult);
    }

    /**
     * 取消订单 重点（判断下单渠道 是否直连订单 或其他渠道下单，这两种情况可能会出现订单号不存在的情况导致订单取消失败）
     *
     * @param xml String
     * @return String
     */
    @Override
    public String cancel(String xml) {
        CancelRq cancelRq = XmlUtils.fromXml(xml, CancelRq.class);
        CancelRqResult cancelRqResult = new CancelRqResult();
        if (Objects.isNull(cancelRq)) {
            log.warn("【飞猪取消订单通知】报文解析异常，xml：{}", xml);
            cancelRqResult.setMessage("取消订单-报文解析异常");
            cancelRqResult.setResultCode("-200");
            return XmlUtils.toXml(cancelRqResult);
        }
        OrderEntity orderEntity = orderService.getById(cancelRq.getOrderId());
        //查不到数据为空则返回成功
        if (Objects.isNull(orderEntity)) {
            log.warn("【飞猪取消订单通知】系统查无此单，xml：{}", xml);
            cancelRqResult.setMessage("不存在此订单");
            cancelRqResult.setResultCode("-204");
            return XmlUtils.toXml(cancelRqResult);
        }
        // 未购买订单直接取消成功
        String buyOrderId = orderEntity.getBuyOrderId();
        if (Objects.isNull(buyOrderId)) {
            log.info("【飞猪取消订单通知】订单未支付，取消成功，xml：{}", xml);
            cancelRqResult.setOrderId(cancelRq.getOrderId().toString());
            cancelRqResult.setMessage("取消成功");
            cancelRqResult.setResultCode("0");
            return XmlUtils.toXml(cancelRqResult);
        }
        // 已购买的订单判断购买渠道是否调用接口取消
        Integer buySources = orderEntity.getBuySource();
        // 默认订单取消中
        orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Canceling.getValue());
        cancelRqResult.setOrderId(cancelRq.getOrderId().toString());
        // 3 是人工线下采购 不能从接口取消订单 需要人工取消订单
        if (buySources.equals(3)) {
            log.info("【飞猪取消订单通知】人工线下采购订单，订单取消中，xml：{}", xml);
            cancelRqResult.setMessage("订单取消中");
            cancelRqResult.setResultCode("0");
            return XmlUtils.toXml(cancelRqResult);
        }
        // 接口取消订单
        cancelRq.setBuyOrderCode(orderEntity.getBuyOrderId());
        orderEntity.setOrderStatus(OrderConstant.AliLocalOrderStatus.Calcel.getValue());
        R r = mtRemoteService.cancel(cancelRq);
        if (r.getCode() == 0) {
            log.info("【飞猪取消订单通知】接口取消订单，取消成功，xml：{}", xml);
            orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Canceled.getValue());
            orderService.updateById(orderEntity);
            cancelRqResult.setMessage("取消成功");
            cancelRqResult.setResultCode("0");
            return XmlUtils.toXml(cancelRqResult);
        }
        if (r.getCode() == 1) {
            log.info("【飞猪取消订单通知】接口取消订单，订单取消中，xml：{}", xml);
            cancelRqResult.setMessage("订单取消中");
            cancelRqResult.setResultCode("0");
            return XmlUtils.toXml(cancelRqResult);
        }
        log.info("【飞猪取消订单通知】接口取消订单，订单取消失败，xml：{}", xml);
        // 订单取消失败
        orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.CancelFail.getValue());
        orderService.updateById(orderEntity);
        cancelRqResult.setMessage("取消失败，请联系客服进行取消");
        cancelRqResult.setResultCode("-200");
        cancelRqResult.setMessage((String) r.get("message"));
        return XmlUtils.toXml(cancelRqResult);
    }

    @Override
    public R manuBook(Integer orderId) {
        OrderEntity orderEntity = orderService.getById(orderId);
        DtStore dtStore = dtStoreService.getById(orderEntity.getDtStoreId());
        List<HtDailyRoom> htDailyRooms = redisUtil.getRatesBatch(DateUtils.getBetweenDays(DateUtils.format(orderEntity.getCheckInDate()), DateUtils.format(orderEntity.getCheckOutDate())), orderEntity.getRateplanCode().toString());
        BookRq bookRq = this.transForm(orderEntity, htDailyRooms, dtStore.getName());
        R r = mtRemoteService.book(bookRq);
        int code = r.getCode();
        if (code != 0) {
            log.warn("【人工下单】支付失败，酒店：{}，rp:{}，订单号：{}", orderEntity.getHotelCode(), orderEntity.getRateplanCode(), orderEntity.getOrderCode());
            orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Purchfailure.getValue());
            orderService.updateById(orderEntity);
            return R.error("下单失败");
        }
        orderEntity.setBuyOrderId(r.get("buyOrderId").toString());
        log.warn("【人工下单】支付成功，酒店：{}，rp:{}，订单号：{}", orderEntity.getHotelCode(), orderEntity.getRateplanCode(), orderEntity.getOrderCode());
        orderEntity.setBuyStatus(OrderConstant.MtLocalOrdBuyStatus.Purchasing.getValue());
        orderService.updateById(orderEntity);
        return R.ok("下单成功");
    }

    /**
     * 订单转换成 BookRq
     *
     * @param orderEntity OrderEntity
     * @return BookRq
     */
    private BookRq transForm(OrderEntity orderEntity, List<HtDailyRoom> htDailyRooms, String username) {
        List<DailyInfo> dailyInfos = new ArrayList<>();
        for (HtDailyRoom htDailyRoom : htDailyRooms) {
            DailyInfo dailyInfo = new DailyInfo();
            dailyInfo.setPrice(htDailyRoom.getPrice());
            dailyInfo.setSubPrice(htDailyRoom.getSubPrice());
            dailyInfo.setDay(htDailyRoom.getDate());
            dailyInfos.add(dailyInfo);
        }
        BookRq bookRq = new BookRq();
        bookRq.setDailyInfos(dailyInfos);
        bookRq.setHotelId(orderEntity.getHotelCode().toString());
        bookRq.setTaoBaoOrderId(orderEntity.getOrderCode().toString());
        bookRq.setCheckIn(DateUtils.format(orderEntity.getCheckInDate()));
        bookRq.setCheckOut(DateUtils.format(orderEntity.getCheckOutDate()));
        bookRq.setRoomNum(orderEntity.getRoomNum());
        bookRq.setContactTel(orderEntity.getPhone());
        bookRq.setRatePlanCode(orderEntity.getRateplanCode().toString());
        bookRq.setLatestArriveTime(DateUtils.format(orderEntity.getLatestTime(), DateUtils.DATE_TIME_PATTERN));
        AuthenticationToken authenticationToken = new AuthenticationToken();
        authenticationToken.setUsername(username);
        bookRq.setAuthenticationToken(authenticationToken);
        // 入住人
        Map<String, Object> map = new HashMap<>();
        map.put("order_code", orderEntity.getOrderCode());
        List<GuestEntity> guestEntities = guestService.listByMap(map);
        List<OrderGuest> orderGuests = new ArrayList<>();
        for (GuestEntity guestEntity : guestEntities) {
            OrderGuest orderGuest = new OrderGuest();
            orderGuest.setName(guestEntity.getName());
            orderGuests.add(orderGuest);
        }
        bookRq.setOrderGuests(orderGuests);
        bookRq.setContactName(orderEntity.getName());
        return bookRq;
    }

}
