package com.qls.ali.biz.order.util;

import com.thoughtworks.xstream.XStream;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * XML工具类
 *
 * @author doyoung
 */
public class XmlUtils {

    private final static String XML_DECLARATION = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";

    private XmlUtils() {
    }

    /**
     * 序列化XML
     *
     * @param obj Object
     * @return String
     */
    public static <T> String toXml(Object obj) {
        XStream stream = getXStream();
        XStream.setupDefaultSecurity(stream);
        stream.allowTypesByRegExp(new String[]{"com.xc.hotelmgr.order.modules.*"});
        stream.processAnnotations(obj.getClass());
        return XML_DECLARATION + stream.toXML(obj);
    }

    /**
     * 反序列化XML
     *
     * @param xmlStr String
     * @param clazz  Class<T>
     * @return T
     */
    public static <T> T fromXml(String xmlStr, Class<T> clazz) {
        XStream stream = getXStream();
        XStream.setupDefaultSecurity(stream);
        stream.allowTypesByRegExp(new String[]{"com.qls.core.ali.order.*"});
        stream.ignoreUnknownElements();
        stream.processAnnotations(clazz);
        Object obj = stream.fromXML(xmlStr);
        return clazz.cast(obj);
//        try {
//            return clazz.cast(obj);
//        } catch (ClassCastException e) {
//            return null;
//        }
    }

    /**
     * 获取指定节点的值
     *
     * @param xpath   String
     * @param dataStr String
     * @return String
     */
    public static String getNodeValue(String xpath, String dataStr) {
        try {
            // 将字符串转为xml
            Document document = DocumentHelper.parseText(dataStr);
            // 查找节点
            Element element = (Element) document.selectSingleNode(xpath);
            if (element != null) {
                return element.getStringValue();
            }
        } catch (DocumentException e) {
            e.printStackTrace();
        }
        return "";
    }

    /**
     * 获取Xstream实例
     *
     * @return XStream
     */
    private static XStream getXStream() {
        return new XStream();
    }
}