package com.qls.ali.biz.service.fz;


import cn.hutool.json.JSONArray;
import com.qls.ali.biz.data.vo.HtRoomTypeVO;
import com.qls.ali.model.vo.OpOurRateplanVo;
import com.qls.core.dto.HtDailyRoom;
import com.qls.core.entity.HtAliHotel;
import com.qls.core.entity.HtHotel;
import com.qls.core.entity.HtSellRoom;

import java.util.List;
import java.util.Map;

/**
 * 酒店操作接口
 *
 * @author doyoung
 */
public interface HotelService {

    /**
     * 酒店的全量推送
     *
     * @param hotelCode Long
     * @param storeId Integer
     * @param appKey  String
     * @param secret  String
     * @param session String
     */
    void hotelPushAll(Long hotelCode, Integer storeId, String appKey, String secret, String session);

    /**
     * 推送酒店
     *
     * @param htHotel HtHotel
     * @param storeId Integer
     * @param appKey  String
     * @param secret  String
     * @param session String
     * @return List<Long> 推送成功的酒店集合
     */
    List<Long> hotelPush(HtHotel htHotel, Integer storeId, String appKey, String secret, String session);

    /**
     * 酒店的删除(酒店 房型 rp rates)
     *
     * @param hotelCode Long
     * @param storeId   Integer
     * @param appKey    String
     * @param secret    String
     * @param session   String
     */
    void delAll(Long hotelCode, Integer storeId, String appKey, String secret, String session);

    /**
     * 酒店删除
     *
     * @param hotelCode Long
     * @param storeId   Integer
     * @param appKey    String
     * @param secret    String
     * @param session   String
     * @return boolean
     */
    boolean hotelDel(Long hotelCode, Integer storeId, String appKey, String secret, String session);

    /**
     * 房型推送
     *
     * @param hotelCodes List<Long>
     * @param storeId
     * @param htPhysicalRooms List<HtPhysicalRoom>
     * @param appKey          String
     * @param secret          String
     * @param session         String
     * @return List<Long> 房型编码集合
     */
    List<HtSellRoom> roomTypePushBatch(List<Long> hotelCodes, Integer storeId, List<HtRoomTypeVO> htPhysicalRooms, String appKey, String secret, String session);

    /**
     * rp推送(批量)
     *
     * @param htSellRooms    List<HtLogicRoom>
     * @param storeId        Integer
     * @param appKey         String
     * @param secret         String
     * @param session        String
     * @param opOurRateplans List<OpOurRateplan>
     * @param partnerId 美团渠道的标识
     * @return Map<String, List < Integer>> Map<Long, List < Integer>> rpId -> [自定义rpId]
     */
    Map<String, List<Integer>> rpPushBatch(List<HtSellRoom> htSellRooms, Integer storeId, String appKey, String secret, String session, List<OpOurRateplanVo> opOurRateplans, Integer partnerId);

    /**
     * rp删除
     *
     * @param rpCode  Long
     * @param crpIds  List<Integer>  为空时删除此酒店下的所有rp  否则删除指定的rp
     * @param storeId Integer
     * @param appKey  String
     * @param secret  String
     * @param session String
     * @return boolean
     */
    boolean rpDel(Long rpCode, List<Integer> crpIds, Integer storeId, String appKey, String secret, String session);

    /**
     * 更新或增加
     *
     * @param rpCode  Long
     * @param crpIds  List<Integer>
     * @param storeId Integer
     * @param appKey  String
     * @param secret  String
     * @param session String
     * @param partnerId 美团渠道的标识
     */
    void rpPush(Long rpCode, List<Integer> crpIds, Integer storeId, String appKey, String secret, String session, Integer partnerId);

    /**
     * 批量增量价态推送
     *
     * @param map     Map<Long, List<String>> k rpCode v 日期集合
     * @param storeId 店铺主键
     */
    void ratesPushBatch(Map<Long, List<HtDailyRoom>> map, Integer storeId);

    /**
     * 获取人工匹配阿里标准酒店列表
     *
     * @param map Map<String, Object>
     */
    List<HtAliHotel> getMatchHotels(Map<String, Object> map);

    /**
     * 人工匹配酒店
     *
     * @param map Map<String, Object>
     */
    Boolean manualMatchHotel(Map<String, Object> map);

    /**
     * 获取人工匹配房型列表
     *
     * @param map Map<String, Object>
     * @return
     */
    JSONArray getMatchRooms(Map<String, Object> map);

    /**
     * 人工匹配房型
     *
     * @param map Map<String, Object>
     */
    Boolean manualMatchRoom(Map<String, Object> map);

}
