package com.qls.ali.biz.service.fz.impl;


import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qls.ali.biz.data.dao.HtHotelDao;
import com.qls.ali.biz.data.util.FzUtil;
import com.qls.ali.biz.data.util.ScoreSuggester;
import com.qls.ali.biz.data.vo.HtHotelVO;
import com.qls.ali.biz.data.vo.HtRoomTypeVO;
import com.qls.ali.biz.service.fz.Cache;
import com.qls.ali.biz.service.fz.HotelService;
import com.qls.ali.biz.service.mybatis.*;
import com.qls.ali.biz.service.mybatis.impl.HtHotelServiceImpl;
import com.qls.ali.model.vo.OpOurRateplanVo;
import com.qls.core.constant.Constant;
import com.qls.core.dto.HtDailyRoom;
import com.qls.core.dto.PushHotelBody;
import com.qls.core.dto.PushRatesBody;
import com.qls.core.entity.*;
import com.qls.core.utils.DateUtils;
import com.qls.core.utils.MapUtil;
import com.qls.rabbitmq.SendMsg;
import com.qls.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;


/**
 * @author yunchao
 */
@Slf4j
@Service
@SuppressWarnings("unchecked")
public class HotelServiceImpl implements HotelService {

    @Autowired
    private OpHtlGroupService opHtlGroupService;

    @Autowired
    private HtSellRoomService htSellRoomService;

    @Autowired
    private HtAliHotelService htAliHotelService;

    @Autowired
    private HtHotelServiceImpl htHotelService;

    @Autowired
    private HtAliRoomService htAliRoomService;

    @Autowired
    private HtPhysicalRoomService htPhysicalRoomService;


    @Autowired
    private OpOurRateplanService opOurRateplanService;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private SendMsg sendMsg;

    @Autowired
    private Cache cache;

    @Autowired
    private HtHotelDao htHotelDao;

    @Override
    public void hotelPushAll(Long hotelCode, Integer storeId, String appKey, String secret, String session) {

        // 查询自定义rp及 自定义rp的价格加幅
        List<OpOurRateplanVo> opOurRateplans = opOurRateplanService.getRatePlanPrice(storeId, hotelCode);
        // 推送必须建立价格政策 没有建立价格政策不进行推送
        if (opOurRateplans.isEmpty()) {
            log.warn("【酒店全量推送】店铺id：{}，酒店id{}：请配置分组自定义价格政策和价格加幅", storeId, hotelCode);
            return;
        }
        HtHotelVO htHotel = htHotelDao.getHotelMapping(hotelCode);
        if (htHotel == null) {
            log.warn("【酒店全量推送】关联查询结果为 null");
            return;
        }
        Integer partnerId = htHotel.getPartnerId();

        // 推送酒店-----------------------------------------------------------------------------------------------------
        List<Long> hotelCodes = this.hotelPush(htHotel, storeId, appKey, secret, session);
        if (hotelCodes.isEmpty()) {
            return;
        }

        // 推送物理房型-----------------------------------------------------------------------------------------------------
        List<HtSellRoom> htLogicRooms = roomTypePushBatch(hotelCodes, storeId, htHotel.getHtRoomTypes(), appKey, secret, session);
        // 如果查询不到 结束推送
        if (htLogicRooms.isEmpty()) {
            return;
        }

        // 推送rp----------------------------------------------------------------------------------------------------
        // 推送 （重点 包含缓存处理）
        Map<String, List<Integer>> stringListMap = rpPushBatch(htLogicRooms, storeId, appKey, secret, session, opOurRateplans, partnerId);

        if (stringListMap.isEmpty()) {
            return;
        }

        // 推送价格的map
        Map<Long, List<HtDailyRoom>> map = new HashMap<>(10);
        for (String k : stringListMap.keySet()) {
            String[] newStr = k.split("_");
            String rpCode = newStr[0];
            // 缓存获取价格
            map.put(Long.valueOf(rpCode), redisUtil.getRatesBatch(DateUtils.format(new Date()), 30, rpCode));
        }
        // 推送价格 放入队列自动消费推送
        for (Long k : map.keySet()) {
            Map<String, Object> globFilterCache = cache.getGlobFilterCache(k.toString());
            List<Integer> storeIds = (List<Integer>) globFilterCache.get("storeIds");
            for (Integer id : storeIds) {
                PushRatesBody pushRatesBody = new PushRatesBody();
                pushRatesBody.setHtDailyRooms(map.get(k));
                pushRatesBody.setRpCode(k);
                sendMsg.execute(Constant.RabbitMq.RATES, id, pushRatesBody);
            }
        }
    }

    @Override
    public List<Long> hotelPush(HtHotel htHotelinfo, Integer storeId, String appKey, String secret, String session) {
        String ok = "xhotel_update_response";
        List<Long> hotelCodes = new ArrayList<>();
        //  获取飞猪标准酒店
        QueryWrapper<HtAliHotel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("hotel_code", htHotelinfo.getHotelCode()).eq("store_id", storeId);
        HtAliHotel htAliHotel = htAliHotelService.getOne(queryWrapper);
        if (htAliHotel == null) {
            htAliHotel = getMatchHotel(htHotelinfo, appKey, session, secret);
            htAliHotel.setStoreId(storeId);
        }
        if (htAliHotel == null) {
            log.warn("飞猪标准酒店 ({})：自动匹配失败", htHotelinfo.getHotelCode());
            return hotelCodes;
        }
        //  获取飞猪标准酒店信息后推送卖家酒店信息（美团酒店信息）
        String result = FzUtil.pushHotel(htHotelinfo, appKey, session, secret);
        if (result == null || !result.contains(ok)) {
            log.warn("{}：酒店推送失败，失败原因：{}", htHotelinfo.getHotelCode(), result);
            return hotelCodes;
        }
        JSONObject r1 = JSONUtil.parseObj(result);
        JSONObject xhotel_update_response = (JSONObject) r1.get("xhotel_update_response");
        JSONObject xhotel = (JSONObject) xhotel_update_response.get("xhotel");
        Long hid = xhotel.getLong("hid");
        htAliHotel.setHid(hid);
        //  自主匹配标准酒店以及卖家酒店
        Long hotelCode = htHotelinfo.getHotelCode();
        com.alibaba.fastjson.JSONObject matchObj = new com.alibaba.fastjson.JSONObject();
        JSONObject param0 = new JSONObject();
        param0.put("hid", hid);
        param0.put("shid", htAliHotel.getShid());
        matchObj.put("param0", param0);
        String matchResult = FzUtil.matchHotel(matchObj, appKey, session, secret);
        if (matchResult == null || !matchResult.contains("alitrip_hotel_hstdf_shotel_matchshotelself_response")) {
            log.warn("{}：自主匹配标准酒店以及卖家酒店失败，失败原因：{}", htHotelinfo.getHotelCode(), matchResult);
            return hotelCodes;
        }
        HtHotel htHotel = htHotelService.getById(htHotelinfo.getHotelCode());
        htHotel.setShid(htAliHotel.getShid());
        htHotelService.saveOrUpdate(htHotel);
        htAliHotelService.saveOrUpdate(htAliHotel);
        hotelCodes.add(hotelCode);
        // 更新数据库上架状态
        Integer isUp = 1;
        if (htHotelinfo.getHotelStatus() == -2) {
            isUp = 0;
        }
        opHtlGroupService.updateIsUpByHotelIdAndStoreId(isUp, hotelCode, storeId);
        return hotelCodes;
    }

    private HtAliHotel getMatchHotel(HtHotel htHotelinfo, String appKey, String sessionKey, String secret) {
        WaitMatchHotel waitMatchHotel = new WaitMatchHotel();
        waitMatchHotel.setCityCode(Convert.toStr(htHotelinfo.getCity()));
        waitMatchHotel.setHotelName(htHotelinfo.getHotelName());
        waitMatchHotel.setAddress(htHotelinfo.getAddress());
        String result = FzUtil.getMatchHotel(waitMatchHotel, appKey, sessionKey, secret);
        if (result == null) {
            log.warn("酒店匹配失败，酒店：{}，data：{}，失败原因-->接口错误！");
            return null;
        }
        if (result.contains("error_response")) {
            log.warn("酒店匹配失败，酒店：{}，data：{}，失败原因：{}");
            return null;
        }
        JSONObject r1 = JSONUtil.parseObj(result);
        JSONObject r2 = (JSONObject) r1.get("alitrip_hotel_hstdf_shotel_exportshotel_response");
        JSONObject top_result_set = (JSONObject) r2.get("top_result_set");
        JSONObject results = (JSONObject) top_result_set.get("results");
        JSONArray hotel_export_param = (JSONArray) results.get("hotel_export_param");
        if (hotel_export_param.isEmpty()) {
            log.warn("酒店匹配失败，成功匹配酒店数据为空！");
            return null;
        }
        for (Object o : hotel_export_param) {
            JSONObject mHotel = new JSONObject(o);
            Double score = Convert.toDouble(mHotel.get("score"));
            Integer statusCode = mHotel.getInt("status_code");
            if (score > 80D && statusCode == 0) {
                HtAliHotel htAliHotel = new HtAliHotel();
                htAliHotel = com.alibaba.fastjson.JSONObject.parseObject(String.valueOf(mHotel), HtAliHotel.class);
                htAliHotel.setHotelCode(Convert.toLong(htHotelinfo.getHotelCode()));
                htAliHotel.setStatusCode(statusCode);
                htAliHotel.setMatchType(0);
                log.info("匹配完成！");
                return htAliHotel;
            }
        }
        return null;

    }


    @Override
    public void delAll(Long hotelCode, Integer storeId, String appKey, String secret, String session) {
        // 查询酒店是否上架到店铺 已经上架进行删除 没有上架的不进行删除
        Integer isUp = opHtlGroupService.getIsUpByStoreIdAndHotelId(storeId, hotelCode);
        if (Objects.isNull(isUp) || isUp.equals(0)) {
            return;
        }
        // 初始化删除状态
        boolean flag = true;
        // 首先删除商品（价格政策 和 价格的日历）
        List<Long> longList = htSellRoomService.getRpCodesByHotelCode(hotelCode);
        for (Long rpCode : longList) {
            // 价格政策和价格日历的删除
            boolean flagRp = rpDel(rpCode, null, storeId, appKey, secret, session);
            // 只要有一个删除不成功 酒店就不进行删除
            if (!flagRp) {
                log.warn("店铺id：{}，酒店编码：{}，全量删除时rp：{}删除失败", storeId, hotelCode, rpCode);
                flag = false;
            }
        }
        // 如果删除失败 酒店不在进行删除
        if (!flag) {
            return;
        }
        // 删除酒店操作
        boolean flagHtl = this.hotelDel(hotelCode, storeId, appKey, secret, session);
        if (!flagHtl) {
            return;
        }
        // 删除成功更新酒店推送状态
        opHtlGroupService.updateIsUpByHotelIdAndStoreId(0, hotelCode, storeId);
    }

    /**
     * 删除酒店操作
     *
     * @param hotelCode Long
     * @param storeId   Integer
     * @param appKey    String
     * @param secret    String
     * @param session   String
     * @return boolean
     */
    @Override
    public boolean hotelDel(Long hotelCode, Integer storeId, String appKey, String secret, String session) {
        String ok = "xhotel_delete_response";
        String result = FzUtil.delHotel(hotelCode.toString(), appKey, session, secret);
        if (result == null || !result.contains(ok)) {
            log.warn("店铺id：{}，删除酒店失败，失败原因：{}", storeId, result);
            return false;
        }
        return true;
    }

    @Override
    public List<HtSellRoom> roomTypePushBatch(List<Long> hotelCodes, Integer storeId, List<HtRoomTypeVO> htPhysicalRooms, String appKey, String secret, String session) {
        Long hotelCode = hotelCodes.get(0);
        List<HtSellRoom> htSellRoomList = new ArrayList<>();
        QueryWrapper<HtAliHotel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("hotel_code", hotelCode).eq("store_id", storeId);
        HtAliHotel htAliHotel = htAliHotelService.getOne(queryWrapper);
        String getMatchRoomResult = FzUtil.getMatchRoom(htAliHotel.getHid(), appKey, session, secret);
        if (getMatchRoomResult == null || !getMatchRoomResult.contains("alitrip_hotel_hstdf_shotel_exportsroomtype_response")) {
            log.warn("酒店{}：查询卖家酒店下的所有标准房型，失败原因：{}", hotelCode, getMatchRoomResult);
            return htSellRoomList;
        }
        JSONObject matchRoomResultObj = JSONUtil.parseObj(getMatchRoomResult);
        JSONObject resultObj = (JSONObject) matchRoomResultObj.get("alitrip_hotel_hstdf_shotel_exportsroomtype_response");
        JSONObject topResultSet = (JSONObject) resultObj.get("result");
        JSONObject results = (JSONObject) topResultSet.get("results");
        JSONArray sroomTypePo = (JSONArray) results.get("sroom_type_po");

        String ok = "xhotel_roomtype_add_response";
        for (HtRoomTypeVO htPhysicalRoom : htPhysicalRooms) {
            Long roomCode = htPhysicalRoom.getRoomCode();
            String result = FzUtil.pushRoom(htPhysicalRoom, appKey, session, secret);
            if (!result.contains(ok)) {
                log.warn("【酒店全量推送】酒店：{}，物理房型：{}，房型推送失败，失败原因：{}", htPhysicalRoom.getHotelCode(), roomCode, result);
                continue;
            }
            QueryWrapper<HtAliRoom> htAliRoomQueryWrapper = new QueryWrapper<>();
            htAliRoomQueryWrapper.eq("room_code",roomCode).eq("store_id", storeId);
            HtAliRoom htAliRoom = htAliRoomService.getOne(htAliRoomQueryWrapper);
            Long srid = 0L;
            Long rid = 0L;
            if (htAliRoom == null) {
                JSONObject r1 = JSONUtil.parseObj(result);
                JSONObject xhotel_roomtype_add_response = (JSONObject) r1.get("xhotel_roomtype_add_response");
                JSONObject xroomtype = (JSONObject) xhotel_roomtype_add_response.get("xroomtype");
                rid = xroomtype.getLong("rid");
                String matchRoomName = xroomtype.getStr("name");
                Integer matchType = 0;
                JSONObject matchRoom = new JSONObject();
                for (Object o : sroomTypePo) {
                    cn.hutool.json.JSONObject sroom = JSONUtil.parseObj(o);
                    if (matchRoomName.equals(sroom.getStr("name"))) {
                        matchType = 1;
                        matchRoom = sroom;
                        break;
                    }
                }
                if (matchType == 0) {
                    matchRoom = handleMatch(xroomtype, sroomTypePo);
                }
                if (matchRoom == null) {
                    continue;
                }
                srid = matchRoom.getLong("srid");
                htAliRoom = com.alibaba.fastjson.JSONObject.parseObject(String.valueOf(matchRoom), HtAliRoom.class);
                htAliRoom.setRoomCode(roomCode);
                htAliRoom.setId(roomCode);
                htAliRoom.setRid(rid);
                htAliRoom.setStoreId(storeId);
            } else {
                rid = htAliRoom.getRid();
                srid = htAliRoom.getSrid();
            }
            com.alibaba.fastjson.JSONObject matchRoomData = new com.alibaba.fastjson.JSONObject();
            com.alibaba.fastjson.JSONObject param0 = new com.alibaba.fastjson.JSONObject();
            param0.put("rid", rid);
            param0.put("srid", srid);
            matchRoomData.put("param0", param0);
            String matchRoomResult = FzUtil.matchRoom(matchRoomData, appKey, session, secret);
            if (matchRoomResult == null || !matchRoomResult.contains("alitrip_hotel_hstdf_shotel_matchsroomself_response")) {
                log.warn("【自动匹配房型失败】酒店：{}，物理房型：{}，失败原因：{}", htPhysicalRoom.getHotelCode(), roomCode, matchRoomResult);
                continue;
            }
            //  更新物理房型信息
            QueryWrapper<HtRoomType> roomQueryWrapper = new QueryWrapper<>();
            roomQueryWrapper.eq("room_code", roomCode);
            HtRoomType htRoomType = htPhysicalRoomService.getOne(roomQueryWrapper);
            htRoomType.setSrid(srid);
            htPhysicalRoomService.saveOrUpdate(htRoomType);
            //  更新飞猪标准房型信息
            htAliRoomService.saveOrUpdate(htAliRoom);
            // 如果推送成功则放入集合中 返回用
            htSellRoomList.addAll(htPhysicalRoom.getHtSellRooms());
        }
        return htSellRoomList;
    }


    ScoreSuggester suggester = new ScoreSuggester();

    private JSONObject handleMatch(JSONObject xroomtype, JSONArray sroomTypePo) {
        List<String> axRoomList = new ArrayList<>(sroomTypePo.size());
        for (int i1 = 0; i1 < sroomTypePo.size(); i1++) {
            String name = sroomTypePo.getJSONObject(i1).getStr("name");
            axRoomList.add(name);
            suggester.addSentence(name);
        }
//      文本推荐
        List<Map<String, Double>> matchRoom = suggester.suggestScore(xroomtype.getStr("name"), 5, 10);
        log.info("{}匹配飞猪酒店标准房型结果为：{}", xroomtype.getStr("name"), matchRoom);
        if (CollUtil.isEmpty(matchRoom)) {
            return null;
        }
//      优先使用推荐最高一个
        String roomName = CollUtil.get(matchRoom.get(0).keySet(), 0);
        JSONObject jsonObject = sroomTypePo.getJSONObject(axRoomList.indexOf(roomName));
        if (jsonObject == null) {
            return null;
        }
        jsonObject.put("matchScore", CollUtil.get(matchRoom.get(0).values(), 0));
        return jsonObject;
    }

    @Override
    public Map<String, List<Integer>> rpPushBatch(List<HtSellRoom> htLogicRooms, Integer storeId, String appKey, String secret,
                                                  String session, List<OpOurRateplanVo> opOurRateplans, Integer partnerId) {
        String infix = "ssj";
        String ok = "xhotel_rateplan_update_response";
        Map<String, List<Integer>> rpCrpIdsMap = new HashMap<>(10);
        for (HtSellRoom htLogicRoom : htLogicRooms) {
            Long rpCode = htLogicRoom.getRpCode();
            // 自定义价格政策主键集合
            List<Integer> crpIds = new ArrayList<>();
            for (OpOurRateplan opOurRateplan : opOurRateplans) {
                Integer id = opOurRateplan.getId();
                // 推送忽略这两个字段
                htLogicRoom.setBreakfastCal(null);
                htLogicRoom.setCancelPolicyCal(null);
                // 自定义早餐
                Integer breakfastNum = opOurRateplan.getBreakfastNum();
                // 早餐 -1 为默认使用接口中的早餐 反之使用自定义早餐
                if (!breakfastNum.equals(-1)) {
                    htLogicRoom.setBreakfastNum(breakfastNum);
                }
                // 自定义退改政策
                String cancel = opOurRateplan.getCancleJsonStr();
                // 如果自定义退改政策不为NONE则使用自定义价格政策
                if (!"NONE".equals(cancel)) {
                    htLogicRoom.setCancelPolicy(cancel);
                }
                // 自定义价格政策编码
                String newRpCode = htLogicRoom.getRpCode() + infix + id;
                // 推送渠道
                String channel = opOurRateplan.getChannel();
                String result = FzUtil.pushRp(htLogicRoom, newRpCode, appKey, session, secret, channel);
                // 如果没有推送成功不做处理
                if (result == null || !result.contains(ok)) {
                    log.warn("{}：rp推送失败，失败原因：{}", rpCode, result);
                    continue;
                }
                // 如果推送成功 自定义价格政策主键放入集合中
                crpIds.add(id);
            }
            // 如果自定义价格政策集合的主键不为空
            if (!crpIds.isEmpty()) {
                // 把 数据放入map  k：售卖房型编码 + 物理房型编码  v：自定义价格政策的集合  k这样拼接主要是为了缓存使用
                rpCrpIdsMap.put(rpCode + "_" + htLogicRoom.getRoomCode(), crpIds);
            }
        }
        // 修改或保存rp缓存 其中storeId是对应的店铺主键 partnerId 是为了区分数据来源 （美团接口有两个数据来源 主要区分这个）
        if (!rpCrpIdsMap.isEmpty()) {
            // 关于rp缓存的操作
            this.saveRpCache(storeId, rpCrpIdsMap, partnerId);
        }
        return rpCrpIdsMap;
    }

    /**
     * @param rpCode  Long
     * @param crpIds  List<Integer>
     * @param storeId Integer
     * @param appKey  String
     * @param secret  String
     * @param session String
     * @return boolean
     */
    @Override
    public boolean rpDel(Long rpCode, List<Integer> crpIds, Integer storeId, String appKey, String secret, String session) {
        // 删除成功的标识
        String ok = "xhotel_rateplan_delete_response";
        String key = rpCode.toString();
        // 查询缓存的自定义价格政策id 如果缓存没有价格政策 认为该酒店已经删除 （酒店删除失败时候会出现这种情况）
        Map<String, Object> rpStoreCache = cache.getRpStoreMap(key, storeId);
        // 直接返回 下面不进行数据库的减操作 保证数据个数的准确性
        if (rpStoreCache.isEmpty()) {
            return true;
        }
        List<Integer> crpIdsCache = (List<Integer>) rpStoreCache.get("crpIds");
        List<Integer> delCrpIds;
        // 如果为空集合 说明自定义价格政策全部删除  不为空只删除集合内的自定义价格政策
        if (Objects.isNull(crpIds)) {
            delCrpIds = crpIdsCache;
        } else {
            delCrpIds = crpIds;
        }
        int count = delCrpIds.size();
        // 删除成功的自定义rpId集合
        List<Integer> deledCrpIds = new ArrayList<>();
        for (Integer crpId : delCrpIds) {
            String newRpCode = key + "ssj" + crpId;
            String result = FzUtil.delRp(newRpCode, appKey, session, secret);
            if (result == null || !result.contains(ok)) {
                log.error("店铺id：{}，rp：{}，删除失败，失败原因：{}", storeId, newRpCode, result);
                continue;
            }
            deledCrpIds.add(crpId);
        }
        if (deledCrpIds.isEmpty()) {
            return false;
        }
        // 删除店铺缓存rp 和全局过滤对应的店铺主键
        boolean flag = cache.delRpStoreMap(key, storeId, deledCrpIds);
        if (!flag) {
            return false;
        }
        // 根据id更新自定义rp的数量
        for (Integer id : deledCrpIds) {
            opOurRateplanService.addCountById(-1, id);
        }
        // 传入的数量和删除的数量一致才认为 rp删除成功
        return deledCrpIds.size() == count;
    }

    @Override
    public void rpPush(Long rpCode, List<Integer> crpIds, Integer storeId, String appKey, String secret, String session, Integer partnerId) {

        if (!redisUtil.hashExistFiled(Constant.Redis.GOODS_BL_CRP_HTL_ROOM_RP_STORE.getValue() + "_" + storeId, rpCode.toString())) {
            return;
        }
        HtSellRoom htLogicRoom = htSellRoomService.getById(rpCode);
        List<HtSellRoom> htLogicRooms = new ArrayList<>();
        htLogicRooms.add(htLogicRoom);
        List<OpOurRateplanVo> opOurRateplans = opOurRateplanService.getRatePlanPrice(storeId, htLogicRoom.getHotelCode());
        rpPushBatch(htLogicRooms, storeId, appKey, secret, session, opOurRateplans, partnerId);
    }

    /**
     * rp缓存
     *
     * @param storeId 店铺id
     * @param map     Map<String, List<Integer>>
     */
    private void saveRpCache(Integer storeId, Map<String, List<Integer>> map, Integer partnerId) {
        for (String v : map.keySet()) {
            String[] newStr = v.split("_");
            String rpCode = newStr[0];
            String roomCode = newStr[1];
            List<Integer> crpIds = map.get(v);
            // 添加缓存
            List<Integer> newCrpIds = cache.addRpStoreMap(roomCode, rpCode, storeId, crpIds, partnerId);
            if (newCrpIds.isEmpty()) {
                return;
            }
            // 每推送成功一个数据库进行加1 统计自定义价格政策的个数
            for (Integer id : newCrpIds) {
                opOurRateplanService.addCountById(1, id);
            }
        }
    }

    @Override
    public void ratesPushBatch(Map<Long, List<HtDailyRoom>> map, Integer storeId) {
        // 根据店铺id 缓存查询 店铺信息 缓存不存在店铺信息 则不进行推送
        DtStore dtStore = cache.getStore(storeId);
        if (Objects.isNull(dtStore)) {
            log.warn("【价格批量更新】店铺id：{} 缓存不存在", storeId);
            return;
        }
        Map<String, List<HtDailyRoom>> stringListMap = new HashMap<>(50);
        for (Long k : map.keySet()) {
            // 查询店铺缓存 下rp推送的roomCode和自定义rp的id
            Map<String, Object> rpStoreMap = cache.getRpStoreMap(k.toString(), storeId);
            // 为空基本不会存在 可能会出现脏数据
            if (rpStoreMap.isEmpty()) {
                log.warn("【价格批量更新】rpCode:{},全局缓存和店铺缓存不同步", k);
                continue;
            }
            String roomCode = (String) rpStoreMap.get("roomCode");
            List<Integer> crpIds = (List<Integer>) rpStoreMap.get("crpIds");
            // 批量查询缓存价态
            List<HtDailyRoom> htDailyRooms = map.get(k);
            // 每个 rp 可以根据自己的情况 进行加工成多个rp
            for (Integer crpId : crpIds) {
                // 获取阶梯价格加幅
                List<OpOurRateplanPrice> opOurRateplanPriceList = cache.getStairPrice(crpId);
                if (opOurRateplanPriceList == null || opOurRateplanPriceList.isEmpty()) {
                    log.warn("【价格批量推送】店铺id:{}，自定义价格政策不存在自定义加幅：{}，价格无法推送上架", storeId, crpId);
                    continue;
                }
                // 价格计算
                calculation(htDailyRooms, opOurRateplanPriceList);
                stringListMap.put(k + "ssj" + crpId + "_" + roomCode, htDailyRooms);
            }
        }
        // 可能为空 比如全局缓存和 店铺缓存不同步的情况下（这一批全部不同步）
        if (stringListMap.isEmpty()) {
            return;
        }
        // 批量推送 每次20个价态的变化
        List<Map<String, List<HtDailyRoom>>> mapList;
        int maxNum = 20;
        if (stringListMap.size() <= maxNum) {
            mapList = new ArrayList<>();
            mapList.add(stringListMap);
        } else {
            // map 拆分最大size 30
            mapList = MapUtil.mapChunk(stringListMap, maxNum);
        }
        // 价格更新
        retryPushBatchRates(mapList, dtStore);
    }

    /**
     * 批量价格更新
     *
     * @param mapList 价格集合
     * @param dtStore 店铺信息
     */
    private void retryPushBatchRates(List<Map<String, List<HtDailyRoom>>> mapList, DtStore dtStore) {
        for (Map<String, List<HtDailyRoom>> mapCut : mapList) {
            String result = FzUtil.pushRates(mapCut, dtStore.getAppKey(), dtStore.getSessionKey(), dtStore.getSecret());
            if (result == null) {
                log.warn("【价格批量推送】推送失败，店铺：{}，data：{}，失败原因-->接口错误！", dtStore.getName(), mapList);
                continue;
            }
            if (result.contains("error_response")) {
                log.warn("【价格批量推送】推送失败，店铺：{}，data：{}，失败原因：{}", dtStore.getName(), mapList, result);
            }
        }
    }

    /**
     * 价格加幅计算
     *
     * @param htDailyRooms           价格集合
     * @param opOurRateplanPriceList 价格加幅集合
     */
    private void calculation(List<HtDailyRoom> htDailyRooms, List<OpOurRateplanPrice> opOurRateplanPriceList) {
        for (HtDailyRoom htDailyRoom : htDailyRooms) {
            Integer price = htDailyRoom.getPrice();
            for (OpOurRateplanPrice opOurRateplanPrice : opOurRateplanPriceList) {
                Integer changePriceStart = opOurRateplanPrice.getChangePriceStart();
                Integer changePriceEnd = opOurRateplanPrice.getChangePriceEnd();
                Integer highestPrice = opOurRateplanPrice.getHighestPrice();
                if (price >= changePriceStart && price < changePriceEnd) {
                    price = price * (100 + opOurRateplanPrice.getChargePricePercent()) / 100 + opOurRateplanPrice.getChargePriceAmount();
                    if (price >= highestPrice) {
                        price = highestPrice;
                    }
                    htDailyRoom.setPrice(price);
                    // 找到对应的加幅范围 加幅完成之后跳出循环
                    break;
                }
            }
        }
    }

    @Override
    public List<HtAliHotel> getMatchHotels(Map<String, Object> map) {
        JSONObject hotel = JSONUtil.parseObj(map);
//        DtStore dtStore = cache.getStore(hotel.getInt("storeId"));
        DtStore dtStore = cache.getStore(1);
        String appKey = dtStore.getAppKey();
        String sessionKey = dtStore.getSessionKey();
        String secret = dtStore.getSecret();
        WaitMatchHotel waitMatchHotel = new WaitMatchHotel();
        waitMatchHotel.setCityCode(hotel.getStr("city"));
        waitMatchHotel.setHotelName(hotel.getStr("hotelName"));
        waitMatchHotel.setAddress(hotel.getStr("address"));
        String result = FzUtil.getMatchHotel(waitMatchHotel, appKey, sessionKey, secret);
        if (result == null) {
            log.warn("酒店匹配失败，酒店：{}，data：{}，失败原因-->接口错误！");
            return null;
        }
        if (result.contains("error_response")) {
            log.warn("酒店匹配失败，酒店：{}，data：{}，失败原因：{}");
            return null;
        }
        JSONObject r1 = JSONUtil.parseObj(result);
        JSONObject r2 = (JSONObject) r1.get("alitrip_hotel_hstdf_shotel_exportshotel_response");
        JSONObject top_result_set = (JSONObject) r2.get("top_result_set");
        JSONObject results = (JSONObject) top_result_set.get("results");
        List<HtAliHotel> hotel_export_param = (List<HtAliHotel>) results.get("hotel_export_param");
        if (hotel_export_param.isEmpty()) {
            return null;
        }
        return hotel_export_param;
    }

    @Override
    public Boolean manualMatchHotel(Map<String, Object> map) {
        JSONObject data = JSONUtil.parseObj(map);
        Long hotelCode = data.getLong("hotelCode");
        Integer partnerId = data.getInt("partnerId");
        HtAliHotel htAliHotel = com.alibaba.fastjson.JSONObject.parseObject(String.valueOf(data.getJSONObject("matchHotelData")), HtAliHotel.class);
        List<Integer> storeIds = opHtlGroupService.getStoreIdByHotelId(hotelCode, partnerId);
        //  更新卖家酒店信息
        HtHotel htHotel = htHotelService.getById(hotelCode);
        htHotel.setShid(htAliHotel.getShid());
        htHotelService.saveOrUpdate(htHotel);
        htAliHotel.setMatchType(1);
        htAliHotel.setHotelCode(hotelCode);
        if (storeIds.isEmpty()){
            //  更新阿里标准酒店信息
            htAliHotelService.saveOrUpdate(htAliHotel);
        }else{
            for (Integer storeId : storeIds) {
                QueryWrapper<HtAliHotel> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("hotel_code", hotelCode).eq("store_id", storeId);
                HtAliHotel oldHtAliHotel = htAliHotelService.getOne(queryWrapper);
                if (oldHtAliHotel != null){
                    htAliHotel.setId(oldHtAliHotel.getId());
                    htAliHotelService.saveOrUpdate(htAliHotel);
                }
                PushHotelBody pushHotelBody = new PushHotelBody();
                pushHotelBody.setHotelId(hotelCode);
                pushHotelBody.setStoreId(storeId);
                pushHotelBody.setType(2);
                //将消息发送到“storeId_hotel”队列中
                sendMsg.execute(Constant.RabbitMq.HOTEL, storeId, pushHotelBody);
            }
        }
        return true;
    }


    @Override
    public JSONArray getMatchRooms(Map<String, Object> map) {
        JSONObject data = JSONUtil.parseObj(map);
        Long hotelCode = data.getLong("hotelCode");
        Integer storeId = data.getInt("storeId");
        DtStore dtStore = cache.getStore(storeId);
        String appKey = dtStore.getAppKey();
        String sessionKey = dtStore.getSessionKey();
        String secret = dtStore.getSecret();
        QueryWrapper<HtAliHotel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("hotel_code", hotelCode).eq("store_id", storeId);
        HtAliHotel htAliHotel = htAliHotelService.getOne(queryWrapper);
        String getMatchRoomResult = FzUtil.getMatchRoom(htAliHotel.getHid(), appKey, sessionKey, secret);
        if (getMatchRoomResult == null || !getMatchRoomResult.contains("alitrip_hotel_hstdf_shotel_exportsroomtype_response")) {
            log.warn("酒店{}：查询卖家酒店下的所有标准房型，失败原因：{}", hotelCode, getMatchRoomResult);
            return null;
        }
        JSONObject matchRoomResultObj = JSONUtil.parseObj(getMatchRoomResult);
        JSONObject resultObj = (JSONObject) matchRoomResultObj.get("alitrip_hotel_hstdf_shotel_exportsroomtype_response");
        JSONObject topResultSet = (JSONObject) resultObj.get("result");
        JSONObject results = (JSONObject) topResultSet.get("results");
        JSONArray sroomTypePo = (JSONArray) results.get("sroom_type_po");
        return sroomTypePo;
    }

    @Override
    public Boolean manualMatchRoom(Map<String, Object> map) {
        JSONObject data = JSONUtil.parseObj(map);
        Integer storeId = data.getInt("storeId");
        DtStore dtStore = cache.getStore(storeId);
        String appKey = dtStore.getAppKey();
        String sessionKey = dtStore.getSessionKey();
        String secret = dtStore.getSecret();
        Long roomCode = data.getLong("roomCode");
        //  获取人工选择房型信息
        HtAliRoom htAliRoom = com.alibaba.fastjson.JSONObject.parseObject(String.valueOf(data.getJSONObject("matchRoomData")), HtAliRoom.class);
        //  查询卖家标准酒店房型信息
        QueryWrapper<HtRoomType> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("room_code", roomCode);
        HtRoomType htRoomType = htPhysicalRoomService.getOne(queryWrapper1);
        //  查询阿里标准房型信息
        QueryWrapper<HtAliRoom> queryWrapper2 = new QueryWrapper<>();
        queryWrapper2.eq("room_code", roomCode).eq("store_id", storeId);
        HtAliRoom oldHtAliRoom = htAliRoomService.getOne(queryWrapper2);
        String ok = "xhotel_roomtype_add_response";
        if (oldHtAliRoom == null){
            htAliRoom.setRoomCode(roomCode);
        }else{
            htAliRoom.setId(oldHtAliRoom.getId());
        }
        String result = FzUtil.pushRoom(htRoomType, appKey, sessionKey, secret);
        if (!result.contains(ok)) {
            return null;
        }
        htAliRoom.setStoreId(storeId);
        Long srid = 0L;
        Long rid = 0L;
        JSONObject r1 = JSONUtil.parseObj(result);
        JSONObject xhotel_roomtype_add_response = (JSONObject) r1.get("xhotel_roomtype_add_response");
        JSONObject xroomtype = (JSONObject) xhotel_roomtype_add_response.get("xroomtype");
        rid = xroomtype.getLong("rid");
        srid = htAliRoom.getSrid();
        com.alibaba.fastjson.JSONObject matchRoomData = new com.alibaba.fastjson.JSONObject();
        com.alibaba.fastjson.JSONObject param0 = new com.alibaba.fastjson.JSONObject();
        param0.put("rid", rid);
        param0.put("srid", srid);
        matchRoomData.put("param0", param0);
        String matchRoomResult = FzUtil.matchRoom(matchRoomData, appKey, sessionKey, secret);
        if (matchRoomResult == null || !matchRoomResult.contains("alitrip_hotel_hstdf_shotel_matchsroomself_response")) {
            return false;
        }
        //  更新物理房型信息
        htRoomType.setSrid(srid);
        htPhysicalRoomService.saveOrUpdate(htRoomType);
        //  更新飞猪标准房型信息
        htAliRoom.setRid(rid);
        htAliRoomService.saveOrUpdate(htAliRoom);
        return true;
    }
}
