package com.qls.consumer.biz.config;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.qls.consumer.biz.service.HtHotelinfoService;
import com.qls.consumer.biz.service.OpHtlGroupService;
import com.qls.consumer.biz.service.OpTaskStatusService;
import com.qls.core.constant.Constant;
import com.qls.core.dto.*;
import com.qls.core.entity.HtHotel;
import com.qls.core.entity.OpTaskStatus;
import com.qls.mt.api.MtRemoteService;
import com.qls.rabbitmq.SendMsg;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;


/**
 * @author doyoung
 */
@Slf4j
@Component
public class ReceiveHandlerBasic {

    @Autowired
    private OpTaskStatusService opTaskStatusService;

    @Autowired
    private MtRemoteService mtRemoteService;

    @Autowired
    private SendMsg sendMsg;

    @Autowired
    private HtHotelinfoService htHotelinfoService;

    @Autowired
    private OpHtlGroupService opHtlGroupService;


    @RabbitListener(queues = "PRODUCT_CHANGE", containerFactory = "callbackProduct")
    public void productChange(List<Message> list) {

        List<ProductChangeMQ> productChangeMQS1 = new ArrayList<>();
        List<ProductChangeMQ> productChangeMQS2 = new ArrayList<>();
        for (Message msg : list) {
            byte[] bytes = msg.getBody();
            ProductChangeMQ productChangeMQ = JSONObject.parseObject(bytes, ProductChangeMQ.class);
            if (productChangeMQ.getPartnerId().equals(29082)) {
                productChangeMQS1.add(productChangeMQ);
            } else {
                productChangeMQS2.add(productChangeMQ);
            }
        }
        // 根据渠道来分组数据
        try {
            ProductChangeMQBatch productChangeMQBatch = new ProductChangeMQBatch();
            if (!productChangeMQS1.isEmpty()) {
                productChangeMQBatch.setProductChangeMQS(productChangeMQS1);
                productChangeMQBatch.setPartnerId(29082);
                mtRemoteService.productBatch(productChangeMQBatch);
            }
//            if (!productChangeMQS2.isEmpty()) {
//                productChangeMQBatch.setProductChangeMQS(productChangeMQS2);
//                productChangeMQBatch.setPartnerId(5173);
//                mtRemoteService.productBatch(productChangeMQBatch);
//            }
        } catch (Exception e) {
            log.error("产品变化远程服务接口调用失败");
        }
    }

    @RabbitListener(queues = "INIT", containerFactory = "initContainerFactory")
    public void init(InitBody msg) {
        Integer taskId = msg.getTaskId();
        Long hotelId = msg.getHotelId();
        this.add(taskId, hotelId);
        this.over(hotelId, taskId);
        if (hotelId.equals(-1L)) {
            return;
        }
        try {
            mtRemoteService.hotelDetail(msg);
        } catch (Exception e) {
            log.error("酒店id：" + hotelId + "酒店初始化失败" + e.getMessage());
        }
    }


    @RabbitListener(queues = "UPDATE_RP", containerFactory = "rpUpdate")
    public void updateRp(Long hotelId) {
        // 调用远程接口进行更新
        try {
            Map<String, Long> params = new HashMap<>(1);
            params.put("hotelId", hotelId);
            mtRemoteService.updateRp(params);
        } catch (Exception e) {
            log.error("更新酒店：{} 下的rp时异常，e：{}", hotelId, e.toString());
        }
    }

    @RabbitListener(queues = "CALLBACK_HOTEL", containerFactory = "callbackHotel")
    public void callbackHotel(JSONObject msg) {
        try {
            JSONArray hotelIds = msg.getJSONArray("hotelIds");
            Integer hotelStatus = msg.getInteger("hotelStatus");
            JSONArray hotelArray = new JSONArray();
            for (Object o : hotelIds) {
                Long hotelId = Convert.toLong(o);
                HtHotel htHotel = htHotelinfoService.getById(hotelId);
                htHotel.setHotelStatus(hotelStatus);
                htHotelinfoService.updateById(htHotel);
                Integer partnerId = htHotel.getPartnerId();

                List<Integer> storeIds = opHtlGroupService.getStoreIdByHotelId(hotelId, partnerId);

                for (Integer storeId : storeIds) {
                    JSONObject hotel = new JSONObject();
                    hotel.put("hotelId", hotelId);
                    hotel.put("storeId", storeId);
                    hotelArray.add(hotel);
                }

            }
            JSONObject end = new JSONObject();
            end.put("hotelId", -1L);
            end.put("storeId", -1);
            Integer count = hotelArray.size();
            hotelArray.add(end);
            //创建任务
            OpTaskStatus opTaskStatus = new OpTaskStatus();
            opTaskStatus.setTaskName("批量酒店上下架");
            opTaskStatus.setTotal(count);
            opTaskStatus.setStartTime(new Date());
            Integer taskCode = Integer.parseInt("" + System.currentTimeMillis() / 1000);
            opTaskStatus.setTaskCode(taskCode);
            opTaskStatus.setDtId(1);
            opTaskStatusService.save(opTaskStatus);
            for (int i = 0; i < hotelArray.size(); i++) {
                JSONObject hotel = hotelArray.getJSONObject(i);
                Integer storeId = hotel.getInteger("storeId");
                PushHotelBody pushHotelBody = new PushHotelBody();
                pushHotelBody.setHotelId(hotel.getLong("hotelId"));
                pushHotelBody.setStoreId(storeId);
                pushHotelBody.setTaskId(taskCode);
                // 2 对酒店进行更新
                pushHotelBody.setType(2);
                sendMsg.execute(Constant.RabbitMq.HOTEL, storeId, pushHotelBody);
            }
        } catch (Exception e) {
            log.error("批量酒店上下架推送失败：" + e.getMessage());
        }

    }

    /**
     * 增加任务数量
     *
     * @param taskCode Integer
     */
    private void add(Integer taskCode, Long code) {
        if (Objects.isNull(taskCode) || code.equals(-1L)) {
            return;
        }
        opTaskStatusService.completedAdd(taskCode, 1);
    }

    /**
     * 结束任务
     *
     * @param code     Long
     * @param taskCode Integer
     */
    private void over(Long code, Integer taskCode) {
        if (Objects.nonNull(taskCode) && code.equals(-1L)) {
            opTaskStatusService.updateByTaskCode(taskCode, new Date());
        }
    }


}
