package com.qls.mt.api;


import com.qls.core.ali.order.bookrq.request.BookRq;
import com.qls.core.ali.order.cancelrq.CancelRq;
import com.qls.core.ali.order.validaterq.request.ValidateRq;
import com.qls.core.dto.*;
import com.qls.core.utils.R;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * @author doyoung
 */
@FeignClient("mt-service")
public interface MtRemoteService {

    /**
     * 拉取酒店
     *
     * @param initBody InitBody
     * @return R
     */
    @PostMapping(value = "mt/hotel/detail")
    R hotelDetail(@RequestBody InitBody initBody);

    /**
     * 拉取酒店 房型 售卖房型 价态信息
     *
     * @param pushHotelBody PushHotelBody
     * @return R
     */
    @PostMapping(value = "mt/hotel/hotel_all")
    R hotelAll(@RequestBody PushHotelBody pushHotelBody);

    /**
     * 拉取酒店列表
     *
     * @param params Map<String, Object>
     * @return R
     */
    @GetMapping(value = "mt/hotel/pull_all")
    R pullAll(@RequestParam Map<String, Object> params);

    /**
     * 下单前校验(美团）
     *
     * @param validateRq ValidateRq
     * @return
     */
    @PostMapping(value = "mt/order/validate")
    R validate(@RequestBody ValidateRq validateRq);

    /**
     * 预约下单(美团）
     *
     * @param bookRq BookRq
     * @return
     */
    @PostMapping(value = "mt/order/book")
    R book(@RequestBody BookRq bookRq);

    /**
     * 取消下单(美团）
     *
     * @param cancelRq CancelRq
     * @return
     */
    @PostMapping(value = "mt/order/cancel")
    R cancel(@RequestBody CancelRq cancelRq);

    /**
     * 产品变更批量处理
     * @param productChangeMQBatch ProductChangeMQBatch
     * @return R
     */
    @PostMapping(value = "mt/hotel/product/batch")
    R productBatch(@RequestBody ProductChangeMQBatch productChangeMQBatch);

    /**
     * 更新rp
     * @param params
     * @return
     */
    @GetMapping(value = "mt/hotel/update_rp")
    R updateRp(@RequestParam Map<String, Long> params);

}
