package com.qls.mt.biz.data.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.qls.core.dto.ProductChangeMQBatch;
import com.qls.core.utils.R;
import com.qls.mt.biz.common.utils.MtResponse;
import com.qls.mt.biz.data.service.ChangeService;
import com.qls.mt.module.callback.PoiChangeBody;
import com.qls.mt.module.callback.ProductChangeBody;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author doyoung
 */
@Slf4j
@RestController
public class ChangeController {

    @Autowired
    private ChangeService changeService;


    /**
     * 美团酒店信息变更通知
     *
     * @param data JSONObject
     * @return MtResponse
     */
    @PostMapping(value = "hotel/poi/change")
    public MtResponse poiChange(@RequestBody JSONObject data) {
        Integer partnerId = data.getInteger("partnerId");
        PoiChangeBody changeBody = JSON.parseObject(data.getString("data"), PoiChangeBody.class);
        try {
            changeService.poi(changeBody, partnerId);
        } catch (Exception e) {
            log.error("美团酒店变化更新失败，请求数据：{}，失败原因：{}", JSON.toJSON(changeBody),e.getStackTrace());
            return MtResponse.error();
        }
        return MtResponse.ok();
    }

    /**
     * 美团产品信息变更通知
     *
     * @param data JSONObject
     * @return MtResponse
     */
    @PostMapping(value = "hotel/product/change")
    public MtResponse productChange(@RequestBody JSONObject data) {
        Integer partnerId = data.getInteger("partnerId");
        ProductChangeBody changeBody = JSON.parseObject(data.getString("data"), ProductChangeBody.class);
        try {
            changeService.productIdFilter(changeBody, partnerId);
        } catch (Exception e) {
            log.error("美团产品变化更新失败，请求数据：{}，失败原因：{}", JSON.toJSON(changeBody), e.getStackTrace());
            return MtResponse.error();
        }
        return MtResponse.ok();
    }

    /**
     * 产品变化的批量操作
     *
     * @param productChangeMQBatch ProductChangeMQBatch
     * @return R
     */
    @PostMapping(value = "/hotel/product/batch")
    public R productBatch(@RequestBody ProductChangeMQBatch productChangeMQBatch) {
        changeService.productBatch(productChangeMQBatch);
        return R.ok();
    }

}
