package com.qls.mt.biz.data.controller;

import com.qls.core.dto.InitBody;
import com.qls.core.dto.PushHotelBody;
import com.qls.core.entity.DtOrderChannels;
import com.qls.core.utils.R;
import com.qls.mt.biz.common.service.DtOrderChannelsService;
import com.qls.mt.biz.data.service.HotelService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Administrator
 */
@RestController
@RequestMapping("hotel")
@Slf4j
public class MtDataController {

    @Autowired
    private HotelService hotelService;

    @Autowired
    private DtOrderChannelsService dtOrderChannelsService;

    /**
     * 酒店信息入库（酒店初始化）
     *
     * @param initBody InitBody
     * @return R
     */
    @PostMapping(value = "detail")
    public R hotelDetail(@RequestBody InitBody initBody) {
        PushHotelBody pushHotelBody = new PushHotelBody();
        pushHotelBody.setHotelId(initBody.getHotelId());
        Map<String, Object> map = new HashMap<>(3);
        map.put("partnerId", initBody.getPartnerId());
        map.put("accessKey", initBody.getAccessKey());
        map.put("secretKey", initBody.getSecretKey());
        // 放入数据库
        hotelService.saveHotel(pushHotelBody, map);
        return R.ok();
    }

    /**
     * 酒店 及酒店下的房型 rp 入库    rates 缓存
     * @param pushHotelBody PushHotelBody
     * @return R
     */
    @PostMapping(value = "hotel_all")
    public R hotelAll(@RequestBody PushHotelBody pushHotelBody) {
        // 放入数据库
        Map<String, Object> map = new HashMap<>(3);
        // 根据酒店查询渠道  美团数据源有两个渠道 在酒店初始化时 酒店信息入库时也会入库 酒店所属的数据源渠道
        DtOrderChannels dtOrderChannels = dtOrderChannelsService.queryByHotelId(pushHotelBody.getHotelId());
        map.put("partnerId", dtOrderChannels.getPartnerId());
        map.put("accessKey", dtOrderChannels.getAccessKey());
        map.put("secretKey", dtOrderChannels.getSecret());
        // 酒店信息入库
        hotelService.saveHotel(pushHotelBody, map);
        // 酒店下的房型入库
        List<Long> listRoom = hotelService.saveRoom(pushHotelBody, map);
        // 房型下的rp入库
        List<Long> listRp = hotelService.saveRp(pushHotelBody, listRoom, map);
        // 价格入缓存
        hotelService.rates(pushHotelBody, listRp, map);
        return R.ok();
    }

    /**
     * 分页拉取酒店列表（拉取酒店数据之前 首先先拉取酒店id）
     *
     * @param params k dtId v value
     * @return R
     */
    @GetMapping(value = "pull_all")
    public R pullAll(@RequestParam Map<String, Object> params) {
        // dtId 控制数据权限
        Integer dtId = Integer.valueOf(params.get("dtId").toString());
        // 拉取酒店Id列表
        hotelService.repeat(dtId);
        return R.ok();
    }

    /**
     * 全量更新 rp
     * @param params
     * @return
     */
    @GetMapping(value = "update_rp")
    public R updateRp(@RequestParam Map<String, Object> params) {
        Long hotelId = Long.valueOf(params.get("hotelId").toString());
        // 放入数据库
        Map<String, Object> map = new HashMap<>(3);
        // 根据酒店查询渠道  美团数据源有两个渠道 在酒店初始化时 酒店信息入库时也会入库 酒店所属的数据源渠道
        DtOrderChannels dtOrderChannels = dtOrderChannelsService.queryByHotelId(hotelId);
        map.put("partnerId", dtOrderChannels.getPartnerId());
        map.put("accessKey", dtOrderChannels.getAccessKey());
        map.put("secretKey", dtOrderChannels.getSecret());
        hotelService.updateRp(hotelId, map);
        return R.ok();
    }

}
