package com.qls.mt.biz.data.service;


import com.alibaba.fastjson.JSONObject;
import com.qls.core.utils.DateUtils;
import com.qls.mt.biz.common.utils.FxBaseApiRequest;
import com.qls.mt.biz.common.utils.MtClient;
import com.qls.mt.biz.data.pojo.rp.BookRules;
import com.qls.mt.biz.data.pojo.rp.Breakfast;
import com.qls.mt.biz.data.pojo.rp.CancelRules;
import lombok.extern.slf4j.Slf4j;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 酒店数据处理的通用工具
 *
 * @author doyoung
 */
@Slf4j
public class HotelUtils {

    /**
     * 接收美团返回值
     *
     * @param data JSONObject
     * @param method String
     * @param map Map<String, Object>
     * @return String
     */
    public static String result(JSONObject data, String method, Map<String, Object> map) {
        FxBaseApiRequest fxBaseApiRequest = new FxBaseApiRequest(Integer.valueOf(map.get("partnerId").toString()),
                map.get("secretKey").toString(),
                map.get("accessKey").toString(),
                method, data.toJSONString(), false, false);
        String result = new MtClient().retryPost(fxBaseApiRequest);
        if(result == null) {
            return null;
        }
        JSONObject hotelData = JSONObject.parseObject(result);
        if (!(hotelData.getInteger("code").equals(0))) {
            log.error("数据源：{}，调用接口：{}，请求参数：{}，返回结果：{}", map.get("partnerId").toString(), fxBaseApiRequest.getMethod() ,data.toJSONString(), result);
        }
        return hotelData.getString("result");
    }

    /**
     * 预定政策
     *
     * @param bookRulesList List<BookRules>
     * @return Map<String, Object>
     */
    public static Map<String, Object> bookRule(List<BookRules> bookRulesList) {
        Map<String, Object> map = new HashMap<>();
        Integer minDays = 1;
        Integer maxDays = 90;
        Integer minBookHours = 0;
        Integer maxBookHours = 720;
        // 默认日期
        Date nowDate = new Date();
        Date startDate = DateUtils.addDateDays(nowDate, -1);
        Date endDate = DateUtils.addDateYears(nowDate, 1);
        String startTime = DateUtils.format(startDate);
        String endTime = DateUtils.format(endDate);
        // 为空默认值
        if (bookRulesList != null && !bookRulesList.isEmpty()) {
            for (BookRules bookRules : bookRulesList) {
                if(bookRules.getSerialCheckinMin() != null){
                    minDays = bookRules.getSerialCheckinMin();
                }
                Integer start = bookRules.getInStartDate();
                Integer end = bookRules.getInEndDate();
                // 0 标识默认生效规则
                if (start != 0) {
                    startDate = DateUtils.stringToDate(DateUtils.dateChange(start.toString()), DateUtils.DATE_PATTERN);
                }
                if (end != 0) {
                    endDate = DateUtils.stringToDate(DateUtils.dateChange(end.toString()), DateUtils.DATE_PATTERN);
                }
                if (DateUtils.isEffectiveDate(nowDate, startDate, endDate)) {
                    Integer bookDaysEarliest = bookRules.getEarliestBookingDays();
                    if (bookDaysEarliest != null && bookDaysEarliest != 0 && bookDaysEarliest != -1) {
                        maxBookHours = 24 * bookDaysEarliest;
                    }
                    String bookHoursEarliest = bookRules.getEarliestBookingHours();
                    if (bookHoursEarliest != null) {
                        maxBookHours += 24 - Integer.parseInt(bookHoursEarliest.substring(0, 2));
                    }
                    Integer bookDaysLatest = bookRules.getLatestBookingDays();
                    if (bookDaysLatest != null && bookDaysLatest != -1 && bookDaysLatest != 0) {
                        minBookHours = 24 * bookDaysLatest;
                    }
                    startTime = DateUtils.format(startDate);
                    endTime = DateUtils.format(endDate);
                    break;
                }
            }
        }
        map.put("minDays", minDays);
        map.put("maxDays", maxDays);
        map.put("minBookHours", minBookHours);
        map.put("maxBookHours", maxBookHours);
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        return map;
    }

    /**
     * 退改政策
     * {"cancelPolicyType":2} 不可取消
     * {"cancelPolicyType":1} 任意取消
     * {"cancelPolicyType":5,"policyInfo":{"timeBefore":6}}表示从入住日24点往前推至少6个小时即入住日18点前可免费取消
     *
     * @param cancelRulesList List<CancelRules>
     * @return String
     */
    public static String cancelRule(List<CancelRules> cancelRulesList) {
        JSONObject cancelPolicy = new JSONObject();
        // 默认不可取消
        cancelPolicy.put("cancelPolicyType", 2);
        if (cancelRulesList != null && !cancelRulesList.isEmpty()) {
            CancelRules cancelRules = cancelRulesList.get(0);
            Integer cancelType = cancelRules.getCancelType();
            Integer aheadCancelDays = cancelRules.getAheadCancelDays();
            Integer deductType = cancelRules.getDeductType();
            String aheadCancelHours = cancelRules.getAheadCancelHours();
            if (cancelType == 1 && aheadCancelDays != null && deductType != null && aheadCancelHours != null) {
                int hours;
                if (deductType == 0) {
                    hours = 24 - Integer.parseInt(aheadCancelHours.substring(0, 2));
                } else {
                    hours = Integer.parseInt(aheadCancelHours);
                }
                hours += 24 * aheadCancelDays;
                cancelPolicy.put("cancelPolicyType", 5);
                JSONObject policyInfo = new JSONObject();
                policyInfo.put("timeBefore", hours);
                cancelPolicy.put("policyInfo", policyInfo);
            }
        }
        return JSONObject.toJSONString(cancelPolicy);
    }

    /**
     * 早餐
     *
     * @param breakfasts List<Breakfast>
     * @return Map<String, Object>
     */
    public static Map<String, Object> breakfast(List<Breakfast> breakfasts) {
        Map<String, Object> map = new HashMap<>();
        int breakfastNum = 0;
        JSONObject breakfastCal = null;
        List<JSONObject> breakfastCals = new ArrayList<>();
        if (breakfasts != null && !breakfasts.isEmpty()) {

            Breakfast breakfast = breakfasts.get(0);
            Integer breakfastType = breakfast.getBreakfastType();
            if (breakfastType == 1) {
                // 统一早餐
                breakfastNum = breakfast.getBreakfastNum();
                for (Breakfast b : breakfasts) {
                    // 日历早餐
                    Integer breakfastNumCal = b.getBreakfastNum();
                    Integer startDate = b.getInStartDate();
                    Integer endDate = b.getInEndDate();
                    if (startDate != 0 && endDate != 0) {
                        breakfastCal = new JSONObject();
                        breakfastCal.put("startDate", DateUtils.dateChange(startDate.toString()));
                        breakfastCal.put("endDate", DateUtils.dateChange(endDate.toString()));
                        breakfastCal.put("breakfast_count", breakfastNumCal);
                        breakfastCals.add(breakfastCal);
                    }
                }
            }
        }
        map.put("breakfastNum", breakfastNum);
        if (!breakfastCals.isEmpty()) {
            map.put("breakfastCals", JSONObject.toJSONString(breakfastCals));
        }
        return map;
    }

    /**
     * 获取最近30天日期
     *
     * @return Map<String, Object>
     */
    public static Map<String, String> nowAndAfterDay() {
        return nowAndAfterDay(30);
    }

    /**
     * 获取最近x天日期
     *
     * @return Map<String, Object>
     */
    public static Map<String, String> nowAndAfterDay(int days) {
        Map<String, String> map = new HashMap<>(2);
        Date now = new Date();
        String startDate = DateUtils.format(now);
        Date nowAfter30 = DateUtils.addDateDays(now, days);
        String endDate = DateUtils.format(nowAfter30);
        map.put("startDate", startDate);
        map.put("endDate", endDate);
        return map;
    }

    /**
     * 产品名称过滤非法字符
     *
     * @param str String
     * @return String
     */
    public static String goodsNameScreen(String str) {
        String regex = "[\\u4E00-\\u9FA5A-Za-z0-9]";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(str);
        StringBuilder string = new StringBuilder();
        while (matcher.find()) {
            string.append(matcher.group(0));
        }
        String name = string.toString();
        if ("".equals(name)) {
            name = "全额预付";
        }
        return name;
    }



}
