package com.qls.mt.biz.data.service.impl;


import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qls.core.constant.Constant;
import com.qls.core.dto.*;
import com.qls.core.entity.DtOrderChannels;
import com.qls.core.entity.HtHotel;
import com.qls.core.entity.HtSellRoom;
import com.qls.core.utils.DateUtils;
import com.qls.mt.biz.common.service.HtHotelinfoService;
import com.qls.mt.biz.common.service.HtLogicRoomService;
import com.qls.mt.biz.common.service.OpHtlGroupService;
import com.qls.mt.biz.common.utils.FxBaseApiRequest;
import com.qls.mt.biz.common.utils.MtClient;
import com.qls.mt.biz.data.pojo.Rp;
import com.qls.mt.biz.data.pojo.rp.*;
import com.qls.mt.biz.data.service.ChangeService;
import com.qls.mt.biz.data.service.HotelUtils;
import com.qls.mt.module.callback.*;
import com.qls.rabbitmq.SendMsg;
import com.qls.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;


/**
 * @author doyoung
 */
@Slf4j
@Service
@SuppressWarnings("unchecked")
public class ChangeServiceImpl implements ChangeService {

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private HtLogicRoomService htLogicRoomService;

    @Autowired
    private OpHtlGroupService opHtlGroupService;

    @Autowired
    private SendMsg sendMsg;

    @Autowired
    private HtHotelinfoService htHotelinfoService;

    @Override
    public void poi(PoiChangeBody poiChangeBody, Integer partnerId) {
        List<PoiChangeInfo> changeInfos = poiChangeBody.getChangeInfos();
        // 只需要1、3、9 变化
        if (changeInfos == null) {
            log.warn("【酒店变更通知】酒店变更消息为null");
            return;
        }
        // 变更类型 1：酒店上线 2：酒店下线 3：酒店基本信息 4：酒店详情信息 5：酒店政策信息 6：酒店设施信息 7：酒店图片信息 8：酒店通知信息 9：酒店房型信息
        for (PoiChangeInfo c : changeInfos) {
            Long hotelId = c.getHotelId();
            List<Integer> storeIds = opHtlGroupService.getStoreIdByHotelId(hotelId, partnerId);
            if (storeIds.isEmpty()) {
                // 酒店不存在推送店铺中
                log.info("【酒店变更通知】酒店：{}不存在推送店铺中或未上架", hotelId);
                continue;
            }
            List<Integer> changeTypes = c.getChangeTypes();
            // 酒店上线
            if (changeTypes.contains(1)) {
                log.info("【酒店变更通知】酒店：{}，上线", JSON.toJSON(poiChangeBody));
                updateAndPushHotel(hotelId, 0, storeIds);
                continue;
            }
            // 酒店下线
            if (changeTypes.contains(2)) {
                log.info("【酒店变更通知】酒店：{}，下线", JSON.toJSON(poiChangeBody));
                updateAndPushHotel(hotelId, -2, storeIds);
                continue;
            }
            // 酒店房型信息变更 暂时不进行操作 看具体情况
            if (changeTypes.contains(9)) {
                log.info("酒店：{}，房型信息变化", JSON.toJSON(poiChangeBody));
            }
        }
    }

    /**
     * 酒店更新
     * 酒店状态 0:正常，-1:删除，-2:停售
     *
     * @param hotelId  酒店id
     * @param status   酒店状态
     * @param storeIds 酒店推送的店铺
     */
    private void updateAndPushHotel(Long hotelId, Integer status, List<Integer> storeIds) {
        HtHotel htHotel = new HtHotel();
        htHotel.setId(hotelId);
        htHotel.setHotelStatus(status);
        htHotelinfoService.updateById(htHotel);
        for (Integer storeId : storeIds) {
            PushHotelBody pushHotelBody = new PushHotelBody();
            pushHotelBody.setHotelId(hotelId);
            pushHotelBody.setStoreId(storeId);
            // 2 对酒店进行更新
            pushHotelBody.setType(2);
            sendMsg.execute(Constant.RabbitMq.HOTEL, storeId, pushHotelBody);
        }
    }

    @Override
    public void productIdFilter(ProductChangeBody productChangeBody, Integer partnerId) {
        List<ProductChangeInfo> changeInfos = productChangeBody.getChangeInfos();
        for (ProductChangeInfo c : changeInfos) {
            //  判断是否此产品对应酒店是否下架，下架则跳过更新
            Boolean isDown =  redisUtil.judeDownOrNot(Constant.Redis.DOWN_HOTEL.getValue(), Convert.toStr(c.getHotelId()));
            if (isDown){
                continue;
            }
            Long productId = c.getProductId();
            // 判断rp是否推送到店铺 和 rp是否是对应的美团渠道
            Map<String, Object> rpCache = (Map<String, Object>) redisUtil.hashGet(Constant.Redis.RP_BL_STORE.getValue(),
                    productId.toString());
            if (Objects.isNull(rpCache) || !rpCache.get("partnerId").equals(partnerId)) {
                continue;
            }
            ProductChangeMQ productChangeMQ = new ProductChangeMQ();
            // 这个酒店推送过 哪些店铺
            List<Integer> storeIds = (List<Integer>) rpCache.get("storeIds");
            // 101：产品上线
            // 102：产品下线
            // 103：产品基本信息
            // 104：产品价格信息
            // 105：产品房态信息
            List<Integer> changeTypes = c.getChangeTypes();
            // 产品上下线情况
            if (changeTypes.contains(102) || changeTypes.contains(101)) {

                // 飞猪状态 1 开启  2 关闭
                int status = 1;
                if (changeTypes.contains(101)) {
                    log.info("【产品变更通知】产品：{} 上线，msg：{}", partnerId, JSON.toJSON(c));
                } else {
                    status = 2;
                    log.info("【产品变更通知】产品：{} 下线，msg：{}", partnerId, JSON.toJSON(c));
                }
                HtSellRoom htSellRoom = new HtSellRoom();
                htSellRoom.setRpStatus(status);
                htSellRoom.setId(c.getProductId());
                updateAndPushRp(htSellRoom, partnerId);
                return;
            }
            productChangeMQ.setChangeEnd(c.getChangeCheckoutDate());
            productChangeMQ.setChangeStart(c.getChangeCheckinDate());
            productChangeMQ.setChangeTypes(changeTypes);
            productChangeMQ.setGoodsId(productId);
            productChangeMQ.setStoreIds(storeIds);
            productChangeMQ.setPartnerId(partnerId);
            sendMsg.execute(Constant.RabbitMq.PRODUCT_CHANGE, null, productChangeMQ);
        }
    }

    @Override
    public void productBatch(ProductChangeMQBatch productChangeMQBatch) {
        //101：产品上线
        //102：产品下线
        //103：产品基本信息
        //104：产品价格信息
        //105：产品房态信息
        // 产品上下线会做单独的操作 不在这里进行
        List<ProductChangeMQ> productChangeMQS = productChangeMQBatch.getProductChangeMQS();
        JSONObject data = new JSONObject();

        List<Long> goodsIds = new ArrayList<>();
        // 取出所有的产品
        productChangeMQS.forEach(item -> goodsIds.add(item.getGoodsId()));
        data.put("goodsIds", goodsIds);
        Date date = new Date();
        data.put("checkinDate", DateUtils.format(date));
        data.put("checkoutDate", DateUtils.format(DateUtils.addDateDays(date, 30)));
        List<String> queryInfoDimensions = new ArrayList<>();
        // 需要所有信息
        queryInfoDimensions.add("BASE");
        queryInfoDimensions.add("PRICE");
        queryInfoDimensions.add("RS");
        // 只要全日房
        data.put("goodsType", 1);
        data.put("queryInfoDimensions", queryInfoDimensions);
        DtOrderChannels dtOrderChannels = (DtOrderChannels) redisUtil.hashGet("channel",
                productChangeMQBatch.getPartnerId().toString());
        Map<String, Object> mapStore = new HashMap<>(3);
        mapStore.put("partnerId", productChangeMQBatch.getPartnerId().toString());
        mapStore.put("accessKey", dtOrderChannels.getAccessKey());
        mapStore.put("secretKey", dtOrderChannels.getSecret());
        String resp = resultV1(data, "hotel.goods.rp", mapStore);
        Rp rp = JSONObject.parseObject(resp, Rp.class);
        if (rp == null || rp.getHotelGoods() == null) {
            log.warn("【产品变更批量消费】批量拉取产品信息时返回数据为空，产品库存全部更新为0，req：{} ，resp：{}", data, resp);
            this.clearInventory(goodsIds);
            return;
        }
        // 酒店产品列表
        List<HotelGoods> hotelGoodsList = rp.getHotelGoods();
        for (HotelGoods hotelGoods : hotelGoodsList) {
            // 产品列表
            List<Goods> goodsList = hotelGoods.getGoods();
            if (goodsList == null || goodsList.isEmpty()) {
                continue;
            }
            for (Goods goods : goodsList) {
                Long goodsId = goods.getGoodsId();
                goodsIds.remove(goodsId);
                List<Integer> types = this.getTypes(goodsId, productChangeMQS);
                this.parsingAndPush(goods, types, productChangeMQBatch.getPartnerId());
            }
        }
        // goodsList 可能为空 意味着有的产品数据没有拉取到 库存更新为零
        if (!goodsIds.isEmpty()) {
            log.warn("【产品变更批量消费】批量拉取产品信息时goodsIds:{} 数据为空，产品库存全部更新为0，req：{}",
                    goodsIds, data);
            this.clearInventory(goodsIds);
        }
    }

    /**
     * 修改推送rp
     *
     * @param htSellRoom 售卖房型信息
     * @param partnerId  数据源渠道id
     */
    private void updateAndPushRp(HtSellRoom htSellRoom, Integer partnerId) {

        htLogicRoomService.updateById(htSellRoom);
        PushRpBody pushRpBody = new PushRpBody();
        pushRpBody.setRpCode(htSellRoom.getId());
        pushRpBody.setPartnerId(partnerId);
        // 1 指产品需要更新
        pushRpBody.setType(1);
        // 放入队列 会进行更新
        Map<String, Object> stores = (Map<String, Object>) redisUtil.hashGet(Constant.Redis.RP_BL_STORE.getValue(), pushRpBody.getRpCode().toString());
        List<Integer> storeIds = (List<Integer>) stores.get("storeIds");
        for (Integer store : storeIds) {
            pushRpBody.setStoreId(store);
            sendMsg.execute(Constant.RabbitMq.RP, store, pushRpBody);
        }
    }

    /**
     * 清除库存
     *
     * @param goodsIds 产品id集合
     */
    private void clearInventory(List<Long> goodsIds) {
        Date now = new Date();
        for (Long goodsId : goodsIds) {
            List<HtDailyRoom> htDailyRoomList = redisUtil.getRatesBatch(DateUtils.getBetweenDays(DateUtils.format(now),
                    DateUtils.format(DateUtils.addDateDays(now, 30))), goodsId.toString());
            htDailyRoomList.forEach(item -> item.setSaleNum(0));
            this.pushRates(htDailyRoomList, goodsId);
        }
    }

    /**
     * 数据解析推送
     *
     * @param goods       Goods
     * @param changeTypes List<Integer>
     */
    private void parsingAndPush(Goods goods, List<Integer> changeTypes, Integer partnerId) {
        //103：产品基本信息
        //104：产品价格信息
        //105：产品房态信息

        // 全部都变化
        if (changeTypes.contains(103) && changeTypes.contains(104) && changeTypes.contains(105)) {
            // 推送rp
            pushRp(goods, partnerId);
            // 推送价态
            List<HtDailyRoom> htDailyRoomList = this.getRates(goods);
            this.pushRates(htDailyRoomList, goods.getGoodsId());
            return;
        }
        if (changeTypes.contains(103) && (changeTypes.contains(104) || changeTypes.contains(105))) {
            // 推送rp
            pushRp(goods, partnerId);
            // 推送价态
            List<HtDailyRoom> htDailyRoomList = this.getRates(goods);
            this.pushRates(htDailyRoomList, goods.getGoodsId());
            return;
        }
        // 只有基本信息变化
        if (changeTypes.contains(103) && !changeTypes.contains(104) && !changeTypes.contains(105)) {
            // 推送rp
            pushRp(goods, partnerId);
            return;
        }
        // 不用判断价格变化 和状态变化是否在一起  推送时都需要一起推送
        // 基本信息没有变化
        if (!changeTypes.contains(103) && (changeTypes.contains(104) || changeTypes.contains(105))) {
            // 推送价态
            List<HtDailyRoom> htDailyRoomList = this.getRates(goods);
            this.pushRates(htDailyRoomList, goods.getGoodsId());
            return;
        }
        log.warn("【产品变化解析推送】判断条件不足，types：{}", changeTypes);
    }

    /**
     * rp推送
     *
     * @param goods     Goods
     * @param partnerId 数据源渠道的id
     */
    private void pushRp(Goods goods, Integer partnerId) {
        HtSellRoom htSellRoom = getSimpleSellRoom(goods);
//      暂时屏蔽最小连住数大于1的rp推送
        if (htSellRoom.getMinDays() > 1) {
            return;
        }
        htSellRoom.setMinDays(1);
        Integer breakfast = htSellRoom.getBreakfastNum();
        Long rpCode = htSellRoom.getId();
        QueryWrapper<HtSellRoom> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", rpCode).eq("", 2);
        HtSellRoom byId = htLogicRoomService.getById(rpCode);
        // 如果早餐一样则不进行更新 （主要更新早餐）
        if (byId.getBreakfastNum() == breakfast) {
            return;
        }
        log.info("【rp变更】原数据：{}，现数据：{}", JSON.toJSON(byId), JSON.toJSON(htSellRoom));
        updateAndPushRp(htSellRoom, partnerId);
    }

    /**
     * 价态推送方法
     *
     * @param dailyRooms List<HtDailyRoom>
     * @param rpCode     Long
     */
    private void pushRates(List<HtDailyRoom> dailyRooms, Long rpCode) {
        Map<String, Object> mapStoreIds = (Map<String, Object>) redisUtil.hashGet(Constant.Redis.RP_BL_STORE.getValue(), rpCode.toString());
        if (mapStoreIds == null) {
            log.warn("【价态增量推送】产品：{}未找到对应的店铺", rpCode);
            return;
        }
        List<Integer> storeIds = (List<Integer>) mapStoreIds.get("storeIds");
        if (storeIds == null || storeIds.isEmpty()) {
            log.warn("【价态增量更新】产品：{}未找到对应的店铺", rpCode);
            return;
        }
        redisUtil.saveOrUpdateBatchRates(dailyRooms);
        for (Integer id : storeIds) {
            PushRatesBody pushRatesBody = new PushRatesBody();
            pushRatesBody.setHtDailyRooms(dailyRooms);
            pushRatesBody.setRpCode(rpCode);
            sendMsg.execute(Constant.RabbitMq.RATES, id, pushRatesBody);
        }
    }

    /**
     * 获取价格 价态
     *
     * @param goods Goods
     * @return List<HtDailyRoom>
     */
    private List<HtDailyRoom> getRates(Goods goods) {
        List<HtDailyRoom> htDailyRoomList = new ArrayList<>();
        List<com.qls.mt.biz.data.pojo.rp.PriceModels> priceModelsList = goods.getPriceModels();
        // 价格赋值  由于美团系统原因  会出现价格小于0的情况 (某个产品的价格日历全部小于0)
        List<Long> bedGoodsIds = new ArrayList<>();
        if (priceModelsList != null && !priceModelsList.isEmpty()) {
            for (com.qls.mt.biz.data.pojo.rp.PriceModels priceModels : priceModelsList) {
                if (priceModels.getSalePrice() < 0) {
                    Long goodsId = goods.getGoodsId();
                    bedGoodsIds.add(goodsId);
                    log.warn("【价态增量更新】商品：{}美团获取价格小于0，data：{}", goodsId, JSON.toJSON(goods));
                    break;
                }
                HtDailyRoom htDailyRoom = new HtDailyRoom();
                htDailyRoom.setDate(DateUtils.format(priceModels.getDate()));
                htDailyRoom.setPrice(priceModels.getSalePrice());
                htDailyRoom.setRpCode(goods.getGoodsId().toString());
                htDailyRoom.setSubPrice(priceModels.getSubPrice());
                htDailyRoomList.add(htDailyRoom);
            }
        }
        List<GoodsStatus> goodsStatuses = goods.getGoodsStatuses();
        // 给对应的价格赋值房态
        if (goodsStatuses != null && !goodsStatuses.isEmpty()) {
            for (HtDailyRoom htDailyRoom : htDailyRoomList) {
                String pDate = htDailyRoom.getDate();
                for (GoodsStatus goodsStatus : goodsStatuses) {
                    String sDate = goodsStatus.getDate();
                    if (pDate.equals(sDate)) {
                        htDailyRoom.setSaleNum(goodsStatus.getStatus() == 1 ? 3 : 0);
                        htDailyRoom.setStatus(goodsStatus.getStatus() == 1 ? 1 : 0);
                        Integer price = htDailyRoom.getPrice();
                        if (price < 0) {
                            htDailyRoom.setStatus(0);
                        }
                        break;
                    }
                }
            }
        }
        // 产品价格小于0的产品价格对应的库存全部清零
        if (!bedGoodsIds.isEmpty()) {
            clearInventory(bedGoodsIds);
        }
        return htDailyRoomList;
    }

    /**
     * 获取产品的 变化类型
     *
     * @param goodsId          Long
     * @param productChangeMQS List<ProductChangeMQ>
     * @return List<Integer>
     */
    private List<Integer> getTypes(Long goodsId, List<ProductChangeMQ> productChangeMQS) {
        for (ProductChangeMQ productChangeMQ : productChangeMQS) {
            if (goodsId.equals(productChangeMQ.getGoodsId())) {
                return productChangeMQ.getChangeTypes();
            }
        }
        return new ArrayList<>();
    }

    /**
     * 获取价格政策
     *
     * @param goods Goods
     * @return HtSellRoom
     */
    private HtSellRoom getSimpleSellRoom(Goods goods) {
        //支付方式 0：预付，2：现付非担保  数据库： 支付类型，只支持：1：预付5：现付6: 信用住7:在线预约8:在线预约信用住。其中5,6,7,8三种类型需要申请权限
        // 不是预付 剔除
        HtSellRoom htLogicRoom = new HtSellRoom();
        Long rpCode = goods.getGoodsId();
        //	产品ID
        htLogicRoom.setId(rpCode);
        List<Breakfast> breakfastList = goods.getBreakfast();
        Map<String, Object> breakfastMap = HotelUtils.breakfast(breakfastList);
        String breakfastCals = (String) breakfastMap.get("breakfastCals");
        int breakfastNum = (int) breakfastMap.get("breakfastNum");
        htLogicRoom.setBreakfastNum(breakfastNum);
        htLogicRoom.setBreakfastCal(breakfastCals);
        // 支付方式 0：预付，2：现付非担保 飞猪预付是 1：预付 5：现付 6: 信用住7:在线预约8:在线预约信用住。其中5,6,7,8三种类型需要申请权限
        int type = 1;
        if (goods.getPaymentType() == null || goods.getPaymentType() != 0) {
            type = 5;
        }

        if (type == 5) {
            // 2 在飞猪为关闭
            htLogicRoom.setRpStatus(2);
            htLogicRoom.setPaymentType(type);
        }
        Map<String, Object> bookRuleMap = HotelUtils.bookRule(goods.getBookRules());
        //  连续入住最少天数，0表示不限制
        htLogicRoom.setMinDays(Integer.parseInt(bookRuleMap.get("minDays").toString()));
        //	连续入住最多天数，0表示不限制
        htLogicRoom.setMaxDays(Integer.parseInt(bookRuleMap.get("maxDays").toString()));
        //  最早预订时间，格式为HH:mm:ss, 空值表示不限制。
        htLogicRoom.setMinHours(Integer.parseInt(bookRuleMap.get("minBookHours").toString()));
        //  最晚预订时间，格式为HH:mm:ss, 空值表示不限制
        htLogicRoom.setMaxHours(Integer.parseInt(bookRuleMap.get("maxBookHours").toString()));
        htLogicRoom.setStartTime(bookRuleMap.get("startTime").toString());
        htLogicRoom.setEndTime(bookRuleMap.get("endTime").toString());
        htLogicRoom.setCancelPolicy(HotelUtils.cancelRule(goods.getCancelRules()));
        return htLogicRoom;
    }

    /**
     * 接收美团返回值v1版本
     *
     * @param data   JSONObject
     * @param method String
     * @param map    Map<String, Object>
     * @return String
     */
    private String resultV1(JSONObject data, String method, Map<String, Object> map) {
        FxBaseApiRequest fxBaseApiRequest = new FxBaseApiRequest(Integer.valueOf(map.get("partnerId").toString()),
                map.get("secretKey").toString(),
                map.get("accessKey").toString(),
                method,
                data.toJSONString(), false, false);
        String result = new MtClient().post(fxBaseApiRequest);
        JSONObject jsonObject = JSONObject.parseObject(result);
        if (!(jsonObject.getInteger("code").equals(0))) {
            log.error("请求美团返回结果：{}", result);
        }
        return jsonObject.getString("result");
    }

}
