package com.qls.mt.biz.data.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qls.core.constant.Constant;
import com.qls.core.dto.HtDailyRoom;
import com.qls.core.dto.InitBody;
import com.qls.core.dto.PushHotelBody;
import com.qls.core.dto.PushRpBody;
import com.qls.core.entity.*;
import com.qls.core.utils.DateUtils;
import com.qls.mt.biz.common.service.*;
import com.qls.mt.biz.data.pojo.*;
import com.qls.mt.biz.data.pojo.hotel.BaseInfo;
import com.qls.mt.biz.data.pojo.hotel.HotelDetails;
import com.qls.mt.biz.data.pojo.room.*;
import com.qls.mt.biz.data.pojo.rp.*;
import com.qls.mt.biz.data.pojo.status.GoodsStatuses;
import com.qls.mt.biz.data.service.HotelService;
import com.qls.mt.biz.data.service.HotelUtils;
import com.qls.rabbitmq.SendMsg;
import com.qls.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 酒店数据源操作
 *
 * @author doyoung
 */
@Service
@Slf4j
public class HotelServiceImpl implements HotelService {

    @Autowired
    private HtHotelinfoService htHotelinfoService;

    @Autowired
    private HtLogicRoomService htLogicRoomService;

    @Autowired
    private HtPhysicalRoomService htPhysicalRoomService;

    @Autowired
    private DtOrderChannelsService dtOrderChannelsService;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private OpTaskStatusService opTaskStatusService;

    @Autowired
    private SendMsg sendMsg;

    @Override
    public void saveHotel(PushHotelBody pushHotelBody, Map<String, Object> map) {
        // 请求参数
        JSONObject data = new JSONObject();
        ArrayList<Long> hotelIds = new ArrayList<>();
        hotelIds.add(pushHotelBody.getHotelId());
        data.put("hotelIds", hotelIds);
        JSONArray include = new JSONArray();
        include.add("HotelContent.baseInfo");
        include.add("HotelContent.detailInfo");
        data.put("include", include);
        //接收返回值
        String res = HotelUtils.result(data, "hotel.content", map);
        Hotel hotel = JSONObject.parseObject(res, Hotel.class);
        // 返回值为空结束
        if (Objects.isNull(hotel)) {
            log.warn("【酒店拉取】 酒店id：{}的酒店失败, req：{}，resp:{}", pushHotelBody.getHotelId(), data, res);
            return;
        }
        List<HotelDetails> hotelDetailsList = hotel.getHotelContents();
        // 数据为null 或者数据为空 结束
        if (hotelDetailsList == null || hotelDetailsList.isEmpty()) {
            log.warn("【酒店拉取】 酒店id：{}的酒店详情为空, req：{}，resp:{}", pushHotelBody.getHotelId(), data, res);
            return;
        }
        HotelDetails hotelDetail = JSONObject.parseObject(res, Hotel.class).getHotelContents().get(0);
        // 每次对一个酒店请求 只取出第一个即可
        BaseInfo baseInfo = hotelDetail.getBaseInfo();
        // 基础信息为空 结束
        if (Objects.isNull(baseInfo)) {
            log.warn("【酒店拉取】 酒店id：{}的酒店基础信息为空, req：{}，resp:{}", pushHotelBody.getHotelId(), data, res);
            return;
        }
        String phone = baseInfo.getPhone();
        // 如果没有联系电话 结束
        if (StringUtils.isBlank(phone)) {
            log.warn("【酒店拉取】 酒店id：{}的酒店联系电话为空, req：{}，resp:{}", pushHotelBody.getHotelId(), data, res);
            return;
        }
        //创建HtHotelinfo对象并赋值
        HtHotel htHotelinfo = new HtHotel();
        //	酒店ID
        htHotelinfo.setHotelCode(hotelDetail.getHotelId());
        htHotelinfo.setId(hotelDetail.getHotelId());
        //	酒店名称
        htHotelinfo.setHotelName(hotelDetail.getBaseInfo().getName());
        //	酒店地址
        String address = baseInfo.getAddress().getAddressLine().getName();
        htHotelinfo.setPartnerId(Integer.valueOf(map.get("partnerId").toString()));
        htHotelinfo.setAddress(address.length() < 299 ? address : address.substring(0, 299));
        // 城市ID与城市名、城市行政区ID与行政区名的映射关系列表请联系分销平台获取
        htHotelinfo.setProvince(baseInfo.getAddress().getProvince().getCode());
        // 城市行政区ID
        htHotelinfo.setDistrict(baseInfo.getAddress().getArea().getCode());
        // 城市ID
        htHotelinfo.setCity(baseInfo.getAddress().getCity().getCode());
        // 酒店所在商圈名
        if (phone.contains(";")) {
            htHotelinfo.setPhone(phone.split(";")[0]);
        }
        if (phone.contains(",")) {
            htHotelinfo.setPhone(phone.split(",")[0]);
        }
        if (phone.length() > 30) {
            htHotelinfo.setPhone(phone.substring(0, 29));
        }
        htHotelinfo.setPhone(phone);
        Boolean flag = baseInfo.getBookable();
        //酒店营业状态：0 营业中,1 已关门,2 筹建中,3 暂停营业
        Integer status = 0;
        if (flag == null || !flag) {
            status = -2;
        } else {
            status = 0;
        }
        htHotelinfo.setHotelStatus(status);
        // 将数据添加到数据库
        htHotelinfoService.saveOrUpdate(htHotelinfo);
    }

    @Override
    public List<Long> saveRoom(PushHotelBody pushHotelBody, Map<String, Object> map) {
        //创建列表
        List<Long> listRoomIds = new ArrayList<>();
        List<HtRoomType> htPhysicalRooms = new ArrayList<>();
        //请求参数
        JSONObject data = new JSONObject();
        data.put("hotelId", pushHotelBody.getHotelId());
        //接收返回值
        String res = HotelUtils.result(data, "hotel.realroom.content", map);
        Room room = JSONObject.parseObject(res, Room.class);
        // 返回值为空结束
        if (room == null) {
            log.warn("【房型拉取】 酒店：{}，房型为空，, req：{}，resp:{}", pushHotelBody.getHotelId(), data, res);
            return listRoomIds;
        }
        // 将返回值转换成BaseInfo对象
        List<RoomContents> roomContents = room.getRoomContents();

        // 房型基础信息为空结束
        if (roomContents.isEmpty()) {
            log.warn("【房型拉取】 酒店：{}，房型为空，, req：{}，resp:{}", pushHotelBody.getHotelId(), data, res);
            return listRoomIds;
        }
        for (RoomContents roomContent : roomContents) {
            RoomBaseInfo baseInfo = roomContent.getBaseInfo();

            String roomName = baseInfo.getName();
            Integer status = baseInfo.getStatus();
            // 床型信息集合
            List<List<RoomBedGroups>> bedTypeList = baseInfo.getBedGroups();
            if (status.equals(0)) {
                log.warn("【房型拉取】 酒店：{}，不可用，, req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(baseInfo));
                continue;
            }
            if (bedTypeList == null || bedTypeList.isEmpty()) {
                log.warn("【房型拉取】 酒店：{}，床型为空，, req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(baseInfo));
                continue;
            }
            // 房型名称 为null 或者为空字符串 或者房型状态为不可用 剔除
            if (Objects.isNull(roomName) || "".equals(roomName)) {
                log.warn("【房型拉取】 酒店：{}，房型名字为空，, req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(baseInfo));
                continue;
            }
            String bedType = bedType(bedTypeList);
            if (bedType == null) {
                log.warn("【房型拉取】 酒店：{}，床型名字为空，, req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(baseInfo));
                continue;
            }
            HtRoomType htPhysicalRoom = new HtRoomType();
            htPhysicalRoom.setBedType(bedType);
            htPhysicalRoom.setBedWidth(bedWidth(bedTypeList));
            // 房型名称清洗
            roomName = roomNameScreen(roomName);
            // 酒店ID。
            htPhysicalRoom.setHotelCode(baseInfo.getHotelId());
            // 物理房型ID。
            Long roomCode = Convert.toLong(baseInfo.getRealRoomId());
            htPhysicalRoom.setRoomCode(roomCode);
            htPhysicalRoom.setId(roomCode);
            htPhysicalRoom.setRoomName(roomName.length() < 599 ? roomName : roomName.substring(0, 599));
            //	房间最大容纳人数, 可能为空
            String maxNum = null;
            if (!Objects.isNull(baseInfo.getOccupancy())) {
                maxNum = Convert.toStr(baseInfo.getOccupancy().getTotal());
            }
            htPhysicalRoom.setMaxNum(maxNum == null ? "NONE" : maxNum);
            //	房间可用面积, 可能为空
            String roomSize = baseInfo.getUseAbleArea();
            htPhysicalRoom.setRoomSize(roomSize == null ? "NONE" : roomSize);
            //  飞猪：0:无窗/1:有窗/2:部分有窗/3:暗窗/4:部分暗窗
            //  美团：0 有窗/1 部分有窗/2 无窗
            Integer hasWindow = baseInfo.getWindow().getHasWindow();
            htPhysicalRoom.setWindows(hasWindow.equals(2) ? 0 : hasWindow.equals(0) ? 1 : 2);
            //	房间所在楼层
            String roomFloor = baseInfo.getFloor();
            htPhysicalRoom.setRoomFloor(roomFloor == null ? "NONE" : roomFloor);
            //	携程房型状态：0无效, 1 有效  数据库：房型状态。飞猪状态0:正常，-1:删除，-2:停售 （前面已经过滤 剩下的都是有效的房型）
            htPhysicalRoom.setRoomStatus(0);
            htPhysicalRooms.add(htPhysicalRoom);
            listRoomIds.add(roomCode);
        }
        if (!htPhysicalRooms.isEmpty()) {
            htPhysicalRoomService.saveOrUpdateBatch(htPhysicalRooms, 50);
        }
        return listRoomIds;
    }

    /**
     * 获取床型
     * // 床型   飞猪要求--> 英文逗号代表某房型一间房中的多张床 如 “大床，水床” 代表此房型中，
     * 一间房同时有大床和水床 代表某房型包含的不同床型 如“大床/双床” 代表此房型，可能是大床房，也可能是双床房
     * 由于床型过于杂乱 只取第一个有用的床型 窗宽同理
     *
     * @param bedTypeList List<List<RoomBedGroups>>
     * @return 床型
     */
    private String bedType(List<List<RoomBedGroups>> bedTypeList) {
        StringBuilder sb = new StringBuilder();
        for (List<RoomBedGroups> lists : bedTypeList) {
            if (lists.isEmpty()) {
                continue;
            }
            for (RoomBedGroups roomBedGroup : lists) {
                if (StringUtils.isEmpty(roomBedGroup.getType())) {
                    continue;
                }
                sb.append(roomBedGroup.getType()).append(",");
            }
            String bedType = sb.toString();
            if (StringUtils.isNotEmpty(bedType)) {
                return bedType.substring(0, bedType.length() - 1);
            }
        }
        return null;
    }

    /**
     * 获取床宽
     *
     * @param bedTypeList List<List<RoomBedGroups>>
     * @return 窗宽
     */
    private String bedWidth(List<List<RoomBedGroups>> bedTypeList) {
        for (List<RoomBedGroups> lists : bedTypeList) {
            if (lists.isEmpty()) {
                continue;
            }
            for (RoomBedGroups roomBedGroup : lists) {
                String bedWidth = roomBedGroup.getDescription();
                if (StringUtils.isEmpty(bedWidth)) {
                    continue;
                }
                // 去除汉字
                bedWidth = bedWidth.replaceAll("[\u4e00-\u9fa5]", "");
                if (bedWidth.contains("×")) {
                    bedWidth = bedWidth.substring(0, bedWidth.indexOf("×")) + "米";
                }
                if (bedWidth.contains("*")) {
                    bedWidth = bedWidth.substring(0, bedWidth.indexOf("*")) + "米";
                }
                return bedWidth;
            }
        }
        return null;
    }

    @Override
    public List<Long> saveRp(PushHotelBody pushHotelBody, List<Long> listRoom, Map<String, Object> map) {
        List<Long> rpCodes = new ArrayList<>();
        if (listRoom.isEmpty()) {
            return rpCodes;
        }
        // 请求参数
        JSONObject data = new JSONObject();
        ArrayList<Long> hotelIds = new ArrayList<>();
        // 获取当前日期
        hotelIds.add(pushHotelBody.getHotelId());
        data.put("hotelIds", hotelIds);
        Date now = new Date();
        data.put("checkinDate", DateUtils.format(now));
        data.put("checkoutDate", DateUtils.format(DateUtils.addDateDays(now, 1)));
        // 只要全日房
        data.put("goodsType", 1);
        List<String> queryInfoDimensions = new ArrayList<>();
        // 只查询基础信息
        queryInfoDimensions.add("BASE");
        data.put("queryInfoDimensions", queryInfoDimensions);
        // 返回值
        String res = HotelUtils.result(data, "hotel.goods.rp", map);
        Rp rp = JSONObject.parseObject(res, Rp.class);
        if (rp == null) {
            log.warn("【产品拉取】酒店id：{}下产品返回值为空, req：{}，resp:{}", pushHotelBody.getHotelId(), data, res);
            return rpCodes;
        }
        List<HotelGoods> list = rp.getHotelGoods();
        if (list == null || list.isEmpty()) {
            log.warn("【产品拉取】酒店id：{}下产品返回值为空, req：{}，resp:{}", pushHotelBody.getHotelId(), data, res);
            return rpCodes;
        }
        // 产品集合
        List<HtSellRoom> htLogicRooms = new ArrayList<>();
        for (HotelGoods good : list) {
            List<Goods> goodsListoods = good.getGoods();
            if (goodsListoods.isEmpty()) {
                log.warn("【产品拉取】酒店id：{}下产品返回值为空, req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(goodsListoods));
                return rpCodes;
            }
            for (Goods goods : goodsListoods) {
                //支付方式 0：预付，2：现付非担保  数据库： 支付类型，只支持：1：预付5：现付6: 信用住7:在线预约8:在线预约信用住。其中5,6,7,8三种类型需要申请权限
                // 不是预付 剔除
                if (goods.getPaymentType() != 0) {
                    log.warn("【产品拉取】酒店id：{}非预付，过滤，req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(goods));
                    continue;
                }
                // 产品没有名称 剔除
                if (goods.getGoodsName() == null) {
                    log.warn("【产品拉取】酒店id：{}不存在名称，过滤，req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(goods));
                    continue;
                }
//                String goodsName = goods.getGoodsName();
                // 优惠产品过滤
//                if (goodsName.contains("连住") || goodsName.contains("学生") || goodsName.contains("天天")) {
//                    log.warn("【产品拉取】酒店id：{}去除【连住】，【学生】，【天天】等的特惠产品，req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(goods));
//                    continue;
//                }
                // 去除没有物理房型的商品 文档解释 每个产品只会对应一个物理房型 也就是此集合中只可能有一个数据 （做一下判断看具体日志情况）
                if (goods.getRoomInfoList() == null || goods.getRoomInfoList().size() != 1) {
                    log.warn("【产品拉取】酒店id：{}没有对应的物理房型或物理房型个数不为1，过滤，req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(goods));
                    continue;
                }
                // 物理房型id可能不存在
                Long realRoomCode = goods.getRoomInfoList().get(0).getRealRoomId();
                if (realRoomCode == null) {
                    log.warn("【产品拉取】酒店id：{}没有对应的物理房型id，过滤，req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(goods));
                    continue;
                }
                // 此接口中的物理房型在 物理房型接口中不存在 剔除
                if (!listRoom.contains(realRoomCode)) {
                    log.warn("【产品拉取】酒店id：{}对应的物理房型不存在于物理房型接口的物理房型中，过滤，req：{}，resp:{}", pushHotelBody.getHotelId(), data, JSON.toJSON(goods));
                    continue;
                }

                HtSellRoom htLogicRoom = new HtSellRoom();
                // 产品名称过滤
                String goodsName = HotelUtils.goodsNameScreen(goods.getGoodsName());
                // 物理房型ID。
                htLogicRoom.setRoomCode(realRoomCode);
                Long rpCode = goods.getGoodsId();
                // 酒店ID
                htLogicRoom.setHotelCode(pushHotelBody.getHotelId());
                htLogicRoom.setId(goods.getGoodsId());
                //  产品名称
                htLogicRoom.setRpName(goodsName.length() < 299 ? goodsName : goodsName.substring(0, 299));
                //	是否需要用户真实手机号
                htLogicRoom.setIsPhone(goods.getNeedRealTel());
                //	美团此字段已经废弃 用价格的状态进行判断 此处默认为1 （可用）
                htLogicRoom.setRpStatus(1);
                //	免费早餐数，breakfastType=1时非空
                List<Breakfast> breakfastList = goods.getBreakfast();
                Map<String, Object> breakfastMap = HotelUtils.breakfast(breakfastList);
                String breakfastCals = (String) breakfastMap.get("breakfastCals");
                int breakfastNum = (int) breakfastMap.get("breakfastNum");
                htLogicRoom.setBreakfastNum(breakfastNum);
                htLogicRoom.setBreakfastCal(breakfastCals);
                // 飞猪预付是 1
                htLogicRoom.setPaymentType(1);
                Map<String, Object> bookRuleMap = HotelUtils.bookRule(goods.getBookRules());
                //  连续入住最少天数，0表示不限制
                htLogicRoom.setMinDays(Integer.parseInt(bookRuleMap.get("minDays").toString()));
                //	连续入住最多天数，0表示不限制
                htLogicRoom.setMaxDays(Integer.parseInt(bookRuleMap.get("maxDays").toString()));
                //  最早预订时间，格式为HH:mm:ss, 空值表示不限制。
                htLogicRoom.setMinHours(Integer.parseInt(bookRuleMap.get("minBookHours").toString()));
                //  最晚预订时间，格式为HH:mm:ss, 空值表示不限制
                htLogicRoom.setMaxHours(Integer.parseInt(bookRuleMap.get("maxBookHours").toString()));
                htLogicRoom.setStartTime(bookRuleMap.get("startTime").toString());
                htLogicRoom.setEndTime(bookRuleMap.get("endTime").toString());
                htLogicRoom.setCancelPolicy(HotelUtils.cancelRule(goods.getCancelRules()));
                // 暂时屏蔽最小连住数大于1的rp推送
                if (htLogicRoom.getMinDays() > 1) {
                    log.warn("【产品拉取】酒店id：{}，暂时屏蔽最小连住数大于1的rp推送，过滤，rpCode：{}", pushHotelBody.getHotelId(), rpCode);
                    continue;
                }
                htLogicRoom.setMinDays(1);
                //	产品ID
                htLogicRoom.setRpCode(rpCode);
                rpCodes.add(rpCode);
                htLogicRooms.add(htLogicRoom);
            }
        }
        if (!htLogicRooms.isEmpty()) {
            htLogicRoomService.saveOrUpdateBatch(htLogicRooms, 50);
        }
        return rpCodes;
    }

    @Override
    public void rates(PushHotelBody pushHotelBody, List<Long> rpCodes, Map<String, Object> map) {
        if (rpCodes.isEmpty()) {
            return;
        }
        List<HtDailyRoom> htDailyrooms = new ArrayList<>();
        //请求参数hotel.goods.rp
        JSONObject data = new JSONObject();
        ArrayList<Long> hotelIds = new ArrayList<>();
        hotelIds.add(pushHotelBody.getHotelId());
        // 默认拉取三十天的价态
        Map<String, String> mapDate = HotelUtils.nowAndAfterDay();
        data.put("hotelIds", hotelIds);
        data.put("checkinDate", mapDate.get("startDate"));
        data.put("checkoutDate", mapDate.get("endDate"));
        data.put("goodsType", 1);
        ArrayList<String> queryInfoDimensions = new ArrayList<>();
        queryInfoDimensions.add("BASE");
        queryInfoDimensions.add("PRICE");
        queryInfoDimensions.add("RS");
        data.put("queryInfoDimensions", queryInfoDimensions);
        //接收返回值
        String res = HotelUtils.result(data, "hotel.goods.rp", map);
        Rp rp = JSONObject.parseObject(res, Rp.class);
        // 返回值为空结束
        if (rp == null) {
            log.warn("【价格查询】酒店id：{}查询价格返回值为空，req：{}，resp：{}", pushHotelBody.getHotelId(), data, res);
            return;
        }
        List<HotelGoods> HotelGoods = rp.getHotelGoods();
        // 商品为空结束
        if (HotelGoods == null) {
            log.warn("【价格查询】酒店id：{}查询产品返回值为空，req：{}，resp：{}", pushHotelBody.getHotelId(), data, res);
            return;
        }
        for (HotelGoods hotelGood : HotelGoods) {
            List<Goods> goods = hotelGood.getGoods();
            if (goods == null || goods.isEmpty()) {
                log.warn("【价格查询】酒店id：{}查询产品返回值为空，req：{}，resp：{}", pushHotelBody.getHotelId(), data, JSONUtil.toJsonStr(HotelGoods));
                return;
            }
            for (Goods good : goods) {
                Long rpCode = good.getGoodsId();
                if (!rpCodes.contains(rpCode)) {
                    log.warn("【价格查询】酒店id：{}上级接口调用的rpId中不包含此价格rpId，req：{}，resp：{}", pushHotelBody.getHotelId(), data, JSONUtil.toJsonStr(good));
                    continue;
                }
                List<PriceModels> priceModels = good.getPriceModels();
                if (priceModels == null || priceModels.isEmpty()) {
                    log.warn("【价格查询】酒店id：{}，rpId:{}，价格集合为空，req：{}，resp：{}", pushHotelBody.getHotelId(), rpCode, data, JSONUtil.toJsonStr(good));
                    continue;
                }
                for (PriceModels priceModel : priceModels) {
                    if (good.getGoodsStatuses() == null || good.getGoodsStatuses().isEmpty()) {
                        continue;
                    }
                    GoodsStatus goodsStatus = good.getGoodsStatuses().get(0);
                    if (goodsStatus == null) {
                        continue;
                    }
                    HtDailyRoom htDailyroom = new HtDailyRoom();
                    Integer status = goodsStatus.getStatus();
                    int num = 0;
                    if (status.equals(1)){
                        num = 2;
                        htDailyroom.setStatus(1);
                    }else{
                        htDailyroom.setStatus(0);
                    }
                    htDailyroom.setSaleNum(num);
                    htDailyroom.setRpCode(rpCode.toString());
                    htDailyroom.setDate(DateUtils.format(priceModel.getDate()));
                    htDailyroom.setPrice((priceModel.getSalePrice()));
                    htDailyroom.setSubPrice(priceModel.getSubPrice());
                    htDailyrooms.add(htDailyroom);
                }
            }
        }
        // 赋值对应的库存
        List<HtDailyRoom> htDailyRoomList = htDailyrooms;
        redisUtil.saveOrUpdateBatchRates(htDailyRoomList);
    }

    /**
     * 拉取全部酒店ID  时间较久异步操作
     *
     * @param dtId Integer
     */
    @Async
    @Override
    public void repeat(Integer dtId) {
        // 创建任务
        Integer taskCode = Integer.valueOf(System.currentTimeMillis() / 1000 + "");
        OpTaskStatus opTaskStatus = new OpTaskStatus();
        opTaskStatus.setTaskName(Constant.Task.HOTEL_ALL_PULL.getName());
        opTaskStatus.setStartTime(new Date());
        opTaskStatus.setTaskCode(taskCode);
        opTaskStatus.setDtId(dtId);
        opTaskStatusService.save(opTaskStatus);
        // 根据优先级 查询数据源渠道的 appk secret 等
        List<DtOrderChannels> dtOrderChannelsEntities = dtOrderChannelsService.getByPriority();
        // 初始化拉取参数
        Long maxId;
        Map<Integer, DtOrderChannels> dtOrderChannelsMap = new HashMap<>();
        dtOrderChannelsEntities.forEach(dtOrderChannels -> dtOrderChannelsMap.put(dtOrderChannels.getPartnerId(), dtOrderChannels));
        Map<Long, Integer> hotelMap = new HashMap<>();
        // 遍历渠道 进行酒店id分页拉取
        for (DtOrderChannels dtOrderChannelsEntity : dtOrderChannelsEntities) {
            Map<String, Object> map = new HashMap<>(3);
            Integer partnerId = dtOrderChannelsEntity.getPartnerId();
            String secretKey = dtOrderChannelsEntity.getSecret();
            String accessKey = dtOrderChannelsEntity.getAccessKey();
            map.put("partnerId", partnerId);
            map.put("secretKey", secretKey);
            map.put("accessKey", accessKey);
            maxId = 0L;
            while (!maxId.equals(-1L)) {
                maxId = getHotelIds(maxId, map, hotelMap);
            }
        }
        // 酒店id的集合
        List<InitBody> initList = new ArrayList<>();
        for (Map.Entry<Long, Integer> entry : hotelMap.entrySet()) {
            DtOrderChannels dtOrderChannels = dtOrderChannelsMap.get(entry.getValue());
            InitBody initBody = new InitBody();
            initBody.setHotelId(entry.getKey());
            initBody.setAccessKey(dtOrderChannels.getAccessKey());
            initBody.setSecretKey(dtOrderChannels.getSecret());
            initBody.setPartnerId(entry.getValue());
            initBody.setTaskId(taskCode);
            initList.add(initBody);
        }
        OpTaskStatus opTask = new OpTaskStatus();
        opTask.setTotal(initList.size());
        opTaskStatusService.getBaseMapper().update(opTask, new UpdateWrapper<OpTaskStatus>().eq("task_code", taskCode));
        // 任务结束标识 （如果判断酒店id为-1 则可以认为任务停止 主要是写入大概的任务完成时间）
        InitBody end = new InitBody();
        end.setHotelId(-1L);
        initList.add(end);
        // 将酒店Id放入rabbitmq队列
        for (InitBody initBody : initList) {
            initBody.setTaskId(opTaskStatus.getTaskCode());
            // 将消息发送到“INIT”队列中
            sendMsg.execute(Constant.RabbitMq.INIT, 1, initBody);
        }
    }

    @SuppressWarnings("unchecked")
    @Override
    public void updateRp(Long hotelId, Map<String, Object> map) {
        // 查询酒店下所有的rp
        QueryWrapper<HtSellRoom> htSellRoomQueryWrapper = new QueryWrapper<>();
        htSellRoomQueryWrapper.eq("hotel_code", hotelId);
        List<HtSellRoom> list = htLogicRoomService.list(htSellRoomQueryWrapper);
        // 酒店下没有rp时不进行更新
        if (list.isEmpty()) {
            return;
        }
        // 接口中的数据
        List<HtSellRoom> sellRoomApiList = getSellRoom(hotelId, map);
        // 需要进行更新的数据
        List<HtSellRoom> needUpdateList = new ArrayList<>();
        for (HtSellRoom sellRoomApi : sellRoomApiList) {
            for (HtSellRoom sellRoom : new ArrayList<>(list)) {
                // 如果id相等 判断状态是否相等
                if (sellRoomApi.getId().equals(sellRoom.getId())) {
                    // 状态不相等 艺接口中为准
                    if (!sellRoomApi.getRpStatus().equals(sellRoom.getRpStatus())) {
                        sellRoom.setRpStatus(sellRoomApi.getRpStatus());
                        needUpdateList.add(sellRoom);
                    }
                    list.remove(sellRoom);
                }
            }
        }
        // 如果数据库中查询的数据还有剩余 说明这些已经无效
        if (!list.isEmpty()) {
            list.forEach(item -> {
                item.setRpStatus(2);
                needUpdateList.add(item);
            });
        }
        // 批量更新数据库数据
        htLogicRoomService.updateBatchById(needUpdateList);
        log.info("需要进行更新的产品：{}", JSON.toJSON(needUpdateList));
        needUpdateList.forEach(item -> {
            Long rpCode = item.getRpCode();
            // 查询这个rp在哪些店铺推送过
            Map<String, Object> mapCache = (Map<String, Object>) redisUtil.hashGet(Constant.Redis.RP_BL_STORE.getValue(), rpCode + "");
            if (mapCache == null || mapCache.isEmpty()) {
                return;
            }
            List<Integer> storeIds = (List<Integer>) mapCache.get("storeIds");
            Integer partnerId = (Integer) mapCache.get("partnerId");
            if (storeIds == null || storeIds.isEmpty()) {
                return;
            }
            storeIds.forEach(storeId -> {
                PushRpBody pushRpBody = new PushRpBody();
                // 1 代表更新
                pushRpBody.setType(1);
                pushRpBody.setStoreId(storeId);
                pushRpBody.setRpCode(rpCode);
                pushRpBody.setPartnerId(partnerId);
                sendMsg.execute(Constant.RabbitMq.RP, storeId, pushRpBody);
            });
        });
    }

    private List<HtSellRoom> getSellRoom(Long hotelId, Map<String, Object> map) {
        // 调用接口
        // 请求参数
        JSONObject data = new JSONObject();
        ArrayList<Long> hotelIds = new ArrayList<>();
        // 获取当前日期
        hotelIds.add(hotelId);
        data.put("hotelIds", hotelIds);
        Date now = new Date();
        data.put("checkinDate", DateUtils.format(now));
        data.put("checkoutDate", DateUtils.format(DateUtils.addDateDays(now, 1)));
        // 只要全日房
        data.put("goodsType", 1);
        List<String> queryInfoDimensions = new ArrayList<>();
        // 只查询基础信息
        queryInfoDimensions.add("BASE");
        data.put("queryInfoDimensions", queryInfoDimensions);
        // 返回值
        String res = HotelUtils.result(data, "hotel.goods.rp", map);
        Rp rp = JSONObject.parseObject(res, Rp.class);
        if (rp == null) {
            log.warn("【产品拉取】酒店id：{}下产品返回值为空, req：{}，resp:{}", hotelId, data, res);
            return new ArrayList<>();
        }
        List<HotelGoods> list = rp.getHotelGoods();
        if (list == null || list.isEmpty()) {
            log.warn("【产品拉取】酒店id：{}下产品返回值为空, req：{}，resp:{}", hotelId, data, res);
            return new ArrayList<>();
        }
        // 产品集合
        List<HtSellRoom> htLogicRooms = new ArrayList<>();
        for (HotelGoods good : list) {
            List<Goods> goodsListoods = good.getGoods();
            if (goodsListoods.isEmpty()) {
                log.warn("【产品拉取】酒店id：{}下产品返回值为空, req：{}，resp:{}", hotelId, data, JSON.toJSON(goodsListoods));
                return htLogicRooms;
            }
            for (Goods goods : goodsListoods) {
                //支付方式 0：预付，2：现付非担保  数据库： 支付类型，只支持：1：预付5：现付6: 信用住7:在线预约8:在线预约信用住。其中5,6,7,8三种类型需要申请权限
                // 不是预付 剔除
                if (goods.getPaymentType() != 0) {
                    log.warn("【产品拉取】酒店id：{}非预付，过滤，req：{}，resp:{}", hotelId, data, JSON.toJSON(goods));
                    continue;
                }
                // 产品没有名称 剔除
                if (goods.getGoodsName() == null) {
                    log.warn("【产品拉取】酒店id：{}不存在名称，过滤，req：{}，resp:{}", hotelId, data, JSON.toJSON(goods));
                    continue;
                }
                // 去除没有物理房型的商品 文档解释 每个产品只会对应一个物理房型 也就是此集合中只可能有一个数据 （做一下判断看具体日志情况）
                if (goods.getRoomInfoList() == null || goods.getRoomInfoList().size() != 1) {
                    log.warn("【产品拉取】酒店id：{}没有对应的物理房型或物理房型个数不为1，过滤，req：{}，resp:{}", hotelId, data, JSON.toJSON(goods));
                    continue;
                }
                // 物理房型id可能不存在
                Long realRoomCode = goods.getRoomInfoList().get(0).getRealRoomId();
                if (realRoomCode == null) {
                    log.warn("【产品拉取】酒店id：{}没有对应的物理房型id，过滤，req：{}，resp:{}", hotelId, data, JSON.toJSON(goods));
                    continue;
                }
                HtSellRoom htLogicRoom = new HtSellRoom();
                // 产品名称过滤
                String goodsName = HotelUtils.goodsNameScreen(goods.getGoodsName());
                // 物理房型ID。
                htLogicRoom.setRoomCode(realRoomCode);
                Long rpCode = goods.getGoodsId();
                //	产品ID
                htLogicRoom.setRpCode(rpCode);
                // 酒店ID
                htLogicRoom.setHotelCode(hotelId);
                htLogicRoom.setId(goods.getGoodsId());
                //  产品名称
                htLogicRoom.setRpName(goodsName.length() < 299 ? goodsName : goodsName.substring(0, 299));
                //	是否需要用户真实手机号
                htLogicRoom.setIsPhone(goods.getNeedRealTel());
                //	美团此字段已经废弃 用价格的状态进行判断 此处默认为1 （可用）
                htLogicRoom.setRpStatus(1);
                //	免费早餐数，breakfastType=1时非空
                List<Breakfast> breakfastList = goods.getBreakfast();
                Map<String, Object> breakfastMap = HotelUtils.breakfast(breakfastList);
                String breakfastCals = (String) breakfastMap.get("breakfastCals");
                int breakfastNum = (int) breakfastMap.get("breakfastNum");
                htLogicRoom.setBreakfastNum(breakfastNum);
                htLogicRoom.setBreakfastCal(breakfastCals);
                // 飞猪预付是 1
                htLogicRoom.setPaymentType(1);
                Map<String, Object> bookRuleMap = HotelUtils.bookRule(goods.getBookRules());
                //  连续入住最少天数，0表示不限制
                htLogicRoom.setMinDays(Integer.parseInt(bookRuleMap.get("minDays").toString()));
                //	连续入住最多天数，0表示不限制
                htLogicRoom.setMaxDays(Integer.parseInt(bookRuleMap.get("maxDays").toString()));
                //  最早预订时间，格式为HH:mm:ss, 空值表示不限制。
                htLogicRoom.setMinHours(Integer.parseInt(bookRuleMap.get("minBookHours").toString()));
                //  最晚预订时间，格式为HH:mm:ss, 空值表示不限制
                htLogicRoom.setMaxHours(Integer.parseInt(bookRuleMap.get("maxBookHours").toString()));
                htLogicRoom.setStartTime(bookRuleMap.get("startTime").toString());
                htLogicRoom.setEndTime(bookRuleMap.get("endTime").toString());
                htLogicRoom.setCancelPolicy(HotelUtils.cancelRule(goods.getCancelRules()));

                // 暂时屏蔽最小连住数大于1的rp推送
                if (htLogicRoom.getMinDays() > 1) {
                    log.warn("【产品拉取】酒店id：{}，暂时屏蔽最小连住数大于1的rp推送，过滤，rpCode：{}", hotelId, rpCode);
                    continue;
                }
                htLogicRoom.setMinDays(1);
                htLogicRooms.add(htLogicRoom);
            }
        }
        return htLogicRooms;
    }

    /**
     * 拉取酒店ID
     *
     * @param maxId    Long
     * @param map      Map<String, Object>
     * @param hotelMap 酒店集合
     * @return maxId
     */
    private Long getHotelIds(Long maxId, Map<String, Object> map, Map<Long, Integer> hotelMap) {
        JSONObject dataStatus = new JSONObject();
        dataStatus.put("maxId", maxId);
        dataStatus.put("pageSize", 1000);
        String res = HotelUtils.result(dataStatus, "hotel.poi.list", map);
        JSONObject hotelIdsJson = JSONObject.parseObject(res);
        maxId = hotelIdsJson.getLong("maxId");
        JSONArray htIds = hotelIdsJson.getJSONArray("hotelIds");
        if (htIds == null) {
            return maxId;
        }
        List<Long> hotelIds = htIds.toJavaList(Long.class);
        for (Long hotelId : hotelIds) {
            hotelMap.put(hotelId, Integer.valueOf(map.get("partnerId").toString()));
        }
        return maxId;
    }

    /**
     * 房型名字清洗
     *
     * @param str 要清洗的字符串
     * @return 清洗完成的字符串
     */
    private String roomNameScreen(String str) {
        String regEx = "\\(.*\\)|\\[.*\\]|\\（.*\\）|\\【.*\\】|\\(.*\\）|\\（.*\\)";
        str = str.replaceAll(regEx, "");
        return str;
    }

}
