package com.qls.mt.biz.order.service.impl;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.qls.ali.api.AliRemoteService;
import com.qls.core.ali.order.bookrq.request.BookRq;
import com.qls.core.ali.order.bookrq.request.OrderGuest;
import com.qls.core.ali.order.cancelrq.CancelRq;
import com.qls.core.ali.order.common.DailyInfo;
import com.qls.core.ali.order.validaterq.request.ValidateRq;
import com.qls.core.constant.Constant;
import com.qls.core.dto.HtDailyRoom;
import com.qls.core.entity.DtOrderChannels;
import com.qls.core.utils.R;
import com.qls.mt.biz.common.service.DtOrderChannelsService;
import com.qls.mt.biz.common.utils.FxBaseApiRequest;
import com.qls.mt.biz.common.utils.MtClient;
import com.qls.mt.biz.order.service.MtOrderService;
import com.qls.mt.module.dto.book.request.MtBook;
import com.qls.mt.module.dto.book.response.MtBookResult;
import com.qls.mt.module.dto.book.response.Result;
import com.qls.mt.module.dto.cancel.CancelRequest;
import com.qls.mt.module.dto.cancel.CancelResponse;
import com.qls.mt.module.dto.check.PriceModels;
import com.qls.mt.module.dto.check.request.Check;
import com.qls.mt.module.dto.check.response.CheckResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author doyoung
 */
@Slf4j
@Service("mtOrderService")
public class MtOrderServiceImpl implements MtOrderService {

    @Autowired
    private DtOrderChannelsService dtOrderChannelsService;

    @Autowired
    private AliRemoteService aliRemoteService;

    /**
     * 下单前校验(美团）
     *
     * @param validateRq ValidateRq
     * @return R
     */
    @Override
    public R validate(ValidateRq validateRq) {
        Long hotelId = Long.valueOf(validateRq.getHotelId());
        Long rpCode = Long.valueOf(validateRq.getRatePlanCode());
        String method = "hotel.order.check";
        Check checkRequest = new Check();
        checkRequest.setHotelId(Long.valueOf(validateRq.getHotelId()));
        checkRequest.setGoodsId(rpCode);
        checkRequest.setCheckinDate(validateRq.getCheckIn());
        checkRequest.setCheckoutDate(validateRq.getCheckOut());
        checkRequest.setRoomNum(Integer.valueOf(validateRq.getRoomNum()));
        Map<String, Integer> stringIntegerMap = calculatePrice(validateRq.getDailyInfos(), Integer.valueOf(validateRq.getRoomNum()));
        checkRequest.setTotalPrice(stringIntegerMap.get("totalPrice"));
        // 查询下单渠道
        DtOrderChannels dtOrderChannels = dtOrderChannelsService.queryByHotelId(hotelId);
        if(dtOrderChannels == null) {
            log.warn("【下单前校验】店铺：{}，请配置采购渠道，接口调用数据：{}", validateRq.getAuthenticationToken().getUsername(), JSON.toJSON(validateRq));
            return R.error();
        }
        FxBaseApiRequest fxBaseApiRequest = new FxBaseApiRequest(dtOrderChannels.getPartnerId(),
                dtOrderChannels.getSecret(),
                dtOrderChannels.getAccessKey(),
                method,
                JSONObject.toJSONString(checkRequest), false, false);
        String mtResult = new MtClient().retryPost(fxBaseApiRequest);
        if(mtResult == null) {
            log.warn("【下单前校验】校验异常，酒店ID：{}，房型ID：{}，RP:{}，接口调用数据：{}，req：{}，resp：为null", validateRq.getHotelId(), validateRq.getRoomTypeId(),
                    validateRq.getRatePlanCode(), JSON.toJSON(validateRq), JSON.toJSON(checkRequest));
            return R.error();
        }
        JSONObject jsonObject = JSONObject.parseObject(mtResult);
        String result = jsonObject.getString("result");
        CheckResult checkResponse = JSONObject.parseObject(result, CheckResult.class);
        List<HtDailyRoom> updateDailyRoom = new ArrayList<>();
        // 校验成功
        if (checkResponse.getCode() == 0 || checkResponse.getCode() == 8) {
            List<PriceModels> priceModelsList = checkResponse.getPriceModels();
            for (PriceModels priceModels : priceModelsList) {
                HtDailyRoom htDailyRoom = new HtDailyRoom();
                htDailyRoom.setSubPrice(priceModels.getSubPrice());
                htDailyRoom.setDate(priceModels.getDate());
                htDailyRoom.setPrice(priceModels.getSalePrice());
                htDailyRoom.setRpCode(rpCode.toString());
                updateDailyRoom.add(htDailyRoom);
            }
            log.info("【下单前校验】校验成功，酒店ID：{},房型ID：{}，RP:{},接口调用：{}，req：{}，resp：{}", validateRq.getHotelId(), validateRq.getRoomTypeId(), validateRq.getRatePlanCode(),
                    JSON.toJSON(validateRq), JSON.toJSON(checkRequest), mtResult);
        } else {
            log.warn("【下单前校验】校验失败，酒店ID：{},房型ID：{}，RP:{},接口调用：{}，req：{}，resp：{}", validateRq.getHotelId(), validateRq.getRoomTypeId(), validateRq.getRatePlanCode(),
                    JSON.toJSON(validateRq), JSON.toJSON(checkRequest), mtResult);
            return R.error();
        }
        return R.ok().put("data", updateDailyRoom);
    }

    /**
     * 预约下单(美团）
     *
     * @param bookRq BookRq
     * @return R
     */
    @Override
    public R book(BookRq bookRq) {
        String method = "hotel.order.booking";
        String hotelId = bookRq.getHotelId();
        DtOrderChannels dtOrderChannels = dtOrderChannelsService.queryByHotelId(Long.valueOf(hotelId));
        if(dtOrderChannels == null) {
            log.warn("【预约下单】请配置飞猪接口用户名为{}的店铺对应的美团渠道账号", bookRq.getAuthenticationToken().getUsername());
            return R.error();
        }
        MtBook mtBook = new MtBook();
        mtBook.setHotelId(Long.valueOf(bookRq.getHotelId()));
        mtBook.setGoodsId(Long.valueOf(bookRq.getRatePlanCode()));
        StringBuilder personNames = new StringBuilder();
        for (OrderGuest guest : bookRq.getOrderGuests()) {
            personNames.append(guest.getName()).append(",");
        }
        personNames = new StringBuilder(personNames.toString().replaceAll("\\,$", ""));
        mtBook.setPersonNames(personNames.toString());
        mtBook.setContactName(bookRq.getContactName());
        mtBook.setContactPhone(bookRq.getContactTel());
        mtBook.setArriveDate(bookRq.getLatestArriveTime());
        mtBook.setCheckinDate(bookRq.getCheckIn());
        mtBook.setCheckoutDate(bookRq.getCheckOut());
        mtBook.setRoomNum(bookRq.getRoomNum());
        Map<String, Integer> priceMap = this.calculatePrice(bookRq.getDailyInfos(), bookRq.getRoomNum());
        mtBook.setTotalPrice(priceMap.get("totalPrice"));
        mtBook.setSettlePrice(priceMap.get("settlePrice"));
        mtBook.setDistributorOrderId(bookRq.getTaoBaoOrderId());
        mtBook.setNeedInvoice(0);
        FxBaseApiRequest fxBaseApiRequest = new FxBaseApiRequest(dtOrderChannels.getPartnerId(), dtOrderChannels.getSecret(), dtOrderChannels.getAccessKey(),
                method, JSONObject.toJSONString(mtBook), false, false);
        String res = new MtClient().retryPost(fxBaseApiRequest);
        if(Objects.isNull(res)) {
            log.warn("【预约下单】下单失败！所属店铺：{}，酒店id：{}，产品id：{}，飞猪订单号：{}，调用接口参数：{}，req：{}，resp：为null",
                    bookRq.getAuthenticationToken().getUsername(), bookRq.getHotelId(), bookRq.getRatePlanCode(), bookRq.getTaoBaoOrderId(), JSON.toJSON(bookRq), mtBook);
            return R.error();
        }
        MtBookResult mtBookResult = JSONObject.parseObject(res, MtBookResult.class);
        Integer code = mtBookResult.getCode();
        if(!code.equals(0)) {
            log.warn("【预约下单】下单失败！所属店铺：{}，酒店id：{}，产品id：{}，飞猪订单号：{}，调用接口参数：{}，req：{}，resp：{}",
                    bookRq.getAuthenticationToken().getUsername(), bookRq.getHotelId(), bookRq.getRatePlanCode(), bookRq.getTaoBaoOrderId(), JSON.toJSON(bookRq), mtBook, res);
            return R.error();
        }
        Result result = mtBookResult.getResult();
        if(Objects.isNull(result)) {
            log.warn("【预约下单】下单失败！所属店铺：{}，酒店id：{}，产品id：{}，飞猪订单号：{}，调用接口参数：{}，req：{}，resp：{}",
                    bookRq.getAuthenticationToken().getUsername(), bookRq.getHotelId(), bookRq.getRatePlanCode(), bookRq.getTaoBaoOrderId(), JSON.toJSON(bookRq), mtBook, res);
            return R.error();
        }
        Integer bookCode = result.getCode();
        if(bookCode.equals(0)) {
            log.info("【预约下单】下单成功！所属店铺：{}，酒店id：{}，产品id：{}，飞猪订单号：{}，调用接口参数：{}，req：{}，resp：{}",
                    bookRq.getAuthenticationToken().getUsername(), bookRq.getHotelId(), bookRq.getRatePlanCode(), bookRq.getTaoBaoOrderId(), JSON.toJSON(bookRq), mtBook, res);
            return R.ok().put("buyOrderId", mtBookResult.getResult().getMtOrderId());
        }
        log.warn("【预约下单】下单失败！所属店铺：{}，酒店id：{}，产品id：{}，飞猪订单号：{}，调用接口参数：{}，req：{}，resp：{}",
                bookRq.getAuthenticationToken().getUsername(), bookRq.getHotelId(), bookRq.getRatePlanCode(), bookRq.getTaoBaoOrderId(), JSON.toJSON(bookRq), mtBook, res);
        return R.error();
    }

    /**
     * 取消订单
     *
     * @param cancelRq CancelRq
     * @return R
     */
    @Override
    public R cancel(CancelRq cancelRq) {
        Long hotelId = Long.valueOf(cancelRq.getHotelId());
        String method = "hotel.order.cancel";
        DtOrderChannels dtOrderChannelsEntity = dtOrderChannelsService.queryByHotelId(hotelId);
        if(dtOrderChannelsEntity == null) {
            log.warn("【订单取消】，请配置飞猪接口用户名为-{}-的店铺对应的美团渠道账号", cancelRq.getAuthenticationToken().getUsername());
            return R.error();
        }
        CancelRequest cancelRequest = new CancelRequest();
        cancelRequest.setDistributorOrderId(cancelRq.getTaoBaoOrderId());
        cancelRequest.setMtOrderId(Long.valueOf(cancelRq.getBuyOrderCode()));
        cancelRequest.setCancelReason("正常取消");
        cancelRequest.setCancelCheck(0);
        FxBaseApiRequest fxBaseApiRequest = new FxBaseApiRequest(dtOrderChannelsEntity.getPartnerId(),
                dtOrderChannelsEntity.getSecret(),
                dtOrderChannelsEntity.getAccessKey(),
                method,
                JSONObject.toJSONString(cancelRequest),
                false, false);
        String cancelResult = new MtClient().retryPost(fxBaseApiRequest);
        if(cancelResult == null) {
            log.warn("【订单取消】取消失败！，所属店铺：{}，飞猪订单号：{}，美团订单号：{}，接口调用参数：{}，req：{}，resp：为null",
                    cancelRq.getAuthenticationToken().getUsername(), cancelRq.getTaoBaoOrderId(), cancelRq.getBuyOrderCode(), JSON.toJSON(cancelRq), JSON.toJSON(cancelRequest));
            return R.error();
        }
        CancelResponse cancelResponse = JSONObject.parseObject(cancelResult, CancelResponse.class);
        // 成功取消
        if (cancelResponse.getCode() == 0) {
            log.info("【订单取消】取消成功！，所属店铺：{}，飞猪订单号：{}，美团订单号：{}，接口调用参数：{}，req：{}，resp：{}",
                    cancelRq.getAuthenticationToken().getUsername(), cancelRq.getTaoBaoOrderId(), cancelRq.getBuyOrderCode(), JSON.toJSON(cancelRq), JSON.toJSON(cancelRequest), cancelResult);
            return R.ok();
        }
        // 订单取消中
        if (cancelResponse.getCode() == 5) {
            log.info("【订单取消】取消中！，所属店铺：{}，飞猪订单号：{}，美团订单号：{}，接口调用参数：{}，req：{}，resp：{}",
                    cancelRq.getAuthenticationToken().getUsername(), cancelRq.getTaoBaoOrderId(), cancelRq.getBuyOrderCode(), JSON.toJSON(cancelRq), JSON.toJSON(cancelRequest), cancelResult);
            return R.ok(1, "订单取消中");
        }
        // 系统繁忙
        if (cancelResponse.getCode() == 1) {
            log.info("【订单取消】系统繁忙！，所属店铺：{}，飞猪订单号：{}，美团订单号：{}，接口调用参数：{}，req：{}，resp：{}",
                    cancelRq.getAuthenticationToken().getUsername(), cancelRq.getTaoBaoOrderId(), cancelRq.getBuyOrderCode(), JSON.toJSON(cancelRq), JSON.toJSON(cancelRequest), cancelResult);
            return R.error(2, "系统繁忙");
        }
        // 其他情况失败
        log.warn("【订单取消】取消失败！，所属店铺：{}，飞猪订单号：{}，美团订单号：{}，接口调用参数：{}，req：{}，resp：{}",
                cancelRq.getAuthenticationToken().getUsername(), cancelRq.getTaoBaoOrderId(), cancelRq.getBuyOrderCode(), JSON.toJSON(cancelRq), JSON.toJSON(cancelRequest), cancelResult);
        return R.error();
    }

    /**
     * 订单状态变更通知
     * //21 预订成功 book_suc
     * //22 预订失败 book_fail
     * //31 取消成功 cancel_suc
     * //50 已入住 book_checkin
     * //40 美团客服介入退款 abort
     *
     * @param callbackOrderData JSONObject
     * @return R
     */
    @Override
    public void callback(JSONObject callbackOrderData) {
        JSONObject dataObject = callbackOrderData.getJSONObject("data");
        if (dataObject == null) {
            log.warn("【订单状态变更通知】通知信息中订单号不存在，msg：{}", callbackOrderData);
            return;
        }
        String fzOrderId =  dataObject.getString("distributorOrderId");
        String mtOrderId = dataObject.getString("mtOrderId");
        JSONObject json = new JSONObject();
        json.put("buyOrderId", mtOrderId);
        Constant.MtOrderStatus mtOrderStatus = Constant.MtOrderStatus.getStatusByValue(dataObject.getInteger("orderStatus"));
        switch (mtOrderStatus) {
            case book_suc:
                log.info("【订单状态变更通知】预订成功！美团订单号：{}，飞猪订单号：{}，msg：{}", mtOrderId, fzOrderId, callbackOrderData);
                json.put("code", 0);
                break;
            case book_fail:
                log.info("【订单状态变更通知】预订失败！美团订单号：{}，飞猪订单号：{}，msg：{}", mtOrderId, fzOrderId, callbackOrderData);
                json.put("code", -1);
                break;
            case cancel_suc:
                log.info("【订单状态变更通知】取消成功！美团订单号：{}，飞猪订单号：{}，msg：{}", mtOrderId, fzOrderId, callbackOrderData);
                json.put("code", 1);
                break;
            case abort:
                log.info("【订单状态变更通知】已入住！美团订单号：{}，飞猪订单号：{}，msg：{}", mtOrderId, fzOrderId, callbackOrderData);
                json.put("code", 2);
                break;
            case book_checkin:
                log.info("【订单状态变更通知】美团客服介入退款！美团订单号：{}，飞猪订单号：{}，msg：{}", mtOrderId, fzOrderId, callbackOrderData);
                json.put("code", 3);
                break;
            default:
                log.info("【订单状态变更通知】未知的订单状态！美团订单号：{}，飞猪订单号：{}，msg：{}", mtOrderId, fzOrderId, callbackOrderData);
                return;
        }
        aliRemoteService.callback(json);
    }

    /**
     * 计算价格
     *
     * @param dailyInfos List<DailyInfo>
     * @param roomNum Integer
     * @return Map<String, Integer>
     */
    private Map<String, Integer> calculatePrice(List<DailyInfo> dailyInfos, Integer roomNum) {
        // 一间房总价
        int totalPriceOne = 0;
        // 佣金
        int subPrice = 0;
        Map<String, Integer> map = new HashMap<>(2);
        for (DailyInfo d : dailyInfos) {
            totalPriceOne += d.getPrice();
            subPrice += d.getSubPrice();
        }
        // 预定所有房间 所有间夜的总价
        int totalPrice = totalPriceOne * roomNum;
        // 预定所有房间 所有间夜下单价
        int settlePrice = (totalPriceOne - subPrice) * roomNum;
        map.put("totalPrice", totalPrice);
        map.put("settlePrice", settlePrice);
        return map;
    }

}
