package com.qls.web.modules.ht.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qls.core.constant.Constant;
import com.qls.core.utils.R;
import com.qls.redis.RedisUtil;
import com.qls.web.common.utils.PageUtils;
import com.qls.web.common.utils.Query;
import com.qls.web.modules.ht.entity.HotSaleHotel;
import com.qls.web.modules.sys.controller.AbstractController;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 热销酒店
 *
 * @author zhouyu
 */
@RestController
@RequestMapping("ht/hotSaleHotel")
@Slf4j
public class HtHotSaleHotelController extends AbstractController {
    @Autowired
    private RedisUtil redisUtil;

    /**
     * 热销列表
     *
     * @param params Map<String, Object>
     * @return R
     */
    @GetMapping(value = "/list")
    @RequiresPermissions("ord:order:list")
    public R list(@RequestParam Map<String, Object> params) {
        IPage<JSONObject> page = new Query<JSONObject>().getPage(params);
        List<JSONObject> list = new ArrayList<>(redisUtil.getHotSaleHotel(Constant.Redis.HOT_SALE_HOTEL.getValue()));
        String hotelId = Convert.toStr(params.get("hotelId"));
        List<JSONObject> newList = new ArrayList<>();
        if (hotelId.isEmpty()) {
            newList = list;
        } else {
            for (JSONObject hotSaleHotel : list) {
                if (hotSaleHotel.getStr("hotelCode").indexOf(hotelId) > -1) {
                    newList.add(hotSaleHotel);
                }
            }
        }
        List<List<JSONObject>> backData = CollUtil.split(newList, Convert.toInt(page.getSize()));
        page.setTotal(newList.size());
        PageUtils pageUtils = new PageUtils(page, backData.get(Convert.toInt(page.getCurrent()) - 1));
        return R.ok().put("page", pageUtils).put("me", getUserId());
    }

    /**
     * 热销酒店导入
     *
     * @param params Map<String, Object>
     * @return R
     */
    @PostMapping(value = "/addHotSaleHotel")
    @RequiresPermissions("ord:order:list")
    public R upload(@RequestBody Map<String, Object> params) {
        JSONArray hotels = new JSONArray(params.get("hotels"));
        for (Object i : hotels) {
            JSONObject hotel = new JSONObject(i);
            redisUtil.HotSaleHotelUpdate(Constant.Redis.HOT_SALE_HOTEL.getValue(), hotel, 1);
        }
        return R.ok();
    }

    /**
     * 热销酒店删除
     *
     * @param ids Long[]
     * @return R
     */
    @RequestMapping(value = "/delete")
    @RequiresPermissions("ord:order:list")
    public R delete(@RequestBody String[] ids) {
        for (String id : ids) {
            JSONObject hotel = new JSONObject();
            hotel.put("hotelCode", id);
            redisUtil.HotSaleHotelUpdate(Constant.Redis.HOT_SALE_HOTEL.getValue(), hotel, 0);
        }
        return R.ok();
    }

}
