package com.qls.web.modules.ht.controller;

import com.qls.ali.api.AliRemoteService;
import com.qls.core.entity.HtAliHotel;
import com.qls.core.entity.HtHotel;
import com.qls.core.utils.R;
import com.qls.mt.api.MtRemoteService;
import com.qls.web.common.utils.PageUtils;
import com.qls.web.modules.ht.service.HtAliHotelService;
import com.qls.web.modules.ht.service.HtHotelinfoService;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @author doyoung
 */
@RestController
@RequestMapping("ht/hthotelinfo")
public class HtHotelinfoController {

    @Autowired
    private MtRemoteService mtRemoteService;

    @Autowired
    private AliRemoteService aliRemoteService;

    @Autowired
    private HtHotelinfoService htHotelinfoService;

    @Autowired
    private HtAliHotelService htAliHotelService;

    /**
     * 列表
     *
     * @return R
     */
    @RequestMapping("/list")
    @RequiresPermissions("ht:hthotelinfo:list")
    public R list(@RequestParam Map<String, Object> params) {
        PageUtils page = htHotelinfoService.sqlQueryPage(params);
        return R.ok().put("page", page);
    }


    /**
     * 信息
     *
     * @return R
     */
    @RequestMapping("/info/{id}")
    @RequiresPermissions("ht:hthotelinfo:info")
    public R info(@PathVariable("id") Long id) {
        HtHotel htHotelinfo = htHotelinfoService.getById(id);
        return R.ok().put("htHotelinfo", htHotelinfo);
    }

    /**
     * 保存
     *
     * @return R
     */
    @RequestMapping("/save")
    @RequiresPermissions("ht:hthotelinfo:save")
    public R save(@RequestBody HtHotel htHotelinfo) {
        htHotelinfoService.save(htHotelinfo);
        return R.ok();
    }

    /**
     * 修改
     *
     * @return R
     */
    @RequestMapping("/update")
    @RequiresPermissions("ht:hthotelinfo:update")
    public R update(@RequestBody HtHotel htHotelinfo) {
        htHotelinfoService.updateById(htHotelinfo);
        return R.ok();
    }

    /**
     * 删除
     *
     * @return R
     */
    @RequestMapping("/delete")
    @RequiresPermissions("ht:hthotelinfo:delete")
    public R delete(@RequestBody Long[] ids) {
        htHotelinfoService.removeByIds(Arrays.asList(ids));
        return R.ok();
    }


    /**
     * 查询酒店匹配信息
     *
     * @return R
     */
    @RequestMapping("/matchInfo/{id}")
    public R matchInfo(@PathVariable("id") Long id) {
        HtAliHotel htAliHotel = htAliHotelService.getById(id);
        return R.ok().put("htAliHotel", htAliHotel);
    }


    /**
     * 获取人工匹配阿里标准酒店列表
     *
     * @return R
     */
    @RequestMapping("/getMatchHotels")
    public R getMatchHotels(@RequestParam Map<String, Object> params) {
        return aliRemoteService.getMatchHotels(params);
    }

    /**
     * 人工匹配酒店
     *
     * @return R
     */
    @RequestMapping("/manualMatchHotel")
    public R manualMatchHotel(@RequestParam Map<String, Object> params) {
        aliRemoteService.manualMatchHotel(params);
        return R.ok();
    }

    /**
     * 获取人工匹配房型列表
     *
     * @return R
     */
    @RequestMapping("/getMatchRooms")
    public R getMatchRooms(@RequestParam Map<String, Object> params) {
        return aliRemoteService.getMatchRooms(params);
    }

    /**
     * 人工匹配房型
     *
     * @return R
     */
    @RequestMapping("/manualMatchRoom")
    public R manualMatchRoom(@RequestParam Map<String, Object> params) {
        aliRemoteService.manualMatchRoom(params);
        return R.ok();
    }
}
