package com.qls.web.modules.ht.service;


import com.alibaba.fastjson.annotation.JSONField;
import lombok.Getter;
import lombok.Setter;

import java.util.Random;
import java.util.UUID;

/**
 * 请求基类
 *
 * @author doyoung
 */

public class FxBaseApiRequest {

    private static final String V1_URL = "https://fenxiao.meituan.com/opdtor/api";

    private static final String V2_URL = "https://fenxiao.meituan.com/opdtor/api/v2";

    private static final Random RANDOM = new Random();

    /**
     * 公钥
     */
    @Getter
    private String accesskey;

    /**
     * 请求参数
     */
    @Getter
    private String data;

    /**
     * 调用的接口标识
     */
    @Getter
    private String method;

    /**
     * 随机正整数
     */
    @Setter
    @Getter
    private Integer nonce;

    /**
     * 分销商的id
     */
    @Getter
    private Integer partnerId;

    /**
     * 系统时间戳
     */
    @Setter
    @Getter
    private Long timestamp;

    /**
     * 版本号
     */
    @Getter
    private String version;

    /**
     * 签名
     */
    @Setter
    @Getter
    private String signature;

    /**
     * 测试专用字段
     */
    @Getter
    private String test;

    /**
     * 密钥 不进行序列化
     */
    @JSONField(serialize = false)
    @Getter
    private String secretKey;

    /**
     * v2 版本字段
     */
    @Getter
    private String customerSessionId;

    /**
     * v2 版本字段
     */
    @Getter
    private String language;

    /**
     * url 1.0 和 2.0 版本url不同
     */
    @JSONField(serialize = false)
    @Getter
    private String url;

    public FxBaseApiRequest(Integer partnerId, String secretKey, String accesskey, String method, String data, boolean v2, boolean test) {
        this.partnerId = partnerId;
        this.accesskey = accesskey;
        this.secretKey = secretKey;
        this.method = method;
        this.data = data;
        this.timestamp = System.currentTimeMillis() / 1000;
        this.nonce = RANDOM.nextInt(Integer.MAX_VALUE);
        if(v2) {
            this.language = "zh_CN";
            this.url = V2_URL;
            this.version = "2.0";
            this.customerSessionId = UUID.randomUUID().toString().replaceAll("-", "");
        } else {
            this.version = "1.0";
            this.url = V1_URL;
        }
        if(test) {
            this.test = "test";
        }
    }

    @Override
    public String toString() {
        return "FxBaseApiRequest={" +
                "accesskey:'" + accesskey + '\'' +
                ", data:'" + data + '\'' +
                ", method:'" + method + '\'' +
                ", nonce:" + nonce +
                ", partnerId:" + partnerId +
                ", timestamp:" + timestamp +
                ", version:'" + version + '\'' +
                ", signature:'" + signature + '\'' +
                ", test:'" + test + '\'' +
                ", secretKey:'" + secretKey + '\'' +
                ", customerSessionId:'" + customerSessionId + '\'' +
                ", language:'" + language + '\'' +
                ", url:'" + url + '\'' +
                '}';
    }
}

