package com.qls.web.modules.ht.service;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.binary.StringUtils;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.security.GeneralSecurityException;

/**
 * @author doyoung
 */
@Slf4j
public class MtClient {


    /**
     * 代签名字符串拼接规则 和签名加密
     *
     * @param request FxBaseApiRequest
     * @return String
     */
    private String calculateSignature(FxBaseApiRequest request) {
        StringBuilder sb = new StringBuilder(18);
        sb.append("accesskey=").append(request.getAccesskey());
        if(request.getCustomerSessionId() != null) {
            sb.append("&customerSessionId=").append(request.getCustomerSessionId());
        }
        sb.append("&data=").append(request.getData());
        if(request.getLanguage() != null) {
            sb.append("&language=").append(request.getLanguage());
        }
        sb.append("&method=").append(request.getMethod());

        sb.append("&nonce=").append(request.getNonce());
        sb.append("&partnerId=").append(request.getPartnerId());
        if(request.getTest() != null) {
            sb.append("&test=").append(request.getTest());
        }
        sb.append("&timestamp=").append(request.getTimestamp());
        sb.append("&version=").append(request.getVersion());
        return this.hmacSha1(sb.toString(), request.getSecretKey());
    }

    /**
     * 签名的加密算法
     *
     * @param plainText String
     * @param secretKey String
     * @return String
     */
    private String hmacSha1(String plainText, String secretKey) {
        SecretKeySpec secretKeySpec = new SecretKeySpec(StringUtils.getBytesUtf8(secretKey), "HmacSHA1");
        try {
            Mac mac = Mac.getInstance("HmacSHA1");
            mac.init(secretKeySpec);
            return Base64.encodeBase64String(mac.doFinal(StringUtils.getBytesUtf8(plainText)));
        } catch (GeneralSecurityException var5) {
            throw new IllegalArgumentException(var5);
        }
    }

    /**
     * 接口调用
     *
     * @param request FxBaseApiRequest
     * @return String
     */
    public String post(FxBaseApiRequest request) {
        request.setSignature(calculateSignature(request));
        String jsonString = JSON.toJSONString(request);
        try {
            return HttpRequest.post(request.getUrl())
                    .header("Content-Type", "application/json; charset=utf-8")
                    .header("Accept", "application/json")
                    .body(jsonString)
                    .timeout(20000)
                    .execute()
                    .body();
        } catch (Exception e) {
            log.error("接口调用异常，method：{}，分销商编码：{}，异常原因：{}", request.getMethod(), request.getPartnerId(), e.getMessage());
            return null;
        }

    }

    /**
     * 重试 破石头
     *
     * @param request
     * @param max 最大重试次数
     * @return
     */
    public String retryPost(FxBaseApiRequest request, int max) {
        if(max < 0) {
            max = 0;
        }
        String result = null;
        while (result == null || max < 0) {
            result = post(request);
            max --;
        }
        return result;
    }

    /**
     * 重试 破石头
     *
     * @param request
     * @return
     */
    public String retryPost(FxBaseApiRequest request) {
        return retryPost(request, 3);
    }
}
