package com.qls.web.modules.job.task;

import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONObject;
import com.qls.core.constant.Constant;
import com.qls.rabbitmq.SendMsg;
import com.qls.redis.RedisUtil;
import com.qls.web.modules.op.service.HtlGroupService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * 定时任务-热销酒店主动更新
 * @author zhouyu
 */
@Slf4j
@Component("hotSaleHotel")
public class HotSaleHotelTask implements ITask {

    @Autowired
    private SendMsg sendMsg;

    @Autowired
    private RedisUtil redisUtil;

    @Override
    public void run(String params) {
        log.info("热销酒店主动更新定时任务开始");
        List<JSONObject> list = new ArrayList<>(redisUtil.getHotSaleHotel(Constant.Redis.HOT_SALE_HOTEL.getValue()));
        // 查询数分组推送过的所有酒店
        List<Long> hotelIds = new ArrayList<>();
        for (JSONObject hotel : list) {
            hotelIds.add(Convert.toLong(hotel.get("hotelCode")));
        }
        if(hotelIds.isEmpty()) {
            return;
        }
        hotelIds.forEach(hotelId-> sendMsg.execute(Constant.RabbitMq.UPDATE_RP, null, hotelId));
        log.info("热销酒店主动更新定时任务结束");
    }
}
