package com.qls.web.modules.op.controller;

import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONObject;
import com.qls.core.constant.Constant;
import com.qls.core.dto.PushHotelBody;
import com.qls.core.dto.PushRpBody;
import com.qls.core.entity.DtStore;
import com.qls.core.entity.HtHotel;
import com.qls.core.entity.OpTaskStatus;
import com.qls.core.utils.R;
import com.qls.mt.api.MtRemoteService;
import com.qls.rabbitmq.SendMsg;
import com.qls.redis.RedisUtil;
import com.qls.web.common.utils.PageUtils;
import com.qls.web.modules.dt.service.StoreService;
import com.qls.web.modules.ht.service.HtLogicRoomService;
import com.qls.web.modules.op.service.HtlGroupService;
import com.qls.web.modules.op.service.TaskStatusService;
import com.qls.web.modules.sys.controller.AbstractController;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.web.bind.annotation.*;

import java.util.*;


/**
 * 任务状态表（前端与服务交互）
 *
 * @author yj
 * @email yj@gmail.com
 * @date 2021-08-26 17:34:02
 */
@RestController
@RequestMapping("op/taskstatus")
public class TaskStatusController extends AbstractController {

    @Autowired
    private MtRemoteService mtRemoteService;

    @Autowired
    private TaskStatusService taskStatusService;

    @Autowired
    private SendMsg sendMsg;

    @Autowired
    private StoreService storeService;

    @Autowired
    private HtLogicRoomService htLogicRoomService;

    @Autowired
    private RedisUtil redisUtil;

    /**
     * 酒店上下架
     *
     * @return R
     */
    @PostMapping("updown")
    public R upDown(@RequestBody Map<String, Object> map) {
        this.execute(map);
        return R.ok();
    }

    /**
     * 下架异步方法
     *
     * @param map
     */
    @Async
    void execute(Map<String, Object> map) {
        Integer status = Integer.valueOf(map.get("status").toString());
        int hotelStatus;
        if (status.equals(1)) {
            hotelStatus = 0;
        } else {
            hotelStatus = -2;
        }
        List<Integer> hotelIds = (List<Integer>) map.get("hotelIds");
        for (Integer hotelId : hotelIds) {
            redisUtil.upOrDown(Constant.Redis.DOWN_HOTEL.getValue(), Convert.toStr(hotelId), hotelStatus);
        }
        com.alibaba.fastjson.JSONObject hotelChange = new com.alibaba.fastjson.JSONObject();
        hotelChange.put("hotelStatus",hotelStatus);
        hotelChange.put("hotelIds",hotelIds);
        sendMsg.execute(Constant.RabbitMq.CALLBACK_HOTEL, null, hotelChange);
    }

    /**
     * 列表
     */
    @RequestMapping("/list")
    @RequiresPermissions("op:taskstatus:list")
    public R list(@RequestParam Map<String, Object> params) {
        params.put("dtId", getUserDtId());
        PageUtils page = taskStatusService.queryPage(params);
        return R.ok().put("page", page);
    }


    /**
     * 信息
     */
    @RequestMapping("/info/{id}")
    @RequiresPermissions("op:taskstatus:info")
    public R info(@PathVariable("id") Integer id) {
        OpTaskStatus taskStatus = taskStatusService.getById(id);

        return R.ok().put("taskStatus", taskStatus);
    }

    /**
     * 保存
     */
    @RequestMapping("/save")
    @RequiresPermissions("op:taskstatus:save")
    public R save(@RequestBody OpTaskStatus taskStatus) {
        taskStatusService.save(taskStatus);

        return R.ok();
    }

    /**
     * 修改
     */
    @RequestMapping("/update")
    @RequiresPermissions("op:taskstatus:update")
    public R update(@RequestBody OpTaskStatus taskStatus) {
        taskStatusService.updateById(taskStatus);

        return R.ok();
    }

    /**
     * 删除所有店铺下 名字包含 xx的rp
     *
     * @return
     */
    @GetMapping(value = "del_rp_by_name")
    public R delRpByName(@RequestParam Map<String, Object> params) {
        List<DtStore> stores = storeService.getStoreIds();
        if (stores.isEmpty()) {
            return R.ok();
        }
        Object nameObj = params.get("name");
        if (nameObj == null) {
            return R.error("请填入rp名称");
        }
        String name = nameObj.toString();
        if ("".equals(name)) {
            return R.error("rp名称不能未空字符串");
        }
        List<Long> rpCodes = htLogicRoomService.getRpCodesByName(name);
        // 结束标志
        rpCodes.add(-1L);
        // 生成任务
        stores.forEach(store -> {
            OpTaskStatus opTaskStatus = new OpTaskStatus();
            Integer taskCode = Integer.valueOf(System.currentTimeMillis() / 1000 + "");
            opTaskStatus.setTaskCode(taskCode);
            opTaskStatus.setTaskName(String.format("店铺[%s]删除rp名字包含[%s]", store.getName(), name));
            opTaskStatus.setStartTime(new Date());
            opTaskStatus.setTotal(rpCodes.size() - 1);
            opTaskStatus.setDtId(getUserDtId());
            taskStatusService.save(opTaskStatus);
            PushRpBody pushRpBody = new PushRpBody();
            for (Long rpCode : rpCodes) {
                pushRpBody.setCrpIds(null);
                pushRpBody.setRpCode(rpCode);
                pushRpBody.setStoreId(store.getId());
                pushRpBody.setTaskId(taskCode);
                pushRpBody.setType(-1);
                sendMsg.execute(Constant.RabbitMq.RP, store.getId(), pushRpBody);
            }
        });
        return R.ok();
    }


    /**
     * 删除
     */
    @RequestMapping("/delete")
    @RequiresPermissions("op:taskstatus:delete")
    public R delete(@RequestBody Integer[] ids) {
        taskStatusService.removeByIds(Arrays.asList(ids));

        return R.ok();
    }

    /**
     * 酒店初始化
     *
     * @return R
     */
    @RequestMapping("/init")
    @RequiresPermissions("op:taskstatus:init")
    public R init() {
        Map<String, Object> map = new HashMap<>();
        map.put("dtId", getUserDtId());
        mtRemoteService.pullAll(map);
        return R.ok();
    }

}
